@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -97,6 +97,7 @@ typedef int32_t tAppId;
 #define APPID_SESSION_TPI_OOO_LOGGED        (1ULL << 42)
 
 #define APPID_SESSION_EXPECTED_EVALUATE     (1ULL << 43)
+#define APPID_SESSION_HOST_CACHE_MATCHED    (1ULL << 44)
 
 #define APPID_SESSION_IGNORE_ID_FLAGS       (APPID_SESSION_IGNORE_FLOW | \
                                              APPID_SESSION_NOT_A_SERVICE | \
@@ -247,7 +248,7 @@ struct AppIdApi
     void (*freeSmbFpData)(struct AppIdData *session, FpSMBData *data);
     char* (*getNetbiosName)(struct AppIdData *session);
     uint32_t (*produceHAState)(void *lwssn, uint8_t *buf);
-    uint32_t (*consumeHAState)(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, struct in6_addr* ip, uint16_t initiatorPort); 
+    uint32_t (*consumeHAState)(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, const struct in6_addr* ip, uint16_t initiatorPort); 
     struct AppIdData * (*getAppIdData)(void *lwssn);
 
     char* (*getDNSQuery)(struct AppIdData *appIdData, uint8_t *query_len);
@@ -259,11 +260,15 @@ struct AppIdApi
     void (*freeHttpNewField)(struct AppIdData *appIdData, HTTP_FIELD_ID fieldId);
     uint16_t (*getHttpFieldOffset)(struct AppIdData *session, HTTP_FIELD_ID fieldId);
     uint16_t (*getHttpFieldEndOffset)(struct AppIdData *session, HTTP_FIELD_ID fieldId);
+    bool (*isHttpInspectionDone)(struct AppIdData *session);
+    void (*dumpDebugHostInfo)(void);
 };
 
 /* For access when including header */
 extern struct AppIdApi appIdApi;
 
+//#define UNIT_TESTING // NOTE These testing #define's are used in service_base.c and fw_appid.c
+//#define UNIT_TEST_FIRST_DECRYPTED_PACKET 12 // WARNING this assumes a single stream in a decrypted pcap
 
 #endif  /* __APPID_API_H__ */
 
