@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2002-2013 Sourcefire, Inc.
 ** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
 **
@@ -691,6 +691,7 @@ typedef enum {
 #define PKT_ERR_CKSUM_IGMP   0x10
 #define PKT_ERR_CKSUM_ANY    0x1F
 #define PKT_ERR_BAD_TTL      0x20
+#define PKT_ERR_SYN_RL_DROP  0x40
 
 /*  D A T A  S T R U C T U R E S  *********************************************/
 typedef int (*LogFunction)(void *ssnptr, uint8_t **buf, uint32_t *len, uint32_t *type);
@@ -1602,6 +1603,9 @@ typedef struct _PPPoE_Tag
 
 #define MPLS_HEADER_LEN    4
 #define NUM_RESERVED_LABELS    16
+#ifdef MPLS_RFC4023_SUPPORT
+#define IPPROTO_MPLS    137
+#endif
 
 typedef struct _MplsHdr
 {
