@@ -1,7 +1,7 @@
 /* $Id$ */
 /*
 ** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2002-2013 Sourcefire, Inc.
 **    Dan Roelker <droelker@sourcefire.com>
 **    Marc Norton <mnorton@sourcefire.com>
@@ -169,7 +169,18 @@ static void DispatchPreprocessors( Packet *p, tSfPolicyId policy_id, SnortPolicy
 int Preprocess(Packet * p)
 {
     int retval = 0;
-    tSfPolicyId policy_id = getNapRuntimePolicy();
+    tSfPolicyId policy_id;
+
+   /* NAP runtime policy may have been updated during decode, 
+    * but preprocess needs default nap policy for session
+    * preprocessor selection, hence use default policy when
+    * called without session pointer set.
+    */
+    if( !p->ssnptr )
+        policy_id = getDefaultPolicy();
+    else
+        policy_id = getNapRuntimePolicy();
+
     SnortPolicy *policy = snort_conf->targeted_policies[policy_id];
 #ifdef PPM_MGR
     uint64_t pktcnt=0;
@@ -200,7 +211,7 @@ int Preprocess(Packet * p)
     }
 #endif
 
-    // If the packet has errors, we won't analyze it.
+    // If the packet has errors or syn over rate, we won't analyze it.
     if ( p->error_flags )
     {
         // process any decoder alerts now that policy has been selected...
@@ -213,6 +224,8 @@ int Preprocess(Packet * p)
 
         if ( p->error_flags & PKT_ERR_BAD_TTL )
             pc.bad_ttl++;
+        else if( p->error_flags & PKT_ERR_SYN_RL_DROP )
+            pc.syn_rate_limit_drops++;
         else
             pc.invalid_checksums++;
     }
@@ -385,10 +398,17 @@ static int CheckTagging(Packet *p)
 static void updateEventAppName (Packet *p, OptTreeNode *otn, Event *event)
 {
     const char *appName;
+    size_t appNameLen;
     AppIdOptionData *app_data = (AppIdOptionData*)otn->ds_list[PLUGIN_APPID];
 
     if (app_data && (app_data->matched_appid) && (appName = appIdApi.getApplicationName(app_data->matched_appid)))
-        memcpy(event->app_name, appName, sizeof(event->app_name));
+    {
+        appNameLen = strlen(appName);
+        if (appNameLen >= sizeof(event->app_name))
+            appNameLen = sizeof(event->app_name) - 1;
+        memcpy(event->app_name, appName, appNameLen);
+        event->app_name[appNameLen] = '\0';
+    }
     else if (p->ssnptr)
     {
         //log most specific appid when rule didn't have any appId
@@ -408,7 +428,11 @@ static void updateEventAppName (Packet *p, OptTreeNode *otn, Event *event)
 
         if ((pickedProtoId) && (appName = appIdApi.getApplicationName(pickedProtoId)))
         {
-            memcpy(event->app_name, appName, sizeof(event->app_name));
+            appNameLen = strlen(appName);
+            if (appNameLen >= sizeof(event->app_name))
+                appNameLen = sizeof(event->app_name) - 1;
+            memcpy(event->app_name, appName, appNameLen);
+            event->app_name[appNameLen] = '\0';
         }
         else
         {
@@ -543,7 +567,12 @@ int Detect(Packet * p)
     {
         return 0;
     }
-
+    
+    if (stream_api && stream_api->is_session_http2(p->ssnptr) 
+        && !(p->packet_flags & PKT_REBUILT_STREAM)) 
+    {
+        return 0;
+    }
 
     if (!snort_conf->ip_proto_array[GET_IPH_PROTO(p)])
     {
@@ -1280,6 +1309,10 @@ int DropAction(Packet * p, OptTreeNode * otn, RuleTreeNode * rtn, Event * event)
                 "\"%s\"\n", otn->sigInfo.message););
 
             Active_DropSession(p);
+            if (pkt_trace_enabled)
+                addPktTraceData(VERDICT_REASON_SNORT, snprintf(trace_line, MAX_TRACE_LINE,
+                    "Snort: gid %u, sid %u, midstream %s\n", otn->sigInfo.generator, otn->sigInfo.id, getPktTraceActMsg()));
+            else addPktTraceData(VERDICT_REASON_SNORT, 0);
             return 1;
         }
     }
@@ -1297,6 +1330,10 @@ int DropAction(Packet * p, OptTreeNode * otn, RuleTreeNode * rtn, Event * event)
 
     CallLogFuncs(p, otn->sigInfo.message, rtn->listhead, event);
 
+    if (pkt_trace_enabled)
+        addPktTraceData(VERDICT_REASON_SNORT, snprintf(trace_line, MAX_TRACE_LINE,
+            "Snort detect_drop: gid %u, sid %u, %s\n", otn->sigInfo.generator, otn->sigInfo.id, getPktTraceActMsg()));
+    else addPktTraceData(VERDICT_REASON_SNORT, 0);
     return 1;
 }
 
@@ -1308,7 +1345,10 @@ int SDropAction(Packet * p, OptTreeNode * otn, Event * event)
 
     // Let's silently drop the packet
     Active_DropSession(p);
-
+    if (pkt_trace_enabled)
+        addPktTraceData(VERDICT_REASON_SNORT, snprintf(trace_line, MAX_TRACE_LINE,
+            "Snort detect_sdrop: gid %u, sid %u, %s\n", otn->sigInfo.generator, otn->sigInfo.id, getPktTraceActMsg()));
+    else addPktTraceData(VERDICT_REASON_SNORT, 0);
     return 1;
 }
 
