@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2003-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -364,7 +364,7 @@ static char* ByteMath_tok_extract(char *src,char *del)
 {
         char *ret_tok=NULL;
         ret_tok = strtok(src," ");
-        if (!strcmp(ret_tok,del))
+        if (ret_tok && !strcmp(ret_tok,del))
         {
            ret_tok = strtok(NULL, ",");
            if (ret_tok)
