@@ -1,7 +1,7 @@
 /* $Id$ */
 /****************************************************************************
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -447,6 +447,16 @@ static int React_Queue (Packet* p, void* pv)
         Active_QueueResponse(React_Send, rd);
 
     Active_DropSession(p);
+    if (pkt_trace_enabled)
+    {
+        if (rd && rd->otn)
+            addPktTraceData(VERDICT_REASON_REACT, snprintf(trace_line, MAX_TRACE_LINE,
+                "Snort React: web page %s, gid %u, sid %u, %s\n", Active_IsRSTCandidate(p)? "is sent" : "isn't sent",
+                rd->otn->sigInfo.generator, rd->otn->sigInfo.id, getPktTraceActMsg()));
+        else addPktTraceData(VERDICT_REASON_REACT, snprintf(trace_line, MAX_TRACE_LINE,
+                "Snort React: web page %s, %s\n", Active_IsRSTCandidate(p)? "is sent" : "isn't sent", getPktTraceActMsg()));
+    }
+    else addPktTraceData(VERDICT_REASON_REACT, 0);
 
     PREPROC_PROFILE_END(reactPerfStats);
     return 0;
