@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * Author: Steven Sturges
@@ -95,6 +95,13 @@ typedef HANDLE PluginHandle;
 #include "file_api.h"
 #include "packet_time.h"
 #include "perf_indicators.h"
+#include "reload.h"
+#include "so_rule_mem_adjust.h"
+
+#ifdef SNORT_RELOAD
+#include "appdata_adjuster.h"
+#endif
+
 #if defined(FEAT_OPEN_APPID)
 #include "appIdApi.h"
 #endif /* defined(FEAT_OPEN_APPID) */
@@ -114,6 +121,10 @@ char *no_file = "unknown";
 int no_line = 0;
 #endif
 
+#ifdef SNORT_RELOAD
+static APPDATA_ADJUSTER *ada;
+#endif
+
 /* Predeclare this */
 void VerifySharedLibUniqueness();
 typedef int (*LoadLibraryFunc)(const char * const path, int indent);
@@ -589,6 +600,10 @@ void CloseDynamicEngineLibs(void)
         plugin = tmpplugin;
     }
     loadedEngines = NULL;
+#ifdef SNORT_RELOAD
+    ada_delete(ada);
+    ada = NULL;
+#endif
 }
 
 void RemovePreprocessorPlugin(DynamicPreprocessorPlugin *plugin)
@@ -1047,6 +1062,11 @@ typedef struct _DynamicRuleSessionData
 } DynamicRuleSessionData;
 
 static uint32_t so_rule_memory = 0;
+
+static size_t SoRuleMemInUse()
+{
+    return (size_t) so_rule_memory;
+}
 /*Only only message will be logged within 60 seconds*/
 static ThrottleInfo error_throttleInfo = {0,60,0};
 
@@ -1100,6 +1120,9 @@ static void DynamicRuleDataFreeSession(void *data)
         if (tmp->data && tmp->cleanupFunc)
             tmp->cleanupFunc(tmp->data);
         DynamicRuleDataFree(tmp);
+#ifdef SNORT_RELOAD
+        ada_appdata_freed(ada, tmp);
+#endif
     }
 }
 
@@ -1153,6 +1176,9 @@ int DynamicSetRuleData(void *p, void *data, uint32_t sid, SessionDataFree sdf)
                 DynamicRuleDataFree(tmp);
                 return -1;
             }
+#ifdef SNORT_RELOAD
+            ada_add( ada, (void *)tmp, pkt->ssnptr );
+#endif
         }
         else
         {
@@ -1368,6 +1394,14 @@ int InitDynamicPreprocessorPlugins(DynamicPreprocessorData *info)
 
         plugin = plugin->next;
     }
+#ifdef SNORT_RELOAD
+    if (!ada) {
+        ada = ada_init(SoRuleMemInUse, PP_SHARED_RULES, (size_t) ScSoRuleMemcap());
+        if (!ada) {
+            FatalError("Failed to initialize so_rule session cache. \n");
+        }
+    }
+#endif
     return 0;
 }
 
@@ -1532,6 +1566,16 @@ void DynamicChangeIpsRuntimePolicy(tSfPolicyId new_id, void *p)
     pkt->configPolicyId = snort_conf->targeted_policies[new_id]->configPolicyId;
 }
 
+static void DynamicAddPktTraceData(int module, int strLen)
+{
+    addPktTraceData(module, strLen);
+}
+
+static const char* DynamicGetPktTraceActionMsg()
+{
+    return getPktTraceActMsg();
+}
+
 static void* DynamicEncodeNew (void)
 {
     return (void*)Encode_New();
@@ -2000,6 +2044,15 @@ void DynamicSetSSLPolicyEnabled(struct _SnortConfig *sc, tSfPolicyId policy, boo
     sc->targeted_policies[policy]->ssl_policy_enabled = value;
 }
 
+#ifdef SNORT_RELOAD
+int DynamicReloadAdjustRegister(SnortConfig* sc, const char* raName, tSfPolicyId raPolicyId,
+                                ReloadAdjustFunc raFunc, void* raUserData,
+                                ReloadAdjustUserFreeFunc raUserFreeFunc)
+{
+    return ReloadAdjustRegister(sc, raName, raPolicyId, raFunc, raUserData, raUserFreeFunc);
+}
+#endif
+
 #if defined(FEAT_OPEN_APPID)
 typedef struct _IsAppIdRequiredFuncNode
 {
@@ -2147,6 +2200,11 @@ static void dummyFreeIndexedStringFromAppIdData(struct AppIdData *appIdData, HTT
     return;
 }
 
+static uint16_t dummyGetHttpFieldOffset(struct AppIdData *appIdData, HTTP_FIELD_ID fieldId)
+{
+    return 0;
+}
+
 static SEARCH_SUPPORT_TYPE dummySearchTypeFromAppIdData(struct AppIdData *appIdData)
 {
     return NOT_A_SEARCH_ENGINE;
@@ -2194,7 +2252,7 @@ static uint32_t dummyProduceHAState(void *lwssn, uint8_t *buf)
     return 0;
 }
 
-static uint32_t dummyConsumeHAState(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, struct in6_addr* ip, uint16_t initiatorPort)
+static uint32_t dummyConsumeHAState(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, const struct in6_addr* ip, uint16_t initiatorPort)
 {
     return 0;
 }
@@ -2232,6 +2290,10 @@ struct AppIdData* dummyRemoveExpectedAppIdData(struct AppIdData *appIdData)
     return NULL;
 }
 
+static void dummyDumpDebugHostInfo(void)
+{
+}
+
 struct AppIdApi appIdApi = {
     dummyGetApplicationName,    /* getApplicationName */
     dummyGetApplicationId,      /* getApplicationId */
@@ -2307,12 +2369,16 @@ struct AppIdApi appIdApi = {
 
     dummyIndexedStringFromAppIdData,        /* getHttpNewField */
     dummyFreeIndexedStringFromAppIdData,    /* freeHttpNewField */
+    dummyGetHttpFieldOffset,    /* getHttpFieldOffset */
+    dummyGetHttpFieldOffset,    /* getHttpFieldEndOffset */
+    dummyCheckAppIdData,        /* isHttpInspectionDone */
+    dummyDumpDebugHostInfo      /* dumpDebugHostInfo */
 };
 #endif /* defined(FEAT_OPEN_APPID) */
 
 static int GetSnortPerfIndicators( void *p )
 {
-	    return( PerfIndicator_GetIndicators( (Perf_Indicator_Descriptor_p_t)p ) );
+    return( PerfIndicator_GetIndicators( (Perf_Indicator_Descriptor_p_t)p ) );
 }
 
 static bool DynamicIsTestMode(void)
@@ -2568,6 +2634,16 @@ int InitDynamicPreprocessors(void)
     preprocData.isTestMode = &DynamicIsTestMode;
 
     preprocData.getCurrentSnortConfig = GetCurrentSnortConfig;
+    preprocData.pkt_tracer_enabled = &pkt_trace_enabled;
+    preprocData.trace = trace_line;
+    preprocData.traceMax = MAX_TRACE_LINE;
+    preprocData.addPktTrace = DynamicAddPktTraceData;
+    preprocData.getPktTraceActionMsg = DynamicGetPktTraceActionMsg;
+
+#ifdef SNORT_RELOAD
+    preprocData.reloadAdjustRegister = DynamicReloadAdjustRegister;
+#endif
+
     return InitDynamicPreprocessorPlugins(&preprocData);
 }
 
@@ -3048,3 +3124,10 @@ void RemoveDuplicateSideChannelPlugins(void)
 }
 
 #endif /* SIDE_CHANNEL */
+
+#ifdef SNORT_RELOAD
+void AdjustSoRuleMemory(SnortConfig *new_config, SnortConfig *old_config) {
+    tSfPolicyId policy_id = getParserPolicy(new_config);
+    ada_reload_adjust_register(ada, policy_id, (void *) new_config,  "so_rule", (size_t) new_config->so_rule_memcap);
+}
+#endif
