@@ -14,7 +14,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * Author: Steven Sturges
@@ -70,6 +70,7 @@
 #endif
 #include "idle_processing.h"
 #include "file_api.h"
+#include "reload_api.h"
 
 struct _PreprocStats;
 
@@ -293,6 +294,14 @@ typedef bool (*IsTestModeFunc)(void);
 
 typedef struct _SnortConfig* (*GetCurrentSnortConfigFunc)(void);
 
+typedef void (*AddPktTraceDataFunc)(int module, int traceLen);
+
+typedef const char* (*GetPktTraceActionMsgFunc)();
+
+typedef int (*ReloadAdjustRegisterFunc)(struct _SnortConfig* sc, const char* raName,
+                                        tSfPolicyId raPolicyId, ReloadAdjustFunc raFunc,
+                                        void *raUserData, ReloadAdjustUserFreeFunc raUserFreeFunc); 
+
 #define ENC_DYN_FWD 0x80000000
 #define ENC_DYN_NET 0x10000000
 
@@ -535,6 +544,12 @@ typedef struct _DynamicPreprocessorData
     ReadModeFunc isReadMode;
     IsTestModeFunc isTestMode;
     GetCurrentSnortConfigFunc getCurrentSnortConfig;
+    bool *pkt_tracer_enabled;
+    char *trace;
+    uint32_t traceMax;
+    AddPktTraceDataFunc addPktTrace;
+    GetPktTraceActionMsgFunc getPktTraceActionMsg;
+    ReloadAdjustRegisterFunc reloadAdjustRegister;
 } DynamicPreprocessorData;
 
 /* Function prototypes for Dynamic Preprocessor Plugins */
