@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * Author: Steve Sturges
@@ -186,7 +186,7 @@ static int GetDynamicContents(void *r, int type, FPContentInfo **contents)
     int base64_buf_flag = 0;
     int mime_buf_flag = 0;
 
-    if ((r == NULL) || (contents == NULL))
+    if ((r == NULL) || (contents == NULL) || (!rule->initialized) || (rule->options == NULL))
         return -1;
 
     *contents = NULL;
@@ -1084,7 +1084,7 @@ static void FreeOneRule(void *data)
     int i;
     Rule *rule = (Rule *)data;
 
-    if (rule == NULL)
+    if (rule == NULL || (!rule->options))
         return;
 
     /* More than one rule may use the same rule option so make sure anything
