@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * Author: Steven Sturges
@@ -469,7 +469,7 @@ int ReloadDynamicRules(SnortConfig *sc)
     {
         int i;
 
-        if (node->rule == NULL)
+        if (node->rule == NULL || (!node->rule->initialized) || node->rule->options == NULL)
             continue;
 
         for (i = 0; node->rule->options[i] != NULL; i++)
