@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -990,6 +990,9 @@ typedef enum
     APP_ID_AOL=1419,
     APP_ID_MDNS=1755,
     APP_ID_APPLE_CORE_MEDIA=2253,
+    APP_ID_HTTP_TUNNEL=2296,
+    APP_ID_RTP_AUDIO=2475,
+    APP_ID_RTP_VIDEO=2476,
     APP_ID_ULTRASURF=2634,
     APP_ID_LYCOS=2775,
     APP_ID_DOGPILE=2804,
@@ -1000,11 +1003,20 @@ typedef enum
     APP_ID_ANYCONNECT_IPSEC_CLIENT=2923,
     APP_ID_ICMP=3501,
     APP_ID_ICMPV6=3558,
+    APP_ID_HTTP_SSL_TUNNEL=3860,
     APP_ID_FTP_ACTIVE=4002,
     APP_ID_FTP_PASSIVE=4003,
     APP_ID_UNKNOWN_UI = 65535  /*This causes the UI to render Unknown instead of pending or blank */
 } appIdEnum;
 
+typedef enum
+{
+    APP_ID_TYPE_SERVICE,
+    APP_ID_TYPE_CLIENT,
+    APP_ID_TYPE_PAYLOAD,
+    APP_ID_TYPE_MAX
+} APP_ID_TYPE;
+
 #define SF_APPID_MAX            40000
 #define SF_APPID_BUILDIN_MAX    30000
 #define APPID_MAX_PRIORITY  3
