@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include "appIdApi.h"
 #include "fw_appid.h"
 #include "thirdparty_appid_api.h"
+#include "appIdConfig.h"
 
 #define SSL_WHITELIST_PKT_LIMIT 20
 
@@ -107,9 +108,20 @@ bool isSessionSslDecrypted(struct AppIdData *appIdData)
 struct AppIdData * getAppIdData(void* lwssn)
 {
     tAppIdData *appIdData = _dpd.sessionAPI->get_application_data(lwssn, PP_APP_ID);
+
     return (appIdData && appIdData->common.fsf_type.flow_type == APPID_SESSION_TYPE_NORMAL)? appIdData : NULL;
 }
 
+bool isHttpInspectionDone(struct AppIdData *appIdSession)
+{
+    if (!appIdSession)
+        return true; // No wait for http discovery if AppId data is unavailable
+    if ((appIdSession->common.fsf_type.flow_type != APPID_SESSION_TYPE_NORMAL)
+        || TPIsAppIdDone(appIdSession->tpsession))
+        return true;
+    return false;
+}
+
 bool IsAppIdInspectingSession(struct AppIdData *appIdSession)
 {
     if (appIdSession && appIdSession->common.fsf_type.flow_type == APPID_SESSION_TYPE_NORMAL)
@@ -133,6 +145,19 @@ bool IsAppIdInspectingSession(struct AppIdData *appIdSession)
         {
             return true;
         }
+        if (appidStaticConfig->recheck_for_unknown_appid)
+        {
+            if( (appIdSession->serviceAppId == APP_ID_UNKNOWN_UI || appIdSession->serviceAppId <= APP_ID_NONE) &&
+                 appIdSession->clientAppId <= APP_ID_NONE &&
+                 appIdSession->payloadAppId <= APP_ID_NONE &&
+                 appIdSession->tpAppId <= APP_ID_NONE &&
+                 appIdSession->miscAppId <= APP_ID_NONE &&
+                 appIdSession->portServiceAppId <= APP_ID_NONE &&
+                 appIdSession->clientServiceAppId <= APP_ID_NONE &&
+                 appIdSession->tpPayloadAppId <= APP_ID_NONE )
+                return true;
+
+        }
     }
     return false;
 }
@@ -464,7 +489,7 @@ uint32_t produceHAState(void *lwssn, uint8_t *buf)
     }
     return sizeof(*appHA);
 }
-uint32_t consumeHAState(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, struct in6_addr *ip,  uint16_t initiatorPort)
+uint32_t consumeHAState(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t proto, const struct in6_addr *ip,  uint16_t initiatorPort)
 {
     AppIdSessionHA *appHA = (AppIdSessionHA *)buf;
     if (appHA->flags & APPID_HA_FLAGS_APP)
@@ -474,7 +499,7 @@ uint32_t consumeHAState(void *lwssn, const uint8_t *buf, uint8_t length, uint8_t
         {
             appIdData = appSharedDataAlloc(proto, ip, initiatorPort);
             _dpd.sessionAPI->set_application_data(lwssn, PP_APP_ID, appIdData, (void (*)(void *))appSharedDataDelete);
-	        appIdData->serviceAppId = appHA->appId[1];
+            appIdData->serviceAppId = appHA->appId[1];
             if (appIdData->serviceAppId == APP_ID_FTP_CONTROL)
             {
                 setAppIdFlag(appIdData, APPID_SESSION_CLIENT_DETECTED | APPID_SESSION_NOT_A_SERVICE | APPID_SESSION_SERVICE_DETECTED);
@@ -556,6 +581,19 @@ uint32_t getDNSTTL(struct AppIdData *appIdData)
     return 0;
 }
 
+static void dumpDebugHostInfo(void)
+{
+    char ipStr[INET6_ADDRSTRLEN];
+
+    ipStr[0] = '\0';
+    if (AppIdDebugHostInfo.family == AF_INET)
+        inet_ntop(AF_INET, (const struct in6_addr*) &AppIdDebugHostInfo.initiatorIp.s6_addr32[3], ipStr, sizeof(ipStr));
+    else
+        inet_ntop(AF_INET6, &AppIdDebugHostInfo.initiatorIp, ipStr, sizeof(ipStr));
+    _dpd.logMsg("AppIdDebugHost: session %s, initiator %s:%u, direction %d, protocol %u, monitorType %d\n",
+                AppIdDebugHostInfo.session ? "not null" : "null", ipStr, AppIdDebugHostInfo.initiatorPort, AppIdDebugHostInfo.direction, AppIdDebugHostInfo.protocol, AppIdDebugHostInfo.monitorType);
+}
+
 static struct AppIdApi appIdDispatchTable = {
     appGetAppName,
     appGetAppId,
@@ -631,7 +669,9 @@ static struct AppIdApi appIdDispatchTable = {
     getHttpNewField,
     freeHttpNewField,
     getHttpFieldOffset,
-    getHttpFieldEndOffset
+    getHttpFieldEndOffset,
+    isHttpInspectionDone,
+    dumpDebugHostInfo
 };
 
 void appIdApiInit(struct AppIdApi *api)
