@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -37,10 +37,23 @@
 #define APP_ID_MEMCAP_DEFAULT       (256*1024*1024ULL)
 #define APP_ID_MEMCAP_UPPER_BOUND   (3*1024*1024*1024ULL)
 #define APP_ID_MEMCAP_LOWER_BOUND   (32*1024*1024ULL)
+#define APP_ID_MEMCAP_ABSOLUTE_LOWER_BOUND (1*1024*1024UL + 16UL)
 
 #define DEFAULT_APPID_DETECTOR_PATH "/usr/local/etc/appid"
 
-void appIdConfigParse(char *args)
+static void appIdConfigDump(tAppidStaticConfig* appidSC)
+{
+    _dpd.logMsg("AppId Configuration\n");
+
+    _dpd.logMsg("    Detector Path:          %s\n", appidSC->app_id_detector_path ? appidSC->app_id_detector_path : "NULL");
+    _dpd.logMsg("    appStats Files:         %s\n", appidSC->app_stats_filename ? appidSC->app_stats_filename : "NULL");
+    _dpd.logMsg("    appStats Period:        %d secs\n", appidSC->app_stats_period);
+    _dpd.logMsg("    appStats Rollover Size: %d bytes\n", appidSC->app_stats_rollover_size);
+    _dpd.logMsg("    appStats Rollover time: %d secs\n", appidSC->app_stats_rollover_time);
+    _dpd.logMsg("\n");
+}
+
+void appIdConfigParse(tAppidStaticConfig* appidSC, char *args)
 {
     char **toks;
     int num_toks;
@@ -49,11 +62,11 @@ void appIdConfigParse(char *args)
     int s_toks;
     char *endPtr;
 
+    memset (appidSC, 0, sizeof(*appidSC));
+
     if ((args == NULL) || (strlen(args) == 0))
         return;
 
-    memset (&appidStaticConfig, 0, sizeof(appidStaticConfig));
-
     toks = _dpd.tokenSplit(args, ",", 0, &num_toks, 0);
     i = 0;
 
@@ -68,12 +81,15 @@ void appIdConfigParse(char *args)
 
         if(!strcasecmp(stoks[0], "conf"))
         {
-            if ((s_toks != 2) || strlen(stoks[1]) >= sizeof(appidStaticConfig.conf_file))
+            if ((s_toks != 2) || !*stoks[1])
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid rna_conf");
             }
 
-            snprintf(appidStaticConfig.conf_file, sizeof(appidStaticConfig.conf_file), "%s", stoks[1]);
+            if (NULL == (appidSC->conf_file = strdup(stoks[1])))
+            {
+                _dpd.fatalMsg("Appid failed to allocate configuration file name\n");
+            }
         }
         else if(!strcasecmp(stoks[0], "debug"))
         {
@@ -82,7 +98,7 @@ void appIdConfigParse(char *args)
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid debug");
             }
             if (!strcasecmp(stoks[1], "yes"))
-                appidStaticConfig.app_id_debug = 1;
+                appidSC->app_id_debug = 1;
         }
         else if(!strcasecmp(stoks[0], "dump_ports"))
         {
@@ -90,7 +106,7 @@ void appIdConfigParse(char *args)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid dump ports specified");
             }
-            appidStaticConfig.app_id_dump_ports = 1;
+            appidSC->app_id_dump_ports = 1;
         }
         else if(!strcasecmp(stoks[0], "memcap"))
         {
@@ -99,23 +115,34 @@ void appIdConfigParse(char *args)
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid memcap");
             }
 
-            appidStaticConfig.memcap = strtoul(stoks[1], &endPtr, 10);
+            appidSC->memcap = strtoul(stoks[1], &endPtr, 10);
             if (!*stoks[1] || *endPtr)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid memcap");
             }
 
-            if (appidStaticConfig.memcap == 0)
-                appidStaticConfig.memcap = APP_ID_MEMCAP_LOWER_BOUND;
+            if (appidSC->memcap == 0)
+                appidSC->memcap = APP_ID_MEMCAP_LOWER_BOUND;
+            if (appidSC->memcap > APP_ID_MEMCAP_UPPER_BOUND)
+                appidSC->memcap = APP_ID_MEMCAP_UPPER_BOUND;
+            if (APP_ID_MEMCAP_ABSOLUTE_LOWER_BOUND > appidSC->memcap)
+            {
+                _dpd.errMsg("AppId invalid memory cap, %lu, overridden with %lu",
+                            appidSC->memcap, APP_ID_MEMCAP_ABSOLUTE_LOWER_BOUND);
+                appidSC->memcap = APP_ID_MEMCAP_ABSOLUTE_LOWER_BOUND;
+            }
         }
         else if(!strcasecmp(stoks[0], "app_stats_filename"))
         {
-            if ((s_toks != 2) || strlen(stoks[1]) >= sizeof(appidStaticConfig.app_stats_filename))
+            if ((s_toks != 2) || !*stoks[1])
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid stats_filename");
             }
 
-            snprintf(appidStaticConfig.app_stats_filename, sizeof(appidStaticConfig.app_stats_filename), "%s", stoks[1]);
+            if (NULL == (appidSC->app_stats_filename = strdup(stoks[1])))
+            {
+                _dpd.fatalMsg("Appid failed to allocate stats file name\n");
+            }
         }
         else if(!strcasecmp(stoks[0], "app_stats_period"))
         {
@@ -124,7 +151,7 @@ void appIdConfigParse(char *args)
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_period");
             }
 
-            appidStaticConfig.app_stats_period = strtoul(stoks[1], &endPtr, 10);
+            appidSC->app_stats_period = strtoul(stoks[1], &endPtr, 10);
             if (!*stoks[1] || *endPtr)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_period");
@@ -137,7 +164,7 @@ void appIdConfigParse(char *args)
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_rollover_size");
             }
 
-            appidStaticConfig.app_stats_rollover_size = strtoul(stoks[1], &endPtr, 10);
+            appidSC->app_stats_rollover_size = strtoul(stoks[1], &endPtr, 10);
             if (!*stoks[1] || *endPtr)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_rollover_size");
@@ -150,7 +177,7 @@ void appIdConfigParse(char *args)
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_rollover_time");
             }
 
-            appidStaticConfig.app_stats_rollover_time = strtoul(stoks[1], &endPtr, 10);
+            appidSC->app_stats_rollover_time = strtoul(stoks[1], &endPtr, 10);
             if (!*stoks[1] || *endPtr)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_stats_rollover_time");
@@ -158,12 +185,15 @@ void appIdConfigParse(char *args)
         }
         else if(!strcasecmp(stoks[0], "app_detector_dir"))
         {
-            if ((s_toks != 2) || strlen(stoks[1]) >= sizeof(appidStaticConfig.app_id_detector_path))
+            if ((s_toks != 2) || !*stoks[1])
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid app_detector_dir");
             }
 
-            snprintf(appidStaticConfig.app_id_detector_path, sizeof(appidStaticConfig.app_id_detector_path), "%s", stoks[1]);
+            if (NULL == (appidSC->app_id_detector_path = strdup(stoks[1])))
+            {
+                _dpd.fatalMsg("Appid failed to allocate detector path\n");
+            }
         }
         else if(!strcasecmp(stoks[0], "instance_id"))
         {
@@ -171,7 +201,7 @@ void appIdConfigParse(char *args)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid instance id");
             }
-            appidStaticConfig.instance_id = strtoul(stoks[1], &endPtr, 10);
+            appidSC->instance_id = strtoul(stoks[1], &endPtr, 10);
             if (!*stoks[1] || *endPtr)
             {
                 _dpd.fatalMsg("Invalid instance id specified");
@@ -179,16 +209,16 @@ void appIdConfigParse(char *args)
         }
         else if(!strcasecmp(stoks[0], "thirdparty_appid_dir"))
         {
-            if (appidStaticConfig.appid_thirdparty_dir)
+            if (appidSC->appid_thirdparty_dir)
             {
-                free((void *)appidStaticConfig.appid_thirdparty_dir);
-                appidStaticConfig.appid_thirdparty_dir = NULL;
+                free((void *)appidSC->appid_thirdparty_dir);
+                appidSC->appid_thirdparty_dir = NULL;
             }
             if (s_toks != 2)
             {
                 _dpd.fatalMsg("%s(%d) => %s\n", *(_dpd.config_file), *(_dpd.config_line), "Invalid ThirdpartyDirectory");
             }
-            if (!(appidStaticConfig.appid_thirdparty_dir = strdup(stoks[1])))
+            if (!(appidSC->appid_thirdparty_dir = strdup(stoks[1])))
             {
                 _dpd.errMsg("Failed to allocate a module directory");
                 return;
@@ -203,33 +233,26 @@ void appIdConfigParse(char *args)
         _dpd.tokenFree(&stoks, s_toks);
     }
 
-    if (!appidStaticConfig.memcap)
-        appidStaticConfig.memcap = APP_ID_MEMCAP_DEFAULT;
-    if (!appidStaticConfig.app_stats_period)
-        appidStaticConfig.app_stats_period = 5*60;
-    if (!appidStaticConfig.app_stats_rollover_size)
-        appidStaticConfig.app_stats_rollover_size = 20 * 1024 * 1024;
-    if (!appidStaticConfig.app_stats_rollover_time)
-        appidStaticConfig.app_stats_rollover_time = 24*60*60;
+    if (!appidSC->memcap)
+        appidSC->memcap = APP_ID_MEMCAP_DEFAULT;
+    if (!appidSC->app_stats_period)
+        appidSC->app_stats_period = 5*60;
+    if (!appidSC->app_stats_rollover_size)
+        appidSC->app_stats_rollover_size = 20 * 1024 * 1024;
+    if (!appidSC->app_stats_rollover_time)
+        appidSC->app_stats_rollover_time = 24*60*60;
 
-    if (!appidStaticConfig.app_id_detector_path[0])
-        snprintf(appidStaticConfig.app_id_detector_path, sizeof(appidStaticConfig.app_id_detector_path), "%s", DEFAULT_APPID_DETECTOR_PATH);
+    if (!appidSC->app_id_detector_path)
+    {
+        if (NULL == (appidSC->app_id_detector_path = strdup(DEFAULT_APPID_DETECTOR_PATH)))
+        {
+            _dpd.fatalMsg("Appid failed to allocate detector path\n");
+        }
+    }
 
     _dpd.tokenFree(&toks, num_toks);
 
-    appIdConfigDump();
-}
-
-void appIdConfigDump(void)
-{
-    _dpd.logMsg("AppId Configuration\n");
-
-    _dpd.logMsg("    Detector Path:          %s\n", appidStaticConfig.app_id_detector_path);
-    _dpd.logMsg("    appStats Files:         %s\n", appidStaticConfig.app_stats_filename? appidStaticConfig.app_stats_filename:"NULL");
-    _dpd.logMsg("    appStats Period:        %d secs\n", appidStaticConfig.app_stats_period);
-    _dpd.logMsg("    appStats Rollover Size: %d bytes\n", appidStaticConfig.app_stats_rollover_size);
-    _dpd.logMsg("    appStats Rollover time: %d secs\n", appidStaticConfig.app_stats_rollover_time);
-    _dpd.logMsg("\n");
+    appIdConfigDump(appidSC);
 }
 
 void AppIdAddGenericConfigItem(tAppIdConfig *pConfig, const char *name, void *pData)
