@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -82,48 +82,6 @@ typedef struct appidGenericConfigItem_
     void    *pData; ///< Module configuration data
 } tAppidGenericConfigItem;
 
-void appIdConfigParse(char *args);
-void appIdConfigDump(void);
-
-/**
- * \struct tAppidStaticConfig
- *
- * \brief AppId static configuration data structure
- *
- * Members of this data structure get populated during initialization and freed
- * during exit. They are not reloadable/reconfigurable.
- * Note: appid_tp_dir can be reconfigured but gets used by 3rd party reload. AppID
- * reload does not look at this variable.
- */
-struct AppidStaticConfig
-{
-    unsigned    disable_safe_search;
-    const char *appid_thirdparty_dir;         /* directory where thirdparty modules are located.*/
-    char app_stats_filename[PATH_MAX];
-    unsigned long app_stats_period;
-    unsigned long app_stats_rollover_size;
-    unsigned long app_stats_rollover_time;
-    char app_id_detector_path[PATH_MAX];
-    unsigned long memcap;
-    int app_id_dump_ports;
-    int app_id_debug;
-    uint32_t instance_id;
-    char conf_file[PATH_MAX];
-    unsigned dns_host_reporting;
-    unsigned referred_appId_disabled;
-    unsigned rtmp_max_packets;
-    unsigned mdns_user_reporting;
-    unsigned ftp_userid_disabled;
-    unsigned chp_userid_disabled;
-    unsigned chp_body_collection_disabled;
-    unsigned chp_fflow_disabled;
-    unsigned chp_body_collection_max;
-    unsigned max_tp_flow_depth;
-    unsigned tp_allow_probes;
-    bool http2_detection_enabled;    // internal HTTP/2 detection
-};
-typedef struct AppidStaticConfig tAppidStaticConfig;
-
 /**
  * \typedef tAppIdConfig
  *
@@ -155,6 +113,8 @@ typedef struct appIdConfig_
     SFXHASH     *AF_indicators;             ///< App Forecasting list of "indicator apps"
     SFXHASH     *AF_actives;                ///< App Forecasting list of hosts to watch for "forecast apps"
 
+    sfaddr_t    *debugHostIp;
+
     struct _AppInfoTableEntry   *AppInfoList;
     struct _AppInfoTableEntry   *AppInfoTable[SF_APPID_MAX];
     struct _AppInfoTableEntry   *AppInfoTableByService[SF_APPID_MAX];
@@ -180,11 +140,57 @@ typedef struct appIdConfig_
     SF_LIST                 genericConfigList;      ///< List of tAppidGenericConfigItem structures
 } tAppIdConfig;
 
+/**
+ * \struct tAppidStaticConfig
+ *
+ * \brief AppId static configuration data structure
+ *
+ * Members of this data structure get populated during initialization and freed
+ * during exit. They are not reloadable/reconfigurable.
+ * Note: appid_tp_dir can be reconfigured but gets used by 3rd party reload. AppID
+ * reload does not look at this variable.
+ */
+struct AppidStaticConfig
+{
+    unsigned    disable_safe_search;
+    const char *appid_thirdparty_dir;         /* directory where thirdparty modules are located.*/
+    char* app_stats_filename;
+    unsigned long app_stats_period;
+    unsigned long app_stats_rollover_size;
+    unsigned long app_stats_rollover_time;
+    char* app_id_detector_path;
+    unsigned long memcap;
+    int app_id_dump_ports;
+    int app_id_debug;
+    uint32_t instance_id;
+    char* conf_file;
+    unsigned dns_host_reporting;
+    unsigned referred_appId_disabled;
+    unsigned rtmp_max_packets;
+    unsigned mdns_user_reporting;
+    unsigned ftp_userid_disabled;
+    unsigned chp_userid_disabled;
+    unsigned chp_body_collection_disabled;
+    unsigned chp_fflow_disabled;
+    unsigned chp_body_collection_max;
+    unsigned max_tp_flow_depth;
+    unsigned tp_allow_probes;
+    unsigned host_port_app_cache_lookup_interval;
+    unsigned host_port_app_cache_lookup_range;
+    bool http2_detection_enabled;    // internal HTTP/2 detection
+    bool is_host_port_app_cache_runtime;
+    bool recheck_for_unknown_appid;
+    tAppIdConfig* newAppIdConfig;    // Used only during reload
+};
+typedef struct AppidStaticConfig tAppidStaticConfig;
+
+void appIdConfigParse(tAppidStaticConfig* appidSC, char *args);
+
 
 /************************** GLOBAL VARIABLES **********************************/
 
 /// AppId static configuration data
-extern tAppidStaticConfig   appidStaticConfig;
+extern tAppidStaticConfig* appidStaticConfig;
 
 /**
  * \brief Pointer to AppId dynamic configuration data
