@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -250,7 +250,7 @@ void appIdStatsInit(char* appFileName, time_t statsPeriod, size_t rolloverSize,
     size_t pathLength;
     char *path;
 
-    if (!appFileName[0])
+    if (!appFileName || !*appFileName)
     {
         enableAppStats = false;
         return;
