@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -42,6 +42,10 @@
 #define CONF_SEPARATORS         "\t\n\r"
 #define MIN_MAX_TP_FLOW_DEPTH   1
 #define MAX_MAX_TP_FLOW_DEPTH   1000000
+#define MIN_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL    1
+#define MAX_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL    1000000
+#define MIN_HOST_PORT_APP_CACHE_LOOKUP_RANGE       1
+#define MAX_HOST_PORT_APP_CACHE_LOOKUP_RANGE       1000000
 
 struct DynamicArray
 { 
@@ -233,7 +237,7 @@ void* appNameHashFind(SFGHASH *appNameHash, const char *appName)
 }
 // End of appName hash
 
-static void appIdConfLoad (const char *path);
+static void appIdConfLoad (tAppidStaticConfig* appidSC, const char *path);
 
 static unsigned int getAppIdStaticIndex(tAppId appid)
 {
@@ -297,7 +301,7 @@ AppInfoTableEntry* appInfoEntryCreate(const char *appName, tAppIdConfig *pConfig
     return entry;
 }
 
-void appInfoTableInit(const char *path, tAppIdConfig* pConfig)
+void appInfoTableInit(tAppidStaticConfig* appidSC, tAppIdConfig* pConfig)
 {
     FILE *tableFile;
     const char *token;
@@ -311,7 +315,7 @@ void appInfoTableInit(const char *path, tAppIdConfig* pConfig)
 
     pConfig->AppInfoTableDyn = dynamicArrayCreate(SF_APPID_DYNAMIC_MIN);
 
-    snprintf(filepath, sizeof(filepath), "%s/odp/%s", path, APP_MAPPING_FILE);
+    snprintf(filepath, sizeof(filepath), "%s/odp/%s", appidSC->app_id_detector_path, APP_MAPPING_FILE);
 
     tableFile = fopen(filepath, "r");
     if (tableFile == NULL)
@@ -436,16 +440,20 @@ void appInfoTableInit(const char *path, tAppIdConfig* pConfig)
     fclose(tableFile);
 
     /* Configuration defaults. */
-    appidStaticConfig.rtmp_max_packets = 15;
-    appidStaticConfig.mdns_user_reporting = 1;
-    appidStaticConfig.dns_host_reporting = 1;
-    appidStaticConfig.max_tp_flow_depth = 5;
-    appidStaticConfig.http2_detection_enabled = 0;
+    appidSC->rtmp_max_packets = 15;
+    appidSC->mdns_user_reporting = 1;
+    appidSC->dns_host_reporting = 1;
+    appidSC->max_tp_flow_depth = 5;
+    appidSC->http2_detection_enabled = 0;
+    appidSC->host_port_app_cache_lookup_interval = 10;
+    appidSC->host_port_app_cache_lookup_range = 100000;
+    appidSC->is_host_port_app_cache_runtime = 1;
+    appidSC->recheck_for_unknown_appid = 0;
 
-    snprintf(filepath, sizeof(filepath), "%s/odp/%s", path, APP_CONFIG_FILE);
-    appIdConfLoad (filepath);
-    snprintf(filepath, sizeof(filepath), "%s/custom/%s", path, USR_CONFIG_FILE);
-    appIdConfLoad (filepath);
+    snprintf(filepath, sizeof(filepath), "%s/odp/%s", appidSC->app_id_detector_path, APP_CONFIG_FILE);
+    appIdConfLoad (appidSC, filepath);
+    snprintf(filepath, sizeof(filepath), "%s/../%s", appidSC->app_id_detector_path, USR_CONFIG_FILE);
+    appIdConfLoad (appidSC, filepath);
 }
 
 void appInfoTableFini(tAppIdConfig *pConfig)
@@ -571,7 +579,7 @@ void appInfoSetActive(tAppId appId, bool active)
 }
 
 
-static void appIdConfLoad (const char *path)
+static void appIdConfLoad (tAppidStaticConfig* appidSC, const char *path)
 {
     FILE *config_file;
     char *token;
@@ -584,6 +592,8 @@ static void appIdConfLoad (const char *path)
     unsigned line = 0;
     tAppIdConfig *pConfig = appIdNewConfigGet();
     int max_tp_flow_depth;
+    int host_port_app_cache_lookup_interval;
+    int host_port_app_cache_lookup_range;
 
     config_file = fopen(path, "r");
     if (config_file == NULL)
@@ -631,11 +641,55 @@ static void appIdConfLoad (const char *path)
                 if (max_tp_flow_depth < MIN_MAX_TP_FLOW_DEPTH || max_tp_flow_depth > MAX_MAX_TP_FLOW_DEPTH)
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: invalid max_tp_flow_depth %d, must be between %d and %d\n.", max_tp_flow_depth, MIN_MAX_TP_FLOW_DEPTH, MAX_MAX_TP_FLOW_DEPTH););
-                }                    
+                }
                 else
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: setting max thirdparty inspection flow depth to %d packets.\n", max_tp_flow_depth););
-                    appidStaticConfig.max_tp_flow_depth = max_tp_flow_depth;
+                    appidSC->max_tp_flow_depth = max_tp_flow_depth;
+                }
+            }
+            else if (!(strcasecmp(conf_key, "host_port_app_cache_lookup_interval")))
+            {
+                host_port_app_cache_lookup_interval = atoi(conf_val);
+                if (host_port_app_cache_lookup_interval < MIN_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL || host_port_app_cache_lookup_interval > MAX_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL)
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: invalid host_port_app_cache_lookup_interval %d, must be between %d and %d\n.",
+                                            host_port_app_cache_lookup_interval, MIN_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL, MAX_HOST_PORT_APP_CACHE_LOOKUP_INTERVAL););
+                }
+                else
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: hostPortCache lookup performed every %d packet(s).\n", host_port_app_cache_lookup_interval););
+                    appidSC->host_port_app_cache_lookup_interval = host_port_app_cache_lookup_interval;
+                }
+            }
+            else if (!(strcasecmp(conf_key, "host_port_app_cache_lookup_range")))
+            {
+                host_port_app_cache_lookup_range = atoi(conf_val);
+                if (host_port_app_cache_lookup_range < MIN_HOST_PORT_APP_CACHE_LOOKUP_RANGE || host_port_app_cache_lookup_range > MAX_HOST_PORT_APP_CACHE_LOOKUP_RANGE)
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: invalid host_port_app_cache_lookup_range %d, must be between %d and %d\n.",
+                                            host_port_app_cache_lookup_range, MIN_HOST_PORT_APP_CACHE_LOOKUP_RANGE, MAX_HOST_PORT_APP_CACHE_LOOKUP_RANGE););
+                }
+                else
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: hostPortCache lookup performed till maximum of %d packet(s) per session.\n", host_port_app_cache_lookup_range););
+                    appidSC->host_port_app_cache_lookup_range = host_port_app_cache_lookup_range;
+                }
+            }
+            else if (!(strcasecmp(conf_key, "is_host_port_app_cache_runtime")))
+            {
+                if (!(strcasecmp(conf_val, "disabled")))
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: hostPortCache not configured for runtime modification.\n"););
+                    appidSC->is_host_port_app_cache_runtime = 0;
+                }
+            }
+            else if (!(strcasecmp(conf_key, "recheck_for_unknown_appid")))
+            {
+                if (!(strcasecmp(conf_val, "enabled")))
+                {
+                    DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: Flows with unknown AppID's will not be ignored.\n"););
+                    appidSC->recheck_for_unknown_appid = 1;
                 }
             }
             else if (!(strcasecmp(conf_key, "tp_allow_probes")))
@@ -643,7 +697,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "enabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: TCP probes will be analyzed by NAVL.\n"););
-                    appidStaticConfig.tp_allow_probes = 1;
+                    appidSC->tp_allow_probes = 1;
                 }
             }
             else if (!(strcasecmp(conf_key, "tp_client_app")))
@@ -661,7 +715,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: disabling safe search enforcement.\n"););
-                    appidStaticConfig.disable_safe_search = 1;
+                    appidSC->disable_safe_search = 1;
                 }
             }
             else if (!(strcasecmp(conf_key, "ssl_squelch")))
@@ -684,7 +738,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: HTTP UserID collection disabled.\n"););
-                    appidStaticConfig.chp_userid_disabled = 1;
+                    appidSC->chp_userid_disabled = 1;
                     continue;
                 }
             }
@@ -693,7 +747,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: HTTP Body header reading disabled.\n"););
-                    appidStaticConfig.chp_body_collection_disabled = 1;
+                    appidSC->chp_body_collection_disabled = 1;
                     continue;
                 }
             }
@@ -702,7 +756,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: HTTP future flow creation disabled.\n"););
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidSC->chp_fflow_disabled = 1;
                     continue;
                 }
             }
@@ -711,7 +765,7 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     DEBUG_WRAP(DebugMessage(DEBUG_APPID, "AppId: FTP userID disabled.\n"););
-                    appidStaticConfig.ftp_userid_disabled = 1;
+                    appidSC->ftp_userid_disabled = 1;
                     continue;
                 }
             }
@@ -737,11 +791,11 @@ static void appIdConfLoad (const char *path)
 
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
-                    appidStaticConfig.referred_appId_disabled = 1;
+                    appidSC->referred_appId_disabled = 1;
                     continue;
                 }
 
-                else if (!appidStaticConfig.referred_appId_disabled)
+                else if (!appidSC->referred_appId_disabled)
                 {
                     referred_app_index=0;
                     referred_app_index += sprintf(referred_app_list, "%d ", atoi(conf_val));
@@ -757,19 +811,19 @@ static void appIdConfLoad (const char *path)
             }
             else if (!(strcasecmp(conf_key, "rtmp_max_packets")))
             {
-                appidStaticConfig.rtmp_max_packets = atoi(conf_val);
+                appidSC->rtmp_max_packets = atoi(conf_val);
             }
             else if (!(strcasecmp(conf_key, "mdns_user_report")))
             {
-                appidStaticConfig.mdns_user_reporting = atoi(conf_val);
+                appidSC->mdns_user_reporting = atoi(conf_val);
             }
             else if (!(strcasecmp(conf_key, "dns_host_report")))
-            {   
-                appidStaticConfig.dns_host_reporting = atoi(conf_val);
+            {
+                appidSC->dns_host_reporting = atoi(conf_val);
             }
             else if (!(strcasecmp(conf_key, "chp_body_max_bytes")))
             {
-                appidStaticConfig.chp_body_collection_max = atoi(conf_val);
+                appidSC->chp_body_collection_max = atoi(conf_val);
             }
             else if (!(strcasecmp(conf_key, "ignore_thirdparty_appid")))
             {
@@ -786,12 +840,12 @@ static void appIdConfLoad (const char *path)
                 if (!(strcasecmp(conf_val, "disabled")))
                 {
                     _dpd.logMsg("AppId: disabling internal HTTP/2 detection.\n");
-                    appidStaticConfig.http2_detection_enabled = 0;
+                    appidSC->http2_detection_enabled = 0;
                 }
                 else if (!(strcasecmp(conf_val, "enabled")))
                 {
                     _dpd.logMsg("AppId: enabling internal HTTP/2 detection.\n");
-                    appidStaticConfig.http2_detection_enabled = 1;
+                    appidSC->http2_detection_enabled = 1;
                 }
                 else
                 {
