@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -65,7 +65,7 @@ struct _AppInfoTableEntry
 };
 typedef struct _AppInfoTableEntry AppInfoTableEntry;
 
-void appInfoTableInit(const char *path, tAppIdConfig* pConfig);
+void appInfoTableInit(tAppidStaticConfig* appidSC, tAppIdConfig* pConfig);
 void appInfoTableFini(tAppIdConfig *pConfig);
 AppInfoTableEntry* appInfoEntryGet(tAppId appId, const tAppIdConfig *pConfig);
 AppInfoTableEntry* appInfoEntryCreate(const char *appName, tAppIdConfig *pConfig);
