@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -612,7 +612,7 @@ void UnconfigureClientApp(tAppIdConfig *pConfig)
  *
  * @param args
  */
-void ClientAppInit(tAppIdConfig *pConfig)
+void ClientAppInit(tAppidStaticConfig* appidSC, tAppIdConfig *pConfig)
 {
     RNAClientAppRecord *li;
 
@@ -623,10 +623,10 @@ void ClientAppInit(tAppIdConfig *pConfig)
     DisplayClientAppConfig(&pConfig->clientAppConfig);
 
     if (pConfig->clientAppConfig.enabled)
-     {
+    {
         client_init_api.debug = app_id_debug;
         client_init_api.pAppidConfig = pConfig;
-        client_init_api.instance_id = appidStaticConfig.instance_id;
+        client_init_api.instance_id = appidSC->instance_id;
 
         for (li = pConfig->clientAppConfig.tcp_client_app_list; li; li = li->next)
             initialize_module(li, &pConfig->clientAppConfig);
