@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 
 #define GENERIC_APP_OFFSET 2000000000
 
-void ClientAppInit(tAppIdConfig *pConfig);
+void ClientAppInit(tAppidStaticConfig* appidSC, tAppIdConfig *pConfig);
 void ClientAppFinalize(tAppIdConfig *pConfig);
 void UnconfigureClientApp(tAppIdConfig *pConfig);
 void CleanupClientApp(tAppIdConfig *pConfig);
