@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -483,7 +483,7 @@ static int dns_udp_validate(ServiceValidationArgs* args)
         rval = (dir == APP_ID_FROM_INITIATOR) ? SERVICE_INVALID_CLIENT:SERVICE_NOMATCH;
         goto udp_done;
     }
-    if ((rval = dns_validate_header(dir, (DNSHeader *)data, appidStaticConfig.dns_host_reporting, flowp)) != SERVICE_SUCCESS)
+    if ((rval = dns_validate_header(dir, (DNSHeader *)data, appidStaticConfig->dns_host_reporting, flowp)) != SERVICE_SUCCESS)
     {
         if (rval == SERVICE_REVERSED)
         {
@@ -493,7 +493,7 @@ static int dns_udp_validate(ServiceValidationArgs* args)
                 {
                     // To get here, we missed the initial query, got a
                     // response, and now we've got another query.
-                    rval = validate_packet(data, size, dir, appidStaticConfig.dns_host_reporting, flowp);
+                    rval = validate_packet(data, size, dir, appidStaticConfig->dns_host_reporting, flowp);
                     if (rval == SERVICE_SUCCESS)
                         goto inprocess;
                 }
@@ -503,7 +503,7 @@ static int dns_udp_validate(ServiceValidationArgs* args)
             {
                 // To get here, we missed the initial query, but now we've got
                 // a response.
-                rval = validate_packet(data, size, dir, appidStaticConfig.dns_host_reporting, flowp);
+                rval = validate_packet(data, size, dir, appidStaticConfig->dns_host_reporting, flowp);
                 if (rval == SERVICE_SUCCESS)
                 {
                     setAppIdFlag(flowp, APPID_SESSION_UDP_REVERSED);
@@ -516,7 +516,7 @@ static int dns_udp_validate(ServiceValidationArgs* args)
         goto udp_done;
     }
 
-    rval = validate_packet(data, size, dir, appidStaticConfig.dns_host_reporting, flowp);
+    rval = validate_packet(data, size, dir, appidStaticConfig->dns_host_reporting, flowp);
     if ((rval == SERVICE_SUCCESS) && (dir == APP_ID_FROM_INITIATOR))
         goto inprocess;
 
@@ -527,24 +527,24 @@ udp_done:
 success:
         setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         dns_service_mod.api->add_service(flowp, args->pkt, dir, &udp_svc_element,
-                                         APP_ID_DNS, NULL, NULL, NULL);
+                                         APP_ID_DNS, NULL, NULL, NULL, NULL);
         return SERVICE_SUCCESS;
     case SERVICE_INVALID_CLIENT:
 invalid:
         dns_service_mod.api->incompatible_data(flowp, args->pkt, dir, &udp_svc_element,
                                                dns_service_mod.flow_data_index,
-                                               args->pConfig);
+                                               args->pConfig, NULL);
         return SERVICE_NOT_COMPATIBLE;
     case SERVICE_NOMATCH:
 nomatch:
         dns_service_mod.api->fail_service(flowp, args->pkt, dir, &udp_svc_element,
                                           dns_service_mod.flow_data_index,
-                                          args->pConfig);
+                                          args->pConfig, NULL);
         return SERVICE_NOMATCH;
     case SERVICE_INPROCESS:
 inprocess:
         dns_udp_client_mod.api->add_app(flowp, APP_ID_NONE, APP_ID_DNS, NULL);
-        dns_service_mod.api->service_inprocess(flowp, args->pkt, dir, &udp_svc_element);
+        dns_service_mod.api->service_inprocess(flowp, args->pkt, dir, &udp_svc_element, NULL);
         return SERVICE_INPROCESS;
     default:
         return rval;
@@ -574,7 +574,7 @@ static int dns_tcp_validate(ServiceValidationArgs* args)
     data += sizeof(DNSTCPHeader);
     size -= sizeof(DNSTCPHeader);
     tmp = ntohs(hdr->length);
-    if (tmp < sizeof(DNSHeader) || dns_validate_header(dir, (DNSHeader *)data, appidStaticConfig.dns_host_reporting, flowp))
+    if (tmp < sizeof(DNSHeader) || dns_validate_header(dir, (DNSHeader *)data, appidStaticConfig->dns_host_reporting, flowp))
     {
         if (dir == APP_ID_FROM_INITIATOR)
             goto not_compatible;
@@ -584,7 +584,7 @@ static int dns_tcp_validate(ServiceValidationArgs* args)
 
     if (tmp > size)
         goto not_compatible;
-    rval = validate_packet(data, size, dir, appidStaticConfig.dns_host_reporting, flowp);
+    rval = validate_packet(data, size, dir, appidStaticConfig->dns_host_reporting, flowp);
     if (rval != SERVICE_SUCCESS)
         goto tcp_done;
 
@@ -634,24 +634,24 @@ tcp_done:
 success:
     setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
     dns_service_mod.api->add_service(flowp, args->pkt, dir, &tcp_svc_element,
-                                     APP_ID_DNS, NULL, NULL, NULL);
+                                     APP_ID_DNS, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 
 not_compatible:
     dns_service_mod.api->incompatible_data(flowp, args->pkt, dir, &tcp_svc_element,
                                            dns_service_mod.flow_data_index,
-                                           args->pConfig);
+                                           args->pConfig, NULL);
     return SERVICE_NOT_COMPATIBLE;
 
 fail:
     dns_service_mod.api->fail_service(flowp, args->pkt, dir, &tcp_svc_element,
                                       dns_service_mod.flow_data_index,
-                                      args->pConfig);
+                                      args->pConfig, NULL);
     return SERVICE_NOMATCH;
 
 inprocess:
     dns_tcp_client_mod.api->add_app(flowp, APP_ID_NONE, APP_ID_DNS, NULL);
-    dns_service_mod.api->service_inprocess(flowp, args->pkt, dir, &tcp_svc_element);
+    dns_service_mod.api->service_inprocess(flowp, args->pkt, dir, &tcp_svc_element, NULL);
     return SERVICE_INPROCESS;
 }
 
