@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -912,6 +912,19 @@ int http_detector_finalize(tAppIdConfig *pConfig)
 
 void http_detector_clean(tDetectorHttpConfig *pHttpConfig)
 {
+    int i;
+    for (i = 0; i < sizeof(pHttpConfig->chp_matchers)/sizeof(pHttpConfig->chp_matchers[0]); i++)
+    {
+        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_matchers[i]);
+        pHttpConfig->chp_matchers[i] = NULL;
+    }
+    // These are same as chp_matchers[] and freed in the loop above
+    pHttpConfig->chp_user_agent_matcher = pHttpConfig->chp_host_matcher =
+        pHttpConfig->chp_referer_matcher = pHttpConfig->chp_uri_matcher =
+        pHttpConfig->chp_cookie_matcher = pHttpConfig->chp_req_body_matcher =
+        pHttpConfig->chp_content_type_matcher = pHttpConfig->chp_location_matcher =
+        pHttpConfig->chp_body_matcher = NULL;
+
     if (pHttpConfig->via_matcher)
     {
         _dpd.searchAPI->search_instance_free(pHttpConfig->via_matcher);
@@ -942,46 +955,6 @@ void http_detector_clean(tDetectorHttpConfig *pHttpConfig)
         _dpd.searchAPI->search_instance_free(pHttpConfig->field_matcher);
         pHttpConfig->field_matcher = NULL;
     }
-    if (pHttpConfig->chp_user_agent_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_user_agent_matcher);
-        pHttpConfig->chp_user_agent_matcher = NULL;
-     }
-    if (pHttpConfig->chp_host_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_host_matcher);
-        pHttpConfig->chp_host_matcher = NULL;
-     }
-    if (pHttpConfig->chp_uri_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_uri_matcher);
-        pHttpConfig->chp_uri_matcher = NULL;
-     }
-    if (pHttpConfig->chp_cookie_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_cookie_matcher);
-        pHttpConfig->chp_cookie_matcher = NULL;
-     }
-    if (pHttpConfig->chp_content_type_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_content_type_matcher);
-        pHttpConfig->chp_content_type_matcher = NULL;
-     }
-    if (pHttpConfig->chp_location_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_location_matcher);
-        pHttpConfig->chp_location_matcher = NULL;
-     }
-    if (pHttpConfig->chp_body_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_body_matcher);
-        pHttpConfig->chp_body_matcher = NULL;
-     }
-    if (pHttpConfig->chp_referer_matcher)
-     {
-        _dpd.searchAPI->search_instance_free(pHttpConfig->chp_referer_matcher);
-        pHttpConfig->chp_referer_matcher = NULL;
-     }
 
     destroyHostUrlMatcher(&pHttpConfig->hostUrlMatcher);
     destroyHostUrlMatcher(&pHttpConfig->RTMPHostUrlMatcher);
@@ -1521,7 +1494,7 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
     if (!mp)
         return APP_ID_NONE;
 
-    if (appidStaticConfig.disable_safe_search)
+    if (appidStaticConfig->disable_safe_search)
     {
         new_field = NULL;
     }
@@ -1564,7 +1537,7 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
             hsession->skip_simple_detect = true;
             break;
         case EXTRACT_USER:
-            if (!*user && !appidStaticConfig.chp_userid_disabled)
+            if (!*user && !appidStaticConfig->chp_userid_disabled)
             {
                 extractCHP(buf, buf_size, tmp->index, match->psize,
                        match->action_data, user);
@@ -1585,13 +1558,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
             }
             break;
         case FUTURE_APPID_SESSION_SIP:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -1599,13 +1572,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
                 hsession->fflow->sip = ffSetIp(buf, buf_size, tmp->index, match->psize);
             break;
         case FUTURE_APPID_SESSION_DIP:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -1613,13 +1586,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
                 hsession->fflow->dip = ffSetIp(buf, buf_size, tmp->index, match->psize);
             break;
         case FUTURE_APPID_SESSION_SPORT:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -1627,13 +1600,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
                 hsession->fflow->sport = ffSetPort(buf, buf_size, tmp->index, match->psize);
             break;
         case FUTURE_APPID_SESSION_DPORT:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -1641,13 +1614,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
                 hsession->fflow->dport = ffSetPort(buf, buf_size, tmp->index, match->psize);
             break;
         case FUTURE_APPID_SESSION_PROTOCOL:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -1655,13 +1628,13 @@ tAppId scanCHP (PatternType ptype, char *buf, int buf_size, MatchedCHPAction *mp
                 hsession->fflow->protocol = ffSetProtocol(buf, buf_size, tmp->index, match->psize);
             break;
         case FUTURE_APPID_SESSION_CREATE:
-            if (appidStaticConfig.chp_fflow_disabled)
+            if (appidStaticConfig->chp_fflow_disabled)
                 break;
             if (!hsession->fflow)
             {
                 if (!(hsession->fflow = (fflow_info*)calloc(1,sizeof(fflow_info))))
                 {
-                    appidStaticConfig.chp_fflow_disabled = 1;
+                    appidStaticConfig->chp_fflow_disabled = 1;
                     break;
                 }
             }
@@ -2642,7 +2615,7 @@ static CLIENT_APP_RETCODE http_client_init(const InitClientAppAPI * const init_a
 {
     unsigned i;
 
-    if (appidStaticConfig.http2_detection_enabled)
+    if (appidStaticConfig->http2_detection_enabled)
     {
         for (i=0; i < sizeof(patterns)/sizeof(*patterns); i++)
         {
@@ -2670,7 +2643,7 @@ static CLIENT_APP_RETCODE http_client_validate(const uint8_t *data, uint16_t siz
     http_client_mod.api->add_app(flowp, APP_ID_HTTP, APP_ID_HTTP + GENERIC_APP_OFFSET, NULL);
     flowp->rnaClientState = RNA_STATE_FINISHED;
     http_service_mod.api->add_service(flowp, pkt, dir, &http_service_element,
-                                      APP_ID_HTTP, NULL, NULL, NULL);
+                                      APP_ID_HTTP, NULL, NULL, NULL, NULL);
     flowp->rnaServiceState = RNA_STATE_FINISHED;
     setAppIdFlag(flowp, APPID_SESSION_CLIENT_DETECTED | APPID_SESSION_SERVICE_DETECTED);
     clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
