@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -982,20 +982,20 @@ static int imap_validate(ServiceValidationArgs* args)
         {
             /* IMAP server response to STARTTLS command from client was OK */
             service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                         APP_ID_IMAPS, NULL, NULL, NULL);
+                                         APP_ID_IMAPS, NULL, NULL, NULL, NULL);
             return SERVICE_SUCCESS;
         }
         if (id->count >= IMAP_COUNT_THRESHOLD && !getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
             service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                         APP_ID_IMAP, NULL, NULL, NULL);
+                                         APP_ID_IMAP, NULL, NULL, NULL, NULL);
             return SERVICE_SUCCESS;
         }
     }
     else if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
     {
         service_mod.api->fail_service(flowp, args->pkt, args->dir, &svc_element,
-                                      service_mod.flow_data_index, args->pConfig);
+                                      service_mod.flow_data_index, args->pConfig, NULL);
         return SERVICE_NOMATCH;
     }
     else
@@ -1005,7 +1005,7 @@ static int imap_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element);
+    service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 }
 
