@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -926,7 +926,7 @@ static KRB_RETCODE krb_walk_server_packet(KRBState *krbs, const uint8_t *s, cons
             if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED) && pkt)
             {
                 service_mod.api->add_service(flowp, pkt, dir, &svc_element, APP_ID_KERBEROS,
-                        NULL, krbs->ver, NULL);
+                        NULL, krbs->ver, NULL, NULL);
                 setAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED);
             }
 
@@ -1097,7 +1097,7 @@ static int krb_server_validate(ServiceValidationArgs* args)
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
             service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
-                                          service_mod.flow_data_index, args->pConfig);
+                                          service_mod.flow_data_index, args->pConfig, NULL);
             return SERVICE_NOMATCH;
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
@@ -1105,7 +1105,7 @@ static int krb_server_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+    service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 }
