@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -682,17 +682,17 @@ static int service_validate(ServiceValidationArgs* args)
     id = csdPatternTreeSearch(data, size, flowp->proto, pkt, &service, false, args->pConfig);
     if (!id) goto fail;
 
-    pattern_service_mod.api->add_service(flowp, pkt, dir, &svc_element, id, NULL, NULL, NULL);
+    pattern_service_mod.api->add_service(flowp, pkt, dir, &svc_element, id, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 
 inprocess:
-    pattern_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+    pattern_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 fail:
     pattern_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
                                           pattern_service_mod.flow_data_index,
-                                          args->pConfig);
+                                          args->pConfig, NULL);
     return SERVICE_NOMATCH;
 }
 
