@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -908,7 +908,7 @@ static int pop3_validate(ServiceValidationArgs* args)
             service_mod.api->add_service_consume_subtype(flowp, pkt, dir, &svc_element,
                                                          dd->client.state == POP3_CLIENT_STATE_STLS_CMD ? APP_ID_POP3S : APP_ID_POP3,
                                                          pd->vendor,
-                                                         pd->version[0] ? pd->version:NULL, pd->subtype);
+                                                         pd->version[0] ? pd->version:NULL, pd->subtype, NULL);
             pd->subtype = NULL;
             return SERVICE_SUCCESS;
         }
@@ -916,7 +916,7 @@ static int pop3_validate(ServiceValidationArgs* args)
     else if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
     {
         service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
-                                      service_mod.flow_data_index, args->pConfig);
+                                      service_mod.flow_data_index, args->pConfig, NULL);
         return SERVICE_NOMATCH;
     }
     else
@@ -926,7 +926,7 @@ static int pop3_validate(ServiceValidationArgs* args)
     }
 
 inprocess:;
-    service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+    service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 }
 
