@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -216,12 +216,12 @@ static CLIENT_APP_RETCODE sip_client_init(const InitClientAppAPI * const init_ap
         }
     }
 
-	unsigned j;
-	for (j=0; j < sizeof(appIdClientRegistry)/sizeof(*appIdClientRegistry); j++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdClientRegistry[j].appId);
-		init_api->RegisterAppId(&sip_client_validate, appIdClientRegistry[j].appId, appIdClientRegistry[j].additionalInfo, init_api->pAppidConfig);
-	}
+    unsigned j;
+    for (j=0; j < sizeof(appIdClientRegistry)/sizeof(*appIdClientRegistry); j++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdClientRegistry[j].appId);
+        init_api->RegisterAppId(&sip_client_validate, appIdClientRegistry[j].appId, appIdClientRegistry[j].additionalInfo, init_api->pAppidConfig);
+    }
 
     if (init_api->pAppidConfig->detectorSipConfig.sipUaMatcher)
     {
@@ -276,12 +276,12 @@ static CLIENT_APP_RETCODE sip_tcp_client_init(const InitClientAppAPI * const ini
         }
     }
 
-	unsigned j;
-	for (j=0; j < sizeof(appIdClientRegistry)/sizeof(*appIdClientRegistry); j++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdClientRegistry[j].appId);
-		init_api->RegisterAppId(&sip_tcp_client_validate, appIdClientRegistry[j].appId, appIdClientRegistry[j].additionalInfo, init_api->pAppidConfig);
-	}
+    unsigned j;
+    for (j=0; j < sizeof(appIdClientRegistry)/sizeof(*appIdClientRegistry); j++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdClientRegistry[j].appId);
+        init_api->RegisterAppId(&sip_tcp_client_validate, appIdClientRegistry[j].appId, appIdClientRegistry[j].additionalInfo, init_api->pAppidConfig);
+    }
 
     return CLIENT_APP_SUCCESS;
 }
@@ -519,7 +519,7 @@ static void createRtpFlow(tAppIdData *flowp, SFSnortPacket *pkt, sfaddr_t *cliIp
         fp->clientAppId = flowp->clientAppId;
         fp->payloadAppId = flowp->payloadAppId;
         fp->serviceAppId = APP_ID_RTP;
-        PopulateExpectedFlow(flowp, fp, APPID_SESSION_IGNORE_ID_FLAGS);
+        PopulateExpectedFlow(flowp, fp, APPID_SESSION_EXPECTED_EVALUATE, APP_ID_APPID_SESSION_DIRECTION_MAX);
     }
 
     // create an RTCP flow as well
@@ -530,41 +530,41 @@ static void createRtpFlow(tAppIdData *flowp, SFSnortPacket *pkt, sfaddr_t *cliIp
         fp2->clientAppId = flowp->clientAppId;
         fp2->payloadAppId = flowp->payloadAppId;
         fp2->serviceAppId = APP_ID_RTCP;
-        PopulateExpectedFlow(flowp, fp2, APPID_SESSION_IGNORE_ID_FLAGS);
+        PopulateExpectedFlow(flowp, fp2, APPID_SESSION_EXPECTED_EVALUATE, APP_ID_APPID_SESSION_DIRECTION_MAX);
     }
 }
 
 static int addFutureRtpFlows( tAppIdData *flowp, const SipDialog *dialog, SFSnortPacket *p)
 {
-	SIP_MediaData *mdataA,*mdataB;
+    SIP_MediaData *mdataA,*mdataB;
 
-	// check the first media session
-	if (NULL == dialog->mediaSessions)
-		return -1;
-	// check the second media session
-	if (NULL == dialog->mediaSessions->nextS)
-		return -1;
+    // check the first media session
+    if (NULL == dialog->mediaSessions)
+        return -1;
+    // check the second media session
+    if (NULL == dialog->mediaSessions->nextS)
+        return -1;
 
-	DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future media sessions ID: %u and %u\n",
-			dialog->mediaSessions->sessionID, dialog->mediaSessions->nextS->sessionID););
-	mdataA = dialog->mediaSessions->medias;
-	mdataB = dialog->mediaSessions->nextS->medias;
-	while((NULL != mdataA)&&(NULL != mdataB))
+    DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future media sessions ID: %u and %u\n",
+            dialog->mediaSessions->sessionID, dialog->mediaSessions->nextS->sessionID););
+    mdataA = dialog->mediaSessions->medias;
+    mdataB = dialog->mediaSessions->nextS->medias;
+    while((NULL != mdataA)&&(NULL != mdataB))
     {
-    	DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future channels Source IP: %s Port: %u\n",
-    			sfip_to_str(&mdataA->maddress), mdataA->mport););
-    	DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future channels Destine IP: %s Port: %u\n",
-    			sfip_to_str(&mdataB->maddress), mdataB->mport););
+        DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future channels Source IP: %s Port: %u\n",
+                sfip_to_str(&mdataA->maddress), mdataA->mport););
+        DEBUG_WRAP(DebugMessage(DEBUG_SIP, "Adding future channels Destine IP: %s Port: %u\n",
+                sfip_to_str(&mdataB->maddress), mdataB->mport););
 
-    	createRtpFlow(flowp, p, &mdataA->maddress, mdataA->mport, &mdataB->maddress,
-    	                            mdataB->mport, IPPROTO_UDP, APP_ID_RTP);
+        createRtpFlow(flowp, p, &mdataA->maddress, mdataA->mport, &mdataB->maddress,
+                                    mdataB->mport, IPPROTO_UDP, APP_ID_RTP);
         createRtpFlow(flowp, p, &mdataB->maddress, mdataB->mport, &mdataA->maddress,
                                     mdataA->mport, IPPROTO_UDP, APP_ID_RTP);
         
-    	mdataA = mdataA->nextM;
-    	mdataB = mdataB->nextM;
+        mdataA = mdataA->nextM;
+        mdataB = mdataB->nextM;
     }
-	return 0;
+    return 0;
 
 }
 
@@ -691,7 +691,7 @@ static void SipSessionCbServiceProcess (SFSnortPacket *p, const SipHeaders *head
             setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
             sip_service_mod.api->add_service(flowp, p, direction, &svc_element,
                     APP_ID_SIP, ss->vendor[0] ? ss->vendor:NULL,
-                    NULL, NULL);
+                    NULL, NULL, NULL);
         }
     }
 }
@@ -750,12 +750,12 @@ static int sip_service_init(const InitServiceAPI * const init_api)
     init_api->RegisterPattern(&sip_service_validate, IPPROTO_TCP, (const uint8_t *) SIP_BYE_BANNER, SIP_BYE_BANNER_LEN, 0, svc_name, init_api->pAppidConfig);
     init_api->RegisterPattern(&sip_service_validate, IPPROTO_UDP, (const uint8_t *) SIP_OPTIONS_BANNER, SIP_OPTIONS_BANNER_LEN, 0, svc_name, init_api->pAppidConfig);
     init_api->RegisterPattern(&sip_service_validate, IPPROTO_TCP, (const uint8_t *) SIP_OPTIONS_BANNER, SIP_OPTIONS_BANNER_LEN, 0, svc_name, init_api->pAppidConfig);
-	unsigned i;
-	for (i=0; i < sizeof(appIdServiceRegistry)/sizeof(*appIdServiceRegistry); i++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdServiceRegistry[i].appId);
-		init_api->RegisterAppId(&sip_service_validate, appIdServiceRegistry[i].appId, appIdServiceRegistry[i].additionalInfo, init_api->pAppidConfig);
-	}
+    unsigned i;
+    for (i=0; i < sizeof(appIdServiceRegistry)/sizeof(*appIdServiceRegistry); i++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdServiceRegistry[i].appId);
+        init_api->RegisterAppId(&sip_service_validate, appIdServiceRegistry[i].appId, appIdServiceRegistry[i].additionalInfo, init_api->pAppidConfig);
+    }
 
     return 0;
 }
@@ -788,7 +788,7 @@ static int sip_service_validate(ServiceValidationArgs* args)
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
             sip_service_mod.api->fail_service(flowp, args->pkt, args->dir, &svc_element,
-                                              sip_service_mod.flow_data_index, args->pConfig);
+                                              sip_service_mod.flow_data_index, args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOMATCH;
@@ -796,7 +796,7 @@ static int sip_service_validate(ServiceValidationArgs* args)
 
     if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
     {
-        sip_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element);
+        sip_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element, NULL);
     }
 
     return SERVICE_INPROCESS;
