@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -47,7 +47,9 @@ typedef enum
     SMTP_CLIENT_STATE_GET_PRODUCT_VERSION,
     SMTP_CLIENT_STATE_SKIP_LINE,
     SMTP_CLIENT_STATE_CONNECTION_ERROR,
-    SMTP_CLIENT_STATE_STARTTLS
+    SMTP_CLIENT_STATE_STARTTLS,
+    SMTP_CLIENT_STATE_LOGIN_USER,
+    SMTP_CLIENT_STATE_LOGIN_PASSWORD
 } SMTPClientState;
 
 #define MAX_HEADER_LINE_SIZE 1024
@@ -119,7 +121,9 @@ typedef struct {
 #define RCPTTO "RCPT TO:"
 #define DATA "DATA"
 #define RSET "RSET"
-#define AUTH "AUTH PLAIN"
+#define AUTH "AUTH "
+#define AUTH_PLAIN "AUTH PLAIN"
+#define AUTH_LOGIN "AUTH LOGIN"
 #define STARTTLS "STARTTLS"
 
 #define STARTTLS_COMMAND_SUCCESS "220 "
@@ -225,7 +229,6 @@ static CLIENT_APP_RETCODE smtp_ca_init(const InitClientAppAPI * const init_api,
 }
 
 #define SMTP_PORT   25
-#define SMTPS_DEPRECATED_PORT   465
 #define SMTP_CLOSING_CONN "closing connection\x0d\x0a"
 
 typedef enum
@@ -272,7 +275,6 @@ static tRNAServiceElement svc_element =
 static RNAServiceValidationPort pp[] =
 {
     {&smtp_svc_validate, SMTP_PORT, IPPROTO_TCP},
-    {&smtp_svc_validate, SMTPS_DEPRECATED_PORT, IPPROTO_TCP}, // here only to support old servers
     {NULL, 0, 0}
 };
 
@@ -290,8 +292,8 @@ tRNAServiceValidationModule smtp_service_mod =
 
 static tAppRegistryEntry appIdRegistry[] =
 {
-    {APP_ID_SMTP,  0},
-    {APP_ID_SMTPS, 0}
+    {APP_ID_SMTP,  APPINFO_FLAG_SERVICE_ADDITIONAL},
+    {APP_ID_SMTPS, APPINFO_FLAG_SERVICE_ADDITIONAL}
 };
 
 typedef struct _SMTP_DETECTOR_DATA
@@ -299,7 +301,6 @@ typedef struct _SMTP_DETECTOR_DATA
     ClientSMTPData client;
     ServiceSMTPData server;
     int need_continue;
-    int watch_for_deprecated_port;
 } SMTPDetectorData;
 
 SF_SO_PUBLIC RNADetectorValidationModule smtp_detector_mod =
@@ -564,7 +565,6 @@ static inline SMTPDetectorData *smtp_get_SMTPDetectorData(tAppIdData *flowp)
     dd->server.state = SMTP_SERVICE_STATE_CONNECTION;
     dd->client.state = SMTP_CLIENT_STATE_HELO;
     dd->need_continue = 1;
-    dd->watch_for_deprecated_port = 1;
     setAppIdFlag(flowp, APPID_SESSION_CLIENT_GETS_SERVER_PACKETS);
     return dd;
 }
@@ -578,6 +578,7 @@ static CLIENT_APP_RETCODE smtp_ca_validate(const uint8_t *data, uint16_t size, c
     ClientSMTPData *fd;
     const uint8_t *end;
     unsigned len;
+    int line_break = 0;
 #ifdef UNIT_TESTING
     SMTPClientState currState = SMTP_CLIENT_STATE_NONE;
 
@@ -635,12 +636,14 @@ static CLIENT_APP_RETCODE smtp_ca_validate(const uint8_t *data, uint16_t size, c
                 data += (sizeof(HELO)-1)-1;
                 fd->nextstate = SMTP_CLIENT_STATE_MAIL_FROM;
                 fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+                fd->flags &= ~ CLIENT_FLAG_STARTTLS_SUCCESS;
             }
             else if (len >= (sizeof(EHLO)-1) && strncasecmp((const char *)data, EHLO, sizeof(EHLO)-1) == 0)
             {
                 data += (sizeof(EHLO)-1)-1;
                 fd->nextstate = SMTP_CLIENT_STATE_MAIL_FROM;
                 fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+                fd->flags &= ~ CLIENT_FLAG_STARTTLS_SUCCESS;
             }
             else goto done;
             break;
@@ -658,6 +661,18 @@ static CLIENT_APP_RETCODE smtp_ca_validate(const uint8_t *data, uint16_t size, c
                 fd->nextstate = fd->state;
                 fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
             }
+            else if (len >= (sizeof(AUTH_PLAIN)-1) && strncasecmp((const char *)data, AUTH_PLAIN, sizeof(AUTH_PLAIN)-1) == 0)
+            {
+                data += (sizeof(AUTH_PLAIN)-1)-1;
+                fd->nextstate = fd->state;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+            }
+            else if (len >= (sizeof(AUTH_LOGIN)-1) && strncasecmp((const char *)data, AUTH_LOGIN, sizeof(AUTH_LOGIN)-1) == 0)
+            {
+                data += (sizeof(AUTH_LOGIN)-1)-1;
+                fd->nextstate = SMTP_CLIENT_STATE_LOGIN_USER;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+            }
             else if (len >= (sizeof(AUTH)-1) && strncasecmp((const char *)data, AUTH, sizeof(AUTH)-1) == 0)
             {
                 data += (sizeof(AUTH)-1)-1;
@@ -671,8 +686,35 @@ static CLIENT_APP_RETCODE smtp_ca_validate(const uint8_t *data, uint16_t size, c
                 fd->nextstate = fd->state;
                 fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
             }
+            /* check for state reversion */
+            else if (len >= (sizeof(HELO)-1) && strncasecmp((const char *)data, HELO, sizeof(HELO)-1) == 0)
+            {
+                data += (sizeof(HELO)-1)-1;
+                fd->nextstate = fd->state;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+                dd->server.state = SMTP_SERVICE_STATE_HELO; // make sure that service side expects the 250
+            }
+            else if (len >= (sizeof(EHLO)-1) && strncasecmp((const char *)data, EHLO, sizeof(EHLO)-1) == 0)
+            {
+                data += (sizeof(EHLO)-1)-1;
+                fd->nextstate = fd->state;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+                dd->server.state = SMTP_SERVICE_STATE_HELO; // make sure that service side expects the 250
+            }
             else goto done;
             break;
+        case SMTP_CLIENT_STATE_LOGIN_USER:
+            {
+                fd->nextstate = SMTP_CLIENT_STATE_LOGIN_PASSWORD;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+            }
+            break;
+        case SMTP_CLIENT_STATE_LOGIN_PASSWORD:
+            {
+                fd->nextstate = SMTP_CLIENT_STATE_MAIL_FROM;
+                fd->state = SMTP_CLIENT_STATE_SKIP_LINE;
+            }
+            break;
         case SMTP_CLIENT_STATE_RCPT_TO:
             if (len >= (sizeof(RCPTTO)-1) && strncasecmp((const char *)data, RCPTTO, sizeof(RCPTTO)-1) == 0)
             {
@@ -729,35 +771,56 @@ static CLIENT_APP_RETCODE smtp_ca_validate(const uint8_t *data, uint16_t size, c
             break;
 
         case SMTP_CLIENT_STATE_GET_PRODUCT_VERSION:
-            if (*data == '\r')
+            if (!fd->headerline)
             {
-                if (fd->headerline && fd->pos)
+                if (!(fd->headerline = malloc(MAX_HEADER_LINE_SIZE)))
+                    goto done;
+            }
+            while((data < end) && (fd->pos < (MAX_HEADER_LINE_SIZE-1)))
+            {
+                if ((*data == ' ') || (*data == '\t'))
+                {
+                    line_break = 0;
+                    fd->headerline[fd->pos++] = *data;
+                }
+                else if((*data == '\n') && (line_break != 1))
+                {
+                    /* Can't have multiple LFs in a row, but if we get one it
+                     * needs to be followed by at least one space */
+                    line_break = 1;
+                }
+                else if(*data == '\r')
+                {
+                    // CR needs to be followed by LF and can't start a line
+                    line_break = 2;
+                }
+                else if (!isprint(*data))
                 {
-                    ExtractVersion(fd, fd->headerline, fd->headerline + fd->pos, flowp, pkt);
                     free(fd->headerline);
                     fd->headerline = NULL;
                     fd->pos = 0;
+                    goto done;
                 }
-                goto done;
+                else if(!line_break)
+                {
+                    fd->headerline[fd->pos++] = *data;
+                }
+                else
+                {
+                     // We have reached the end of the header
+                     break;
+                }
+                data++;
             }
-            else if (!isprint(*data))
+            data--;
+            if (line_break || fd->pos >= (MAX_HEADER_LINE_SIZE-1))
             {
+                ExtractVersion(fd, fd->headerline, fd->headerline + fd->pos, flowp, pkt);
                 free(fd->headerline);
                 fd->headerline = NULL;
                 fd->pos = 0;
                 goto done;
             }
-            else
-            {
-                if (!fd->headerline)
-                {
-                    if (!(fd->headerline = malloc(MAX_HEADER_LINE_SIZE)))
-                        goto done;
-                }
-
-                if (fd->pos < (MAX_HEADER_LINE_SIZE-1))
-                    fd->headerline[fd->pos++] = *data;
-            }
             break;
 
         case SMTP_CLIENT_STATE_SKIP_LINE:
@@ -903,7 +966,7 @@ static int smtp_svc_validate(ServiceValidationArgs* args)
         {
             // client side gave up because everything is encrypted.
             smtp_service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                       APP_ID_SMTPS, NULL, NULL, NULL);
+                                       APP_ID_SMTPS, NULL, NULL, NULL, NULL);
             return SERVICE_SUCCESS;
         }
         else if (getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
@@ -918,17 +981,6 @@ static int smtp_svc_validate(ServiceValidationArgs* args)
     {
         goto inprocess; // allow client validator to have it's shot.
     }
-    if (dd->watch_for_deprecated_port)
-    {
-        dd->watch_for_deprecated_port = 0;
-        // If we have caught a response on port 465 that isn't encrypted+decrypted it isn't SMTP at all.
-        // The new IANA assignment for this port is non-SSL and NOT SMTP. Only an old server will survive the test.
-        if (args->pkt->src_port == SMTPS_DEPRECATED_PORT && !getAppIdFlag(flowp, APPID_SESSION_DECRYPTED))
-        {
-            // This is not an SMTPS port because we have not ALREADY gone through the SSL handshake
-            goto fail;
-        }
-    }
 
     fd = &dd->server;
 
@@ -954,6 +1006,7 @@ static int smtp_svc_validate(ServiceValidationArgs* args)
                 if (service_strstr(data, size, (const uint8_t *)SMTP_CLOSING_CONN, sizeof(SMTP_CLOSING_CONN)-1))
                     goto success;
             case 520:
+            case 554:
                 fd->state = SMTP_SERVICE_STATE_CONNECTION_ERROR;
                 break;
             default:
@@ -966,6 +1019,7 @@ static int smtp_svc_validate(ServiceValidationArgs* args)
             case 250:
                 fd->state = SMTP_SERVICE_STATE_TRANSFER;
                 break;
+            case 220:
             case 500:
             case 501:
             case 504:
@@ -1001,7 +1055,7 @@ static int smtp_svc_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    smtp_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element);
+    smtp_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 success:
@@ -1009,11 +1063,11 @@ success:
        setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
 
     smtp_service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                      APP_ID_SMTP, NULL, NULL, NULL);
+                                      APP_ID_SMTP, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 
 fail:
     smtp_service_mod.api->fail_service(flowp, args->pkt, args->dir, &svc_element,
-                                       smtp_service_mod.flow_data_index, args->pConfig);
+                                       smtp_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOMATCH;
 }
