@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -232,10 +232,9 @@ typedef struct AppIdData
     char *serviceVendor;
     char *serviceVersion;
     struct _RNAServiceSubtype *subtype;
-    AppIdServiceIDState *id_state;
+    FLOW_SERVICE_ID_STATE search_state;
     char *netbios_name;
     SF_LIST * candidate_service_list;
-    unsigned int num_candidate_services_tried;
     int got_incompatible_services;
 
     /**AppId matching client side */
@@ -324,6 +323,8 @@ typedef struct AppIdData
 
     bool is_http2;
     SEARCH_SUPPORT_TYPE search_support_type;
+
+    uint16_t hostCacheVersion;
 } tAppIdData;
 /**
  * Mark a flow with a particular flag
