@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,31 @@
 
 /*#define  RNA_FULL_CLEANUP   1 */
 
+typedef enum
+{
+    APPID_DEBUG_HOST_MONITOR0,
+    APPID_DEBUG_HOST_MONITOR1,
+    APPID_DEBUG_HOST_MONITOR2,
+    APPID_DEBUG_HOST_MONITOR3,
+    APPID_DEBUG_HOST_MONITOR4,
+    APPID_DEBUG_HOST_MONITOR5,
+    APPID_DEBUG_HOST_NOT_MONITORED,
+} AppIdDebugHostMonitorType;
+
+typedef struct
+{
+    struct in6_addr initiatorIp;
+    int family;
+    tAppIdData* session;
+    uint16_t initiatorPort;
+    APPID_SESSION_DIRECTION direction;
+    uint8_t protocol;
+    int monitorType;
+} AppIdDebugHostInfo_t;
+
+
 extern uint8_t  appIdPriorityArray[SF_APPID_MAX+1];
+extern AppIdDebugHostInfo_t AppIdDebugHostInfo;
 
 struct AppIdData * getAppIdData(void* lwssn);
 
@@ -70,7 +94,7 @@ void AppIdAddDnsResponseInfo(tAppIdData *flow,
 void AppIdResetDnsInfo(tAppIdData *flow);
 
 void AppIdAddPayload(tAppIdData *flow, tAppId payload_id);
-tAppIdData* appSharedDataAlloc(uint8_t proto, struct in6_addr *ip, uint16_t initiator_port);
+tAppIdData* appSharedDataAlloc(uint8_t proto, const struct in6_addr *ip, uint16_t initiator_port);
 tAppId getOpenAppId(void *ssnptr);
 
 void appSetServiceValidator(RNAServiceValidationFCN fcn, tAppId appId, unsigned extractsInfo, tAppIdConfig *pConfig);
@@ -166,6 +190,16 @@ static inline int TPIsAppIdAvailable(void * tpSession)
     return true;
 }
 
+static inline int isTPProcessingDone(tAppIdData *flow)
+{
+    if (thirdparty_appid_module &&
+	!getAppIdFlag(flow, APPID_SESSION_NO_TPI) &&
+        (!TPIsAppIdDone(flow->tpsession) ||
+        getAppIdFlag(flow, APPID_SESSION_APP_REINSPECT | APPID_SESSION_APP_REINSPECT_SSL)))
+        return 0;
+    else
+        return 1;
+}
 static inline tAppId isAppDetectionDone(tAppIdData *flow)
 {
     return getAppIdFlag(flow, APPID_SESSION_SERVICE_DETECTED);
@@ -278,7 +312,7 @@ static inline tAppId fwPickServiceAppId(tAppIdData *session)
 {
     tAppId appId;
     appId = pickServiceAppId(session);
-    if (appId == APP_ID_NONE)
+    if (appId == APP_ID_NONE || appId== APP_ID_UNKNOWN_UI)
         appId = session->encrypted.serviceAppId;
     return appId;
 }
