@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -43,13 +43,21 @@ typedef struct _tHostPortKey {
 
 typedef struct _tHostPortVal {
     tAppId appId;
-    unsigned type;
+    APP_ID_TYPE type;
 } tHostPortVal;
 
+void hostPortAppCacheDynamicInit();
+void hostPortAppCacheDynamicFini();
+tHostPortVal *hostPortAppCacheDynamicFind(const sfaddr_t *ip, uint16_t port, uint16_t proto);
+int hostPortAppCacheDynamicAdd(const struct in6_addr *ip, uint16_t port, uint16_t proto, APP_ID_TYPE type, tAppId appId);
+void hostPortAppCacheDynamicDump();
+
 void hostPortAppCacheInit(struct appIdConfig_ *pConfig);
 void hostPortAppCacheFini(struct appIdConfig_ *pConfig);
 tHostPortVal *hostPortAppCacheFind(const sfaddr_t *ip, uint16_t port, uint16_t proto, const struct appIdConfig_ *pConfig);
-int hostPortAppCacheAdd(const struct in6_addr *ip, uint16_t port, uint16_t proto, unsigned type, tAppId appId, struct appIdConfig_ *pConfig);
+int hostPortAppCacheAdd(const struct in6_addr *ip, uint16_t port, uint16_t proto, APP_ID_TYPE type, tAppId appId, struct appIdConfig_ *pConfig);
 void hostPortAppCacheDump(const struct appIdConfig_ *pConfig);
 
+void updateHostCacheVersion(uint16_t *session_version);
+bool isHostCacheUpdated(uint16_t session_version);
 #endif
