@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 
 #include <sys/types.h>
 #include <inttypes.h>
+#include <pthread.h>
 #include "client_app_base.h"
 #include "service_base.h"
 
@@ -131,6 +132,8 @@ typedef struct _Detector
     struct _PreprocStats *pPerfStats;
 #endif
 
+    pthread_mutex_t luaReloadMutex;
+
 } Detector;
 
 /**data directly accessed by Lua code should be here.
