@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -138,17 +138,17 @@ typedef void (*ServiceAnalyzeFP)(tAppIdData *, unsigned, unsigned, uint32_t);
 typedef int (*AddService)(tAppIdData *flow, const SFSnortPacket *pkt, int dir,
                           const tRNAServiceElement *svc_element,
                           tAppId service, const char *vendor, const char *version,
-                          const RNAServiceSubtype *subtype);
+                          const RNAServiceSubtype *subtype, AppIdServiceIDState *id_state);
 typedef int (*AddServiceConsumeSubtype)(tAppIdData *flow, const SFSnortPacket *pkt, int dir,
                                         const tRNAServiceElement *svc_element,
                                         tAppId service, const char *vendor, const char *version,
-                                        RNAServiceSubtype *subtype);
+                                        RNAServiceSubtype *subtype, AppIdServiceIDState *id_state);
 typedef int (*ServiceInProcess)(tAppIdData *flow, const SFSnortPacket *pkt, int dir,
-                                const tRNAServiceElement *svc_element);
+                                const tRNAServiceElement *svc_element, AppIdServiceIDState *id_state);
 typedef int (*FailService)(tAppIdData *flow, const SFSnortPacket *pkt, int dir,
-                           const tRNAServiceElement *svc_element, unsigned flow_data_index, const struct appIdConfig_ *pConfig);
+                           const tRNAServiceElement *svc_element, unsigned flow_data_index, const struct appIdConfig_ *pConfig, AppIdServiceIDState *id_state);
 typedef int (*IncompatibleData)(tAppIdData *flow, const SFSnortPacket *pkt, int dir,
-                                const tRNAServiceElement *svc_element, unsigned flow_data_index, const struct appIdConfig_ *pConfig);
+                                const tRNAServiceElement *svc_element, unsigned flow_data_index, const struct appIdConfig_ *pConfig, AppIdServiceIDState *id_state);
 typedef void (*AddHostInfo)(tAppIdData *flow, SERVICE_HOST_INFO_CODE code, const void *info);
 typedef void (*AddPayload)(tAppIdData *, tAppId);
 typedef void (*AddUser)(tAppIdData *, const char *, tAppId, int);
