@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -57,7 +57,7 @@ void ServiceRegisterPatternDetector(RNAServiceValidationFCN fcn,
                                     u_int8_t proto, const u_int8_t *pattern, unsigned size,
                                     int position, struct _Detector *userdata,
                                     const char *name);
-int AppIdDiscoverService(SFSnortPacket *p, int direction, tAppIdData *rnaData, const tAppIdConfig *pConfig);
+int AppIdDiscoverService(SFSnortPacket *p, APPID_SESSION_DIRECTION direction, tAppIdData *rnaData, const tAppIdConfig *pConfig);
 tAppId getPortServiceId(uint8_t proto, uint16_t port, const tAppIdConfig *pConfig);
 
 void AppIdFreeServiceIDState(AppIdServiceIDState *id_state);
@@ -65,17 +65,17 @@ void AppIdFreeServiceIDState(AppIdServiceIDState *id_state);
 int AppIdServiceAddService(tAppIdData*flow, const SFSnortPacket *pkt, int dir,
                            const tRNAServiceElement *svc_element,
                            tAppId appId, const char *vendor, const char *version,
-                           const RNAServiceSubtype *subtype);
+                           const RNAServiceSubtype *subtype, AppIdServiceIDState *id_state);
 int AppIdServiceAddServiceSubtype(tAppIdData*flow, const SFSnortPacket *pkt, int dir,
                                   const tRNAServiceElement *svc_element,
                                   tAppId appId, const char *vendor, const char *version,
-                                  RNAServiceSubtype *subtype);
+                                  RNAServiceSubtype *subtype, AppIdServiceIDState *id_state);
 int AppIdServiceInProcess(tAppIdData*flow, const SFSnortPacket *pkt, int dir,
-                          const tRNAServiceElement *svc_element);
+                          const tRNAServiceElement *svc_element, AppIdServiceIDState *id_state);
 int AppIdServiceIncompatibleData(tAppIdData*flow, const SFSnortPacket *pkt, int dir,
-                                 const tRNAServiceElement *svc_element, unsigned flow_data_index, const tAppIdConfig *pConfig);
+                                 const tRNAServiceElement *svc_element, unsigned flow_data_index, const tAppIdConfig *pConfig, AppIdServiceIDState *id_state);
 int AppIdServiceFailService(tAppIdData*flow, const SFSnortPacket *pkt, int dir,
-                            const tRNAServiceElement *svc_element, unsigned flow_data_index, const tAppIdConfig *pConfig);
+                            const tRNAServiceElement *svc_element, unsigned flow_data_index, const tAppIdConfig *pConfig, AppIdServiceIDState *id_state);
 int AddFTPServiceState(tAppIdData *fp);
 void AppIdFreeDhcpInfo(DHCPInfo *dd);
 void AppIdFreeSMBData(FpSMBData *sd);
@@ -100,23 +100,47 @@ static inline bool compareServiceElements(const tRNAServiceElement *first, const
     return (first->validate != second->validate || first->userdata != second->userdata);
 }
 
+// change UNIT_TESTING and UNIT_TEST_FIRST_DECRYPTED_PACKET in appIdApi.h
 static inline uint32_t AppIdServiceDetectionLevel(tAppIdData * session)
 {
     if (getAppIdFlag(session, APPID_SESSION_DECRYPTED)) return 1;
+#ifdef UNIT_TESTING
+    if (session->session_packet_count >= UNIT_TEST_FIRST_DECRYPTED_PACKET)
+        return 1;
+#endif
     return 0;
 }
 
-static inline void PopulateExpectedFlow(tAppIdData* parent, tAppIdData* expected, uint64_t flags)
+static inline void PopulateExpectedFlow(tAppIdData* parent, tAppIdData* expected, uint64_t flags, APPID_SESSION_DIRECTION dir)
 {
-    setAppIdFlag(expected, flags |
-                 getAppIdFlag(parent,
-                              APPID_SESSION_RESPONDER_MONITORED |
-                              APPID_SESSION_INITIATOR_MONITORED |
-                              APPID_SESSION_SPECIAL_MONITORED |
-                              APPID_SESSION_RESPONDER_CHECKED |
-                              APPID_SESSION_INITIATOR_CHECKED |
-                              APPID_SESSION_DISCOVER_APP |
-                              APPID_SESSION_DISCOVER_USER));
+    if (dir == APP_ID_FROM_INITIATOR)
+    {
+        setAppIdFlag(expected, flags |
+                     getAppIdFlag(parent,
+                                  APPID_SESSION_RESPONDER_MONITORED |
+                                  APPID_SESSION_INITIATOR_MONITORED |
+                                  APPID_SESSION_SPECIAL_MONITORED |
+                                  APPID_SESSION_RESPONDER_CHECKED |
+                                  APPID_SESSION_INITIATOR_CHECKED |
+                                  APPID_SESSION_DISCOVER_APP |
+                                  APPID_SESSION_DISCOVER_USER));
+    }
+    else if (dir == APP_ID_FROM_RESPONDER)
+    {
+        if (getAppIdFlag(parent, APPID_SESSION_INITIATOR_MONITORED))
+           flags |= APPID_SESSION_RESPONDER_MONITORED;
+        if (getAppIdFlag(parent, APPID_SESSION_INITIATOR_CHECKED))
+           flags |= APPID_SESSION_RESPONDER_CHECKED;
+        if (getAppIdFlag(parent, APPID_SESSION_RESPONDER_MONITORED))
+           flags |= APPID_SESSION_INITIATOR_MONITORED;
+        if (getAppIdFlag(parent, APPID_SESSION_RESPONDER_CHECKED))
+           flags |= APPID_SESSION_INITIATOR_CHECKED;
+        setAppIdFlag(expected, flags |
+                     getAppIdFlag(parent, 
+                                  APPID_SESSION_SPECIAL_MONITORED |
+                                  APPID_SESSION_DISCOVER_APP |
+                                  APPID_SESSION_DISCOVER_USER));
+    }
     expected->rnaServiceState = RNA_STATE_FINISHED;
     expected->rnaClientState = RNA_STATE_FINISHED;
 }
