@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -233,17 +233,17 @@ static int bgp_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    bgp_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element);
+    bgp_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 fail:
     bgp_service_mod.api->fail_service(flowp, args->pkt, args->dir, &svc_element,
-                                      bgp_service_mod.flow_data_index, args->pConfig);
+                                      bgp_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOMATCH;
 
 success:
     bgp_service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                     APP_ID_BGP, NULL, NULL, NULL);
+                                     APP_ID_BGP, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 }
 
