@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -131,12 +131,12 @@ static int ftp_init(const InitServiceAPI * const init_api)
     init_api->RegisterPattern(&ftp_validate, IPPROTO_TCP, (uint8_t *)FTP_PATTERN2, sizeof(FTP_PATTERN2)-1, 0, "ftp", init_api->pAppidConfig);
     init_api->RegisterPattern(&ftp_validate, IPPROTO_TCP, (uint8_t *)FTP_PATTERN3, sizeof(FTP_PATTERN3)-1, -1, "ftp", init_api->pAppidConfig);
     init_api->RegisterPattern(&ftp_validate, IPPROTO_TCP, (uint8_t *)FTP_PATTERN4, sizeof(FTP_PATTERN4)-1, -1, "ftp", init_api->pAppidConfig);
-	unsigned i;
-	for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
-		init_api->RegisterAppId(&ftp_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
-	}
+    unsigned i;
+    for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
+        init_api->RegisterAppId(&ftp_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
+    }
 
     return 0;
 }
@@ -744,19 +744,26 @@ static inline void WatchForCommandResult(ServiceFTPData *fd, tAppIdData *flowp,
     fd->cmd = command;
 }
 
-static inline void InitializeDataSession(tAppIdData *flowp,tAppIdData *fp)
+static inline void CreateExpectedSession(tAppIdData *flowp, SFSnortPacket *ctrlPkt, sfaddr_t *cliIp, uint16_t cliPort,
+                sfaddr_t *srvIp, uint16_t srvPort, uint8_t proto, int flags, APPID_SESSION_DIRECTION dir)
 {
-    unsigned encryptedFlag = getAppIdFlag(flowp, APPID_SESSION_ENCRYPTED | APPID_SESSION_DECRYPTED);
-    if (encryptedFlag == APPID_SESSION_ENCRYPTED)
+    tAppIdData *fp;
+
+    fp = ftp_service_mod.api->flow_new(flowp, ctrlPkt, cliIp, cliPort, srvIp, srvPort, proto, ftp_data_app_id, flags);
+    if (fp) // initialize data session
     {
-        fp->serviceAppId = APP_ID_FTPSDATA;
+        unsigned encryptedFlag = getAppIdFlag(flowp, APPID_SESSION_ENCRYPTED | APPID_SESSION_DECRYPTED);
+        if (encryptedFlag == APPID_SESSION_ENCRYPTED)
+        {
+            fp->serviceAppId = APP_ID_FTPSDATA;
+        }
+        else
+        {
+            encryptedFlag = 0; // change (APPID_SESSION_ENCRYPTED | APPID_SESSION_DECRYPTED) case to zeroes.
+            fp->serviceAppId = APP_ID_FTP_DATA;
+        }
+        PopulateExpectedFlow(flowp, fp, APPID_SESSION_IGNORE_ID_FLAGS | encryptedFlag, dir);
     }
-    else
-    {
-        encryptedFlag = 0; // change (APPID_SESSION_ENCRYPTED | APPID_SESSION_DECRYPTED) case to zeroes.
-        fp->serviceAppId = APP_ID_FTP_DATA;
-    }
-    PopulateExpectedFlow(flowp, fp, APPID_SESSION_IGNORE_ID_FLAGS | encryptedFlag);
 }
 
 static int ftp_validate(ServiceValidationArgs* args)
@@ -772,7 +779,6 @@ static int ftp_validate(ServiceValidationArgs* args)
     int code_index;
     uint32_t address;
     uint16_t port;
-    tAppIdData *fp;
     int retval = SERVICE_INPROCESS;
     tAppIdData *flowp = args->flowp;
     const uint8_t *data = args->data;
@@ -821,6 +827,8 @@ static int ftp_validate(ServiceValidationArgs* args)
                                   size-(sizeof(FTP_PORT_CMD)-1),
                                   &fd->address, &fd->port) == 0)
             {
+                CreateExpectedSession(flowp, pkt, GET_DST_IP(pkt), 0, &fd->address, fd->port, flowp->proto,
+                                      APPID_EARLY_SESSION_FLAG_FW_RULE, APP_ID_FROM_RESPONDER);
                 WatchForCommandResult(fd, flowp, FTP_CMD_PORT_EPRT);
             }
         }
@@ -831,6 +839,8 @@ static int ftp_validate(ServiceValidationArgs* args)
                                   size-(sizeof(FTP_EPRT_CMD)-1),
                                   &fd->address, &fd->port) == 0)
             {
+                CreateExpectedSession(flowp, pkt, GET_DST_IP(pkt), 0, &fd->address, fd->port, flowp->proto,
+                                      APPID_EARLY_SESSION_FLAG_FW_RULE, APP_ID_FROM_RESPONDER);
                 WatchForCommandResult(fd, flowp, FTP_CMD_PORT_EPRT);
             }
         }
@@ -1098,20 +1108,13 @@ static int ftp_validate(ServiceValidationArgs* args)
                         sip = GET_SRC_IP(pkt);
                         addr = htonl(address);
                         sfip_set_raw(&ip, &addr, AF_INET);
-                        fp = ftp_service_mod.api->flow_new(flowp, pkt, dip, 0, &ip, port, flowp->proto, ftp_data_app_id,
-                                                           APPID_EARLY_SESSION_FLAG_FW_RULE);
-                        if (fp)
-                        {
-                            InitializeDataSession(flowp,fp);
-                        }
+                        CreateExpectedSession(flowp, pkt, dip, 0, &ip, port, flowp->proto,
+                                              APPID_EARLY_SESSION_FLAG_FW_RULE, APP_ID_FROM_INITIATOR);
+
                         if (!sfip_fast_eq6(&ip, sip))
                         {
-                            fp = ftp_service_mod.api->flow_new(flowp, pkt, dip, 0, sip, port, flowp->proto, ftp_data_app_id,
-                                                               APPID_EARLY_SESSION_FLAG_FW_RULE);
-                            if (fp)
-                            {
-                                InitializeDataSession(flowp,fp);
-                            }
+                            CreateExpectedSession(flowp, pkt, dip, 0, sip, port, flowp->proto,
+                                                  APPID_EARLY_SESSION_FLAG_FW_RULE, APP_ID_FROM_INITIATOR);
                         }
                         ftp_service_mod.api->add_payload(flowp, APP_ID_FTP_PASSIVE); // Passive mode FTP is reported as a payload id
                     }
@@ -1134,12 +1137,8 @@ static int ftp_validate(ServiceValidationArgs* args)
 
                         dip = GET_DST_IP(pkt);
                         sip = GET_SRC_IP(pkt);
-                        fp = ftp_service_mod.api->flow_new(flowp, pkt, dip, 0, sip, port, flowp->proto, ftp_data_app_id,
-                                                           APPID_EARLY_SESSION_FLAG_FW_RULE);
-                        if (fp)
-                        {
-                            InitializeDataSession(flowp,fp);
-                        }
+                        CreateExpectedSession(flowp, pkt, dip, 0, sip, port, flowp->proto,
+                                              APPID_EARLY_SESSION_FLAG_FW_RULE, APP_ID_FROM_INITIATOR);
                         ftp_service_mod.api->add_payload(flowp, APP_ID_FTP_PASSIVE); // Passive mode FTP is reported as a payload id
                     }
                     else if (code < 0)
@@ -1151,15 +1150,7 @@ static int ftp_validate(ServiceValidationArgs* args)
             case 200:
                 if (fd->cmd == FTP_CMD_PORT_EPRT)
                 {
-                    sfaddr_t *sip;
-
-                    sip = GET_SRC_IP(pkt);
-                    fp = ftp_service_mod.api->flow_new(flowp, pkt, sip, 0, &fd->address, fd->port, flowp->proto, ftp_data_app_id,
-                                                       APPID_EARLY_SESSION_FLAG_FW_RULE);
-                    if (fp)
-                    {
-                        InitializeDataSession(flowp,fp);
-                    }
+                    // Expected session is created on PORT/EPRT command
                     ftp_service_mod.api->add_payload(flowp, APP_ID_FTP_ACTIVE); // Active mode FTP is reported as a payload id
                 }
                 break;
@@ -1181,7 +1172,7 @@ static int ftp_validate(ServiceValidationArgs* args)
 inprocess:
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
-            ftp_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+            ftp_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
         }
         return SERVICE_INPROCESS;
 
@@ -1193,7 +1184,7 @@ inprocess:
                                              encryptedFlag == APPID_SESSION_ENCRYPTED ? // FTPS only when encrypted==1 decrypted==0
                                                 APP_ID_FTPS : APP_ID_FTP_CONTROL,
                                              fd->vendor[0] ? fd->vendor:NULL,
-                                             fd->version[0] ? fd->version:NULL, NULL);
+                                             fd->version[0] ? fd->version:NULL, NULL, NULL);
         }
         return SERVICE_SUCCESS;
 
@@ -1202,7 +1193,7 @@ fail:
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
             ftp_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
-                                              ftp_service_mod.flow_data_index, args->pConfig);
+                                              ftp_service_mod.flow_data_index, args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOMATCH;
