@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -202,7 +202,8 @@ static int rexec_validate(ServiceValidationArgs* args)
                                      APPID_SESSION_NO_TPI |
                                      APPID_SESSION_SERVICE_DETECTED |
                                      APPID_SESSION_NOT_A_SERVICE |
-                                     APPID_SESSION_PORT_SERVICE_DONE);
+                                     APPID_SESSION_PORT_SERVICE_DONE,
+                                     APP_ID_FROM_RESPONDER);
                 pf->rnaServiceState = RNA_STATE_STATEFUL;
             }
             else
@@ -285,7 +286,7 @@ static int rexec_validate(ServiceValidationArgs* args)
 inprocess:
     if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
     {
-        rexec_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+        rexec_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     }
     return SERVICE_INPROCESS;
 
@@ -293,7 +294,7 @@ success:
     if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
     {
         rexec_service_mod.api->add_service(flowp, pkt, dir, &svc_element,
-                                           APP_ID_EXEC, NULL, NULL, NULL);
+                                           APP_ID_EXEC, NULL, NULL, NULL, NULL);
     }
     return SERVICE_SUCCESS;
 
@@ -302,7 +303,7 @@ bail:
     {
         rexec_service_mod.api->incompatible_data(flowp, pkt, dir, &svc_element,
                                                  rexec_service_mod.flow_data_index,
-                                                 args->pConfig);
+                                                 args->pConfig, NULL);
     }
     clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
     return SERVICE_NOT_COMPATIBLE;
@@ -312,7 +313,7 @@ fail:
     {
         rexec_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
                                             rexec_service_mod.flow_data_index,
-                                            args->pConfig);
+                                            args->pConfig, NULL);
     }
     clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
     return SERVICE_NOMATCH;
