@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -147,17 +147,17 @@ static int rlogin_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    rlogin_service_mod.api->service_inprocess(flowp, pkt, args->dir, &svc_element);
+    rlogin_service_mod.api->service_inprocess(flowp, pkt, args->dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 success:
     rlogin_service_mod.api->add_service(flowp, pkt, args->dir, &svc_element,
-                                        APP_ID_RLOGIN, NULL, NULL, NULL);
+                                        APP_ID_RLOGIN, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 
 fail:
     rlogin_service_mod.api->fail_service(flowp, pkt, args->dir, &svc_element,
-                                         rlogin_service_mod.flow_data_index, args->pConfig);
+                                         rlogin_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOMATCH;
 }
 
