@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -495,7 +495,7 @@ done:
     case SERVICE_INPROCESS:
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
-            rpc_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+            rpc_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
         }
         return SERVICE_INPROCESS;
 
@@ -517,7 +517,7 @@ done:
             }
             else subtype = NULL;
             rpc_service_mod.api->add_service(flowp, pkt, dir, &svc_element,
-                                             APP_ID_SUN_RPC, NULL, NULL, subtype);
+                                             APP_ID_SUN_RPC, NULL, NULL, subtype, NULL);
         }
         setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_SUCCESS;
@@ -527,7 +527,7 @@ done:
         {
             rpc_service_mod.api->incompatible_data(flowp, pkt, dir, &svc_element,
                                                    rpc_service_mod.flow_data_index,
-                                                   args->pConfig);
+                                                   args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOT_COMPATIBLE;
@@ -537,7 +537,7 @@ done:
         {
             rpc_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
                                               rpc_service_mod.flow_data_index,
-                                              args->pConfig);
+                                              args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOMATCH;
@@ -848,7 +848,7 @@ done:
 inprocess:
         if (!getAppIdFlag(flowp, APPID_SESSION_SERVICE_DETECTED))
         {
-            rpc_service_mod.api->service_inprocess(flowp, pkt, dir, &tcp_svc_element);
+            rpc_service_mod.api->service_inprocess(flowp, pkt, dir, &tcp_svc_element, NULL);
         }
         return SERVICE_INPROCESS;
 
@@ -870,7 +870,7 @@ inprocess:
             }
             else subtype = NULL;
             rpc_service_mod.api->add_service(flowp, pkt, dir, &tcp_svc_element,
-                                             APP_ID_SUN_RPC, NULL, NULL, subtype);
+                                             APP_ID_SUN_RPC, NULL, NULL, subtype, NULL);
         }
         setAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_SUCCESS;
@@ -880,7 +880,7 @@ inprocess:
         {
             rpc_service_mod.api->incompatible_data(flowp, pkt, dir, &tcp_svc_element,
                                                    rpc_service_mod.flow_data_index,
-                                                   args->pConfig);
+                                                   args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOT_COMPATIBLE;
@@ -891,7 +891,7 @@ fail:
         {
             rpc_service_mod.api->fail_service(flowp, pkt, dir, &tcp_svc_element,
                                               rpc_service_mod.flow_data_index,
-                                              args->pConfig);
+                                              args->pConfig, NULL);
         }
         clearAppIdFlag(flowp, APPID_SESSION_CONTINUE);
         return SERVICE_NOMATCH;
