@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -86,15 +86,15 @@ static int16_t app_id = 0;
 
 static int rshell_init(const InitServiceAPI * const init_api)
 {
-	unsigned i;
+    unsigned i;
 
     app_id = init_api->dpd->addProtocolReference("rsh-error");
 
-	for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
-		init_api->RegisterAppId(&rshell_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
-	}
+    for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
+        init_api->RegisterAppId(&rshell_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
+    }
 
     return 0;
 }
@@ -203,7 +203,8 @@ static int rshell_validate(ServiceValidationArgs* args)
                                      APPID_SESSION_NO_TPI |
                                      APPID_SESSION_SERVICE_DETECTED |
                                      APPID_SESSION_NOT_A_SERVICE |
-                                     APPID_SESSION_PORT_SERVICE_DONE);
+                                     APPID_SESSION_PORT_SERVICE_DONE, 
+                                     APP_ID_FROM_RESPONDER);
                 pf->rnaServiceState = RNA_STATE_STATEFUL;
             }
             else
@@ -296,22 +297,22 @@ static int rshell_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    rshell_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+    rshell_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 success:
     rshell_service_mod.api->add_service(flowp, pkt, dir, &svc_element,
-                                        APP_ID_SHELL, NULL, NULL, NULL);
+                                        APP_ID_SHELL, NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 
 bail:
     rshell_service_mod.api->incompatible_data(flowp, pkt, dir, &svc_element,
-                                              rshell_service_mod.flow_data_index, args->pConfig);
+                                              rshell_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOT_COMPATIBLE;
 
 fail:
     rshell_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
-                                         rshell_service_mod.flow_data_index, args->pConfig);
+                                         rshell_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOMATCH;
 }
 
