@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -137,12 +137,12 @@ static int snmp_init(const InitServiceAPI * const init_api)
     init_api->RegisterPattern(&snmp_validate, IPPROTO_UDP, SNMP_PATTERN_2, sizeof(SNMP_PATTERN_2), 4, "snmp", init_api->pAppidConfig);
     init_api->RegisterPattern(&snmp_validate, IPPROTO_UDP, SNMP_PATTERN_3, sizeof(SNMP_PATTERN_3), 4, "snmp", init_api->pAppidConfig);
     init_api->RegisterPattern(&snmp_validate, IPPROTO_UDP, SNMP_PATTERN_4, sizeof(SNMP_PATTERN_4), 4, "snmp", init_api->pAppidConfig);
-	unsigned i;
-	for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
-	{
-		_dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
-		init_api->RegisterAppId(&snmp_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
-	}
+    unsigned i;
+    for (i=0; i < sizeof(appIdRegistry)/sizeof(*appIdRegistry); i++)
+    {
+        _dpd.debugMsg(DEBUG_LOG,"registering appId: %d\n",appIdRegistry[i].appId);
+        init_api->RegisterAppId(&snmp_validate, appIdRegistry[i].appId, appIdRegistry[i].additionalInfo, init_api->pAppidConfig);
+    }
 
     return 0;
 }
@@ -480,10 +480,10 @@ static int snmp_validate(ServiceValidationArgs* args)
                 tmp_sd->state = SNMP_STATE_ERROR;
                 return SERVICE_ENULL;
             }
-            PopulateExpectedFlow(flowp, pf, APPID_SESSION_EXPECTED_EVALUATE);
+            PopulateExpectedFlow(flowp, pf, APPID_SESSION_EXPECTED_EVALUATE, APP_ID_APPID_SESSION_DIRECTION_MAX);
             pf->rnaServiceState = RNA_STATE_STATEFUL;
             pf->scan_flags |= SCAN_HOST_PORT_FLAG;
-	    sfaddr_copy_to_raw(&pf->common.initiator_ip, sip);
+            sfaddr_copy_to_raw(&pf->common.initiator_ip, sip);
        }
         break;
     case SNMP_STATE_RESPONSE:
@@ -524,7 +524,7 @@ static int snmp_validate(ServiceValidationArgs* args)
     }
 
 inprocess:
-    snmp_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element);
+    snmp_service_mod.api->service_inprocess(flowp, pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 success:
@@ -548,19 +548,19 @@ success:
     }
     snmp_service_mod.api->add_service(flowp, pkt, dir, &svc_element,
                                       APP_ID_SNMP,
-                                      SNMP_VENDOR_STR, version_str, NULL);
+                                      SNMP_VENDOR_STR, version_str, NULL, NULL);
     return SERVICE_SUCCESS;
 
 bail:
     snmp_service_mod.api->incompatible_data(flowp, pkt, dir, &svc_element,
                                             snmp_service_mod.flow_data_index,
-                                            args->pConfig);
+                                            args->pConfig, NULL);
     return SERVICE_NOT_COMPATIBLE;
 
 fail:
     snmp_service_mod.api->fail_service(flowp, pkt, dir, &svc_element,
                                        snmp_service_mod.flow_data_index,
-                                       args->pConfig);
+                                       args->pConfig, NULL);
     return SERVICE_NOMATCH;
 }
 
