@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -563,23 +563,23 @@ done:
     {
     case SERVICE_INPROCESS:
 inprocess:
-        ssh_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element);
+        ssh_service_mod.api->service_inprocess(flowp, args->pkt, args->dir, &svc_element, NULL);
         return SERVICE_INPROCESS;
 
     case SERVICE_SUCCESS:
         ssh_service_mod.api->add_service(flowp, args->pkt, args->dir, &svc_element,
-                                         APP_ID_SSH, ss->vendor, ss->version, NULL);
+                                         APP_ID_SSH, ss->vendor, ss->version, NULL, NULL);
         return SERVICE_SUCCESS;
 
     case SERVICE_NOMATCH:
 fail:
         ssh_service_mod.api->fail_service(flowp, args->pkt, args->dir, &svc_element,
-                                          ssh_service_mod.flow_data_index, args->pConfig);
+                                          ssh_service_mod.flow_data_index, args->pConfig, NULL);
         return SERVICE_NOMATCH;
 
 not_compatible:
         ssh_service_mod.api->incompatible_data(flowp, args->pkt, args->dir, &svc_element,
-                                               ssh_service_mod.flow_data_index, args->pConfig);
+                                               ssh_service_mod.flow_data_index, args->pConfig, NULL);
         return SERVICE_NOT_COMPATIBLE;
 
     default:
