@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -851,7 +851,7 @@ not_v2:;
     }
 
 inprocess:
-    ssl_service_mod.api->service_inprocess(flowp, args->pkt, dir, &svc_element);
+    ssl_service_mod.api->service_inprocess(flowp, args->pkt, dir, &svc_element, NULL);
     return SERVICE_INPROCESS;
 
 fail:
@@ -862,7 +862,7 @@ fail:
     ss->certs_data = NULL;
     ss->host_name = ss->common_name = ss->org_name = NULL;
     ssl_service_mod.api->fail_service(flowp, args->pkt, dir, &svc_element,
-                                      ssl_service_mod.flow_data_index, args->pConfig);
+                                      ssl_service_mod.flow_data_index, args->pConfig, NULL);
     return SERVICE_NOMATCH;
 
 success:
@@ -927,7 +927,7 @@ success:
         ss->host_name = ss->common_name = ss->org_name = NULL;
     }
     ssl_service_mod.api->add_service(flowp, args->pkt, dir, &svc_element,
-                                     getSslServiceAppId(args->pkt->src_port), NULL, NULL, NULL);
+                                     getSslServiceAppId(args->pkt->src_port), NULL, NULL, NULL, NULL);
     return SERVICE_SUCCESS;
 }
 
