@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -39,22 +39,86 @@
 #include "service_base.h"
 
 /*#define DEBUG_SERVICE_STATE 1 */
+//#define DEBUG_APPID_MEMCAP_PRUNING 1
 
-static SFXHASH *serviceStateCache4;
-static SFXHASH *serviceStateCache6;
+STATIC SFXHASH *serviceStateCache4;
+STATIC SFXHASH *serviceStateCache6;
 
 #define SERVICE_STATE_CACHE_ROWS    65536
 
-static int AppIdServiceStateFree(void *key, void *data)
+bool AppIdServiceStateReloadAdjust(bool idle, unsigned long memcap)
 {
-    AppIdServiceIDState* id_state = (AppIdServiceIDState*)data;
-    if (id_state->serviceList)
+    unsigned max_work = idle ? 512 : 8;
+    static bool adjustStart = true;
+    static unsigned numIpv4Entries = 0;
+    static unsigned numIpv6Entries = 0;
+    static unsigned numIpv4EntriesPruned = 0;
+    static unsigned numIpv6EntriesPruned = 0;
+    static unsigned ipv4MemUsed = 0;
+    static unsigned ipv6MemUsed = 0;
+    unsigned target = max_work;
+
+    memcap >>= 1;
+
+    if (adjustStart)
     {
-        AppIdFreeServiceMatchList(id_state->serviceList);
-        id_state->serviceList = NULL;
+        adjustStart = false;
+        numIpv4Entries = sfxhash_count(serviceStateCache4);
+        numIpv4EntriesPruned = 0;
+        ipv4MemUsed = serviceStateCache4->mc.memused;
+        numIpv6Entries = sfxhash_count(serviceStateCache6);
+        numIpv6EntriesPruned = 0;
+        ipv6MemUsed = serviceStateCache6->mc.memused;
     }
 
-    return 0;
+#ifdef DEBUG_APPID_MEMCAP_PRUNING
+    if (memcap < serviceStateCache4->mc.memused)
+    {
+        unsigned count = sfxhash_count(serviceStateCache4);
+        _dpd.logMsg("AppId: IPv4 cache mem used = %u, num entries = %u, pruning up to %u entries\n",
+                    serviceStateCache4->mc.memused, count, (max_work < count) ? max_work : count);
+    }
+#endif // DEBUG_APPID_MEMCAP_PRUNING
+
+    if (SFXHASH_OK != sfxhash_change_memcap(serviceStateCache4, memcap, &max_work))
+    {
+        numIpv4EntriesPruned += (target - max_work);
+        return false;
+    }
+
+    numIpv4EntriesPruned += (target - max_work);
+    if (target != max_work)
+    {
+        _dpd.logMsg("AppId: IPv4 cache pruning done - initial mem used = %u, initial entries = %u, pruned %u entries, current mem used = %u\n",
+                    ipv4MemUsed, numIpv4Entries, numIpv4EntriesPruned, serviceStateCache4->mc.memused);
+        target = max_work;
+    }
+
+#ifdef DEBUG_APPID_MEMCAP_PRUNING
+    if (memcap < serviceStateCache6->mc.memused)
+    {
+        unsigned count = sfxhash_count(serviceStateCache6);
+        _dpd.logMsg("AppId: IPv6 cache mem used = %u, num entries = %u, pruning up to %u entries\n",
+                    serviceStateCache6->mc.memused, count, (max_work < count) ? max_work : count);
+    }
+#endif // DEBUG_APPID_MEMCAP_PRUNING
+
+    if (SFXHASH_OK != sfxhash_change_memcap(serviceStateCache6, memcap, &max_work))
+    {
+        numIpv6EntriesPruned += (target - max_work);
+        return false;
+    }
+
+    numIpv6EntriesPruned += (target - max_work);
+
+    if (numIpv4EntriesPruned == 0)
+        _dpd.logMsg("AppId: IPv4 cache pruning done - initial mem used = %u, initial entries = %u, pruned %u entries, current mem used = %u\n",
+                    ipv4MemUsed, numIpv4Entries, numIpv4EntriesPruned, serviceStateCache4->mc.memused);
+    _dpd.logMsg("AppId: IPv6 cache pruning done - initial mem used = %u, initial entries = %u, pruned %u entries, current mem used = %u\n",
+                ipv6MemUsed, numIpv6Entries, numIpv6EntriesPruned, serviceStateCache6->mc.memused);
+
+    adjustStart = true;
+    return true;
 }
 
 int AppIdServiceStateInit(unsigned long memcap)
@@ -64,8 +128,8 @@ int AppIdServiceStateInit(unsigned long memcap)
                              sizeof(AppIdServiceIDState),
                              memcap >> 1,
                              1,
-                             &AppIdServiceStateFree,
-                             &AppIdServiceStateFree,
+                             NULL,
+                             NULL,
                              1);
     if (!serviceStateCache4)
     {
@@ -77,8 +141,8 @@ int AppIdServiceStateInit(unsigned long memcap)
                              sizeof(AppIdServiceIDState),
                              memcap >> 1,
                              1,
-                             &AppIdServiceStateFree,
-                             &AppIdServiceStateFree,
+                             NULL,
+                             NULL,
                              1);
     if (!serviceStateCache6)
     {
@@ -177,7 +241,7 @@ AppIdServiceIDState* AppIdGetServiceIDState(sfaddr_t *ip, uint16_t proto, uint16
 AppIdServiceIDState* AppIdAddServiceIDState(sfaddr_t *ip, uint16_t proto, uint16_t port, uint32_t level)
 {
     AppIdServiceStateKey k;
-    AppIdServiceIDState *ss;
+    AppIdServiceIDState *ss = NULL;
     SFXHASH *cache;
     char ipstr[INET6_ADDRSTRLEN];
 
@@ -211,6 +275,8 @@ AppIdServiceIDState* AppIdAddServiceIDState(sfaddr_t *ip, uint16_t proto, uint16
 #ifdef DEBUG_SERVICE_STATE
     _dpd.logMsg("ServiceState: Added to hash: %s:%u:%u:%u %p\n",ipstr, (unsigned)proto, (unsigned)port, level, ss);
 #endif
+    if (ss)
+        memset(ss, 0, sizeof(*ss));
     return ss;
 }
 
