@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2005-2013 Sourcefire, Inc.
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -71,6 +71,7 @@ static void AppIdProcess(SFSnortPacket *p, void *context)
     if (p->stream_session && _dpd.sessionAPI->get_ignore_direction(p->stream_session) == SSN_DIR_BOTH)
     {
         _dpd.sessionAPI->disable_preproc_for_session( p->stream_session, PP_NETWORK_DISCOVERY );
+        PREPROC_PROFILE_END(appMatchPerfStats);
         return;
     }
     fwAppIdSearch(p);
@@ -104,6 +105,18 @@ static int AppIDCheckConfig(struct _SnortConfig *sc)
     return 0;
 }
 
+static void AppIdStaticConfigFree(tAppidStaticConfig* appidSC)
+{
+    if (appidSC)
+    {
+        free(appidSC->app_id_detector_path);
+        free(appidSC->conf_file);
+        free(appidSC->app_stats_filename);
+        if (appidSC->newAppIdConfig)
+            AppIdCommonUnload(appidSC->newAppIdConfig);
+    }
+}
+
 #ifdef SNORT_RELOAD
 
 /********** AppId Reload Functions **********/
@@ -119,7 +132,7 @@ static void reloadWait(void)
         {
             appIdReloadInProgress = true;
             pthread_mutex_unlock(&appIdReloadMutex);
-            break;
+            return;
         }
         pthread_mutex_unlock(&appIdReloadMutex);
         nanosleep(&reloadPollTime, NULL);
@@ -134,9 +147,23 @@ static void reloadUnlock(void)
     pthread_mutex_unlock(&appIdReloadMutex);
 }
 
+STATIC bool AppIdReloadAdjust(bool idle, tSfPolicyId raPolicyId, void* userData)
+{
+    return AppIdServiceStateReloadAdjust(idle, appidStaticConfig->memcap);
+}
+
 static int AppIdReloadReloadVerify(struct _SnortConfig *sc, void *swap_config)
 {
     initializeAppIDForDispatch(sc);
+    if (swap_config)
+    {
+        tAppidStaticConfig* newConfig = (tAppidStaticConfig*)swap_config;
+        if (newConfig->memcap != appidStaticConfig->memcap)
+        {
+            _dpd.logMsg("AppId: old memcap %lu, new memcap %lu\n", appidStaticConfig->memcap, newConfig->memcap);
+            _dpd.reloadAdjustRegister(sc, "AppID", 0, AppIdReloadAdjust, NULL, NULL);
+        }
+    }
     return 0;
 }
 
@@ -153,7 +180,7 @@ static int AppIdReloadReloadVerify(struct _SnortConfig *sc, void *swap_config)
  */
 STATIC void AppIdReload(struct _SnortConfig *sc, char *args, void **new_config)
 {
-    tSfPolicyId           policy_id;
+    tSfPolicyId policy_id;
 
     reloadWait();
 
@@ -164,8 +191,15 @@ STATIC void AppIdReload(struct _SnortConfig *sc, char *args, void **new_config)
 
     if (*new_config == NULL)
     {
+        tAppidStaticConfig* newConfig;
+        if (NULL == (newConfig = (tAppidStaticConfig*)malloc(sizeof(*newConfig))))
+        {
+            _dpd.fatalMsg("AppID failed to allocate memory for new configuration\n");
+        }
+        appIdConfigParse(newConfig, args);
         // Start loading AppId configuration into new_config
-        AppIdCommonReload(new_config);
+        AppIdCommonReload(newConfig, (void**)&newConfig->newAppIdConfig);
+        *new_config = (void*)newConfig;
     }
 }
 
@@ -186,13 +220,17 @@ STATIC void *AppIdReloadSwap(struct _SnortConfig *sc, void *swap_config)
     struct timeval  startTime;
     struct timeval  endTime;
     double          elapsedTime;
-    void            *old_config = NULL;
-
-    gettimeofday(&startTime, NULL);
+    tAppidStaticConfig* tmpConfig = NULL;
 
     if (swap_config)
     {
-        old_config = AppIdCommonReloadSwap(swap_config);
+        gettimeofday(&startTime, NULL);
+
+        tmpConfig = appidStaticConfig;
+        appidStaticConfig = (tAppidStaticConfig*)swap_config;
+
+        tmpConfig->newAppIdConfig = AppIdCommonReloadSwap(appidStaticConfig->newAppIdConfig);
+        appidStaticConfig->newAppIdConfig = NULL;
         ThirdPartyAppIDReconfigure();
 
         gettimeofday(&endTime, NULL);
@@ -202,7 +240,7 @@ STATIC void *AppIdReloadSwap(struct _SnortConfig *sc, void *swap_config)
     }
 
     // Return old configuration data structure
-    return old_config;
+    return (void*)tmpConfig;
 }
 
 /**
@@ -217,12 +255,7 @@ STATIC void *AppIdReloadSwap(struct _SnortConfig *sc, void *swap_config)
  */
 STATIC void AppIdReloadFree(void *old_context)
 {
-    if (old_context)
-    {
-        // Free old configuration, which was returned by AppIdReloadSwap().
-        AppIdCommonUnload(old_context);
-    }
-
+    AppIdStaticConfigFree((tAppidStaticConfig*)old_context);
     reloadUnlock();
 }
 
@@ -251,7 +284,7 @@ STATIC int AppIdReconfigure(uint16_t type, const uint8_t *data, uint32_t length,
 
     if (*new_context == NULL)
     {
-        AppIdCommonReload(new_context);
+        AppIdCommonReload(appidStaticConfig, new_context);
     }
 
     return 0;
@@ -350,6 +383,7 @@ static void AppIdResetStats(int signal, void *data)
 static void AppIdCleanExit(int signal, void *unused)
 {
     AppIdCommonFini();
+    AppIdStaticConfigFree(appidStaticConfig);
 }
 
 static int ThirdPartyReload(uint16_t type, void *new_context, void **old_context)
@@ -359,7 +393,7 @@ static int ThirdPartyReload(uint16_t type, void *new_context, void **old_context
         thirdparty_appid_module->print_stats();
     }
     ThirdPartyAppIDFini();
-    ThirdPartyAppIDInit(&appidStaticConfig);
+    ThirdPartyAppIDInit(appidStaticConfig);
     return 0;
 }
 
@@ -383,11 +417,16 @@ static void AppIdInit(struct _SnortConfig *sc, char *args)
         _dpd.addPreprocProfileFunc("custom", &luaCustomPerfStats, 2, &luaDetectorsPerfStats, NULL);
 #       endif
         appid_preproc_status_bit = _dpd.sessionAPI->get_preprocessor_status_bit();
-        appIdConfigParse(args);
 
-        AppIdCommonInit(&appidStaticConfig);
-        ThirdPartyAppIDInit(&appidStaticConfig);
-        if (appidStaticConfig.app_id_dump_ports)
+        if (NULL == (appidStaticConfig = (tAppidStaticConfig*)malloc(sizeof(*appidStaticConfig))))
+        {
+            _dpd.fatalMsg("AppID failed to allocate memory for the configuration\n"); 
+        }
+        appIdConfigParse(appidStaticConfig, args);
+
+        AppIdCommonInit(appidStaticConfig);
+        ThirdPartyAppIDInit(appidStaticConfig);
+        if (appidStaticConfig->app_id_dump_ports)
         {
             dumpPorts(stdout, pAppidActiveConfig);
             appInfoTableDump(pAppidActiveConfig);
