@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -59,7 +59,8 @@ extern const char *smb_com_strings[SMB_MAX_NUM_COMS];
 extern const char *smb_transaction_sub_command_strings[TRANS_SUBCOM_MAX];
 extern const char *smb_transaction2_sub_command_strings[TRANS2_SUBCOM_MAX];
 extern const char *smb_nt_transact_sub_command_strings[NT_TRANSACT_SUBCOM_MAX];
-extern char smb_file_name[DCE2_SMB_MAX_PATH_LEN+1];
+extern uint8_t smb_file_name[2*DCE2_SMB_MAX_PATH_LEN + UTF_16_LE_BOM_LEN + 2];
+extern uint16_t smb_file_name_len;
 
 /********************************************************************
  * Enums
@@ -134,8 +135,8 @@ typedef struct _DCE2_SmbFileTracker
 
     bool is_ipc;
     bool is_smb2;
-    char *file_name;
-    uint16_t file_name_size;
+    uint8_t *file_name;
+    uint16_t file_name_len;
     union
     {
         struct
@@ -246,7 +247,8 @@ typedef struct _DCE2_SmbRequestTracker
 
     // Used for requests to cache data that will ultimately end up in
     // the file tracker upon response.
-    char *file_name;
+    uint8_t *file_name;
+    uint16_t file_name_len;
     uint64_t file_size;
     uint64_t file_offset;
     bool sequential_only;
