@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,62 +52,43 @@ static void DCE2_Smb2Inspect(DCE2_SmbSsnData *ssd, const Smb2Hdr *smb_hdr,
  * Function: DCE2_Smb2GetFileName()
  *
  * Purpose:
- *  Parses data passed in and returns an ASCII string.
+ *  Parses data passed in and returns a BOM prepended byte stream.
  *
  * Arguments:
  *  const uint8_t *  - pointer to data
  *  uint32_t         - data length
- *  bool             - true if the data is unicode (UTF-16LE)
- *  bool             - true if the function should only return the
- *                     file name instead of the entire path
+ *  uint16_t *       - Returns the length of the output buffer including the NULL terminated bytes
  *
  * Returns:
- *  char *  - NULL terminated ASCII string
+ *  uint8_t *        - NULL terminated byte stream (UTF-16LE with BOM)
  *
  ********************************************************************/
-static char * DCE2_Smb2GetFileName(const uint8_t *data,
-        uint32_t data_len, bool unicode, bool get_file)
+static uint8_t * DCE2_Smb2GetFileName(const uint8_t *data,
+        uint32_t data_len, uint16_t *file_name_len)
 {
-    char *str;
-    uint32_t i, j, k = unicode ? data_len - 1 : data_len;
-    uint8_t inc = unicode ? 2 : 1;
+    uint8_t *fname = NULL;
+    int i;
 
-    if (data_len < inc)
+    if (data_len < 2)
+        return NULL;
+    if(data_len > 2*DCE2_SMB_MAX_PATH_LEN)
         return NULL;
 
-    // Move forward.  Don't know if the end of data is actually
-    // the end of the string.
-    for (i = 0, j = 0; i < k; i += inc)
+    for (i = 0; i < data_len; i += 2)
     {
-        uint16_t uchar = unicode ? SmbNtohs((uint16_t *)(data + i)) : data[i];
-
+        uint16_t uchar =  SmbNtohs((uint16_t *)(data + i));
         if (uchar == 0)
             break;
-        else if (get_file && ((uchar == 0x002F) || (uchar == 0x005C)))  // slash and back-slash
-            j = i + inc;
     }
-
-    // Only got a NULL byte or nothing after slash/back-slash or too big.
-    if ((i == 0) || (j == i)
-            || (get_file && (i > DCE2_SMB_MAX_COMP_LEN))
-            || (i > DCE2_SMB_MAX_PATH_LEN))
+    fname = (uint8_t *)DCE2_Alloc(i + UTF_16_LE_BOM_LEN + 2, DCE2_MEM_TYPE__SMB_SSN);
+    if (fname == NULL)
         return NULL;
 
-    str = (char *)DCE2_Alloc(((i-j)>>(inc-1))+1, DCE2_MEM_TYPE__SMB_SSN);
-    if (str == NULL)
-        return NULL;
+    memcpy(fname, UTF_16_LE_BOM, UTF_16_LE_BOM_LEN);//Prepend with BOM
+    memcpy(fname + UTF_16_LE_BOM_LEN, data, i);
+    *file_name_len = i + UTF_16_LE_BOM_LEN + 2;
 
-    for (k = 0; j < i; j += inc, k++)
-    {
-        if ((int)data[j])
-            str[k] = (char)data[j];
-        else
-            str[k] = '.';
-    }
-
-    str[k] = 0;
-
-    return str;
+    return fname;
 }
 
 static inline uint32_t Smb2Tid(const Smb2Hdr *hdr)
@@ -266,17 +247,17 @@ static inline void DCE2_Smb2FreeFileName(DCE2_SmbFileTracker *ftracker)
 {
     if (ftracker->file_name)
     {
-        DCE2_Free(ftracker->file_name, ftracker->file_name_size, DCE2_MEM_TYPE__SMB_SSN);
+        DCE2_Free(ftracker->file_name, ftracker->file_name_len, DCE2_MEM_TYPE__SMB_SSN);
         ftracker->file_name = NULL;
     }
-    ftracker->file_name_size = 0;
+    ftracker->file_name_len = 0;
 }
 
 static inline void DCE2_Smb2ResetFileName(DCE2_SmbFileTracker *ftracker)
 {
-    DCE2_UnRegMem(ftracker->file_name_size, DCE2_MEM_TYPE__SMB_SSN);
+    DCE2_UnRegMem(ftracker->file_name_len, DCE2_MEM_TYPE__SMB_SSN);
     ftracker->file_name = NULL;
-    ftracker->file_name_size = 0;
+    ftracker->file_name_len = 0;
 }
 
 static inline void DCE2_Smb2ProcessFileData(DCE2_SmbSsnData *ssd, const uint8_t *file_data,
@@ -303,10 +284,14 @@ static inline void DCE2_Smb2ProcessFileData(DCE2_SmbSsnData *ssd, const uint8_t
         DCE2_FileDetect(&ssd->sd);
     }
 
-    _dpd.fileAPI->file_segment_process(fileCache, (void *)ssd->sd.wire_pkt,
+    if ((ssd->max_file_depth == 0) ||
+            (ssd->ftracker.tracker.file.file_offset <  ssd->max_file_depth))
+    {
+        _dpd.fileAPI->file_segment_process(fileCache, (void *)ssd->sd.wire_pkt,
             ssd->ftracker.fid_v2, ssd->ftracker.tracker.file.file_size,
             file_data, data_size, ssd->ftracker.tracker.file.file_offset,
             upload);
+    }
 }
 
 /********************************************************************
@@ -383,11 +368,9 @@ static void DCE2_Smb2CreateRequest(DCE2_SmbSsnData *ssd, const Smb2Hdr *smb_hdr,
             return;
         if (ssd->ftracker.file_name)
         {
-            DCE2_Free((void *)ssd->ftracker.file_name, ssd->ftracker.file_name_size, DCE2_MEM_TYPE__SMB_SSN);
+            DCE2_Free((void *)ssd->ftracker.file_name, ssd->ftracker.file_name_len, DCE2_MEM_TYPE__SMB_SSN);
         }
-        ssd->ftracker.file_name = DCE2_Smb2GetFileName(file_data, size, true, false);
-        if (ssd->ftracker.file_name)
-            ssd->ftracker.file_name_size = strlen(ssd->ftracker.file_name) + 1;
+        ssd->ftracker.file_name = DCE2_Smb2GetFileName(file_data, size, &ssd->ftracker.file_name_len);
     }
 }
 
@@ -413,10 +396,10 @@ static void DCE2_Smb2CreateResponse(DCE2_SmbSsnData *ssd, const Smb2Hdr *smb_hdr
         ssd->ftracker.tracker.file.file_size = file_size;
     }
 
-    if (ssd->ftracker.file_name && ssd->ftracker.file_name_size)
+    if (ssd->ftracker.file_name && ssd->ftracker.file_name_len)
     {
         _dpd.fileAPI->file_cache_update_entry(fileCache, (void *)ssd->sd.wire_pkt, ssd->ftracker.fid_v2,
-                (uint8_t *) ssd->ftracker.file_name,  ssd->ftracker.file_name_size, file_size);
+                (uint8_t *) ssd->ftracker.file_name,  ssd->ftracker.file_name_len, file_size);
     }
     DCE2_Smb2ResetFileName(&(ssd->ftracker));
 }
@@ -936,3 +919,45 @@ void DCE2_Smb2Close(void)
         fileCache = NULL;
     }
 }
+
+bool DCE2_IsFileCache(void *ptr)
+{
+    return fileCache == ptr;
+}
+
+#ifdef SNORT_RELOAD
+
+/*********************************************************************
+ * Function: DCE2_Smb2AdjustFileCache
+ *
+ * Purpose: Adapts file cache to new config in bursts of effort
+ *
+ * Arguments:   work            - size of work burst
+*               cache_enabled   - cache is enabled in new config
+ *
+ * Returns: true when file cache meets memcaps, else false
+ *
+ *********************************************************************/
+bool DCE2_Smb2AdjustFileCache(uint8_t work, bool cache_enabled)
+{
+    //TODO (nice to have) Adjust number of rows in hash table based on new size
+
+    /* Delete files until new max files cap is met
+     * Delete files until new segment memcap is met
+     * Delete file cache if no longer used
+     * return true when done, else false
+    */
+    bool file_cache_adapted = _dpd.fileAPI->file_cache_shrink_to_memcap(fileCache, &work);
+    if (file_cache_adapted && !cache_enabled)
+       DCE2_Smb2Close();
+
+    return file_cache_adapted;
+}
+
+void DCE2_SetSmbMemcap(uint64_t smbMemcap)
+{
+    _dpd.fileAPI->file_cache_set_memcap(fileCache, smbMemcap);
+}
+
+
+#endif
