@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -287,5 +287,15 @@ DCE2_SmbVersion DCE2_Smb2Version(const SFSnortPacket *p);
 
 /* Update statistics for smb2 processing */
 void DCE2_Smb2UpdateStats(void);
+
+bool DCE2_IsFileCache(void *ptr);
+#ifdef SNORT_RELOAD
+/*ensures that file cache is obeying new memcap
+ *deletes file cache if necessary
+*/
+bool DCE2_Smb2AdjustFileCache(uint8_t work, bool cache_enabled);
+void DCE2_SetSmbMemcap(uint64_t smbMemcap);
+#endif
+
 #endif  /* _DCE2_SMB2_H_ */
 
