@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,12 +30,20 @@
 #include "sf_types.h"
 #include "snort_debug.h"
 
+#ifdef SNORT_RELOAD
+#include "appdata_adjuster.h"
+#endif
+
 /********************************************************************
  * Macros
  ********************************************************************/
 #define DCE2_PROTO_REF_STR__NBSS    "netbios-ssn"   /* Essentially SMB */
 #define DCE2_PROTO_REF_STR__DCERPC  "dcerpc"
 
+#ifdef SNORT_RELOAD
+extern APPDATA_ADJUSTER *ada;
+#endif
+
 /********************************************************************
  * Enumerations
  ********************************************************************/
