@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #endif
 
 #include <assert.h>
+#include <stddef.h>
 
 #include "sf_types.h"
 #include "spp_dce2.h"
@@ -50,6 +51,14 @@
 #include "sfPolicy.h"
 #include "sfPolicyUserData.h"
 
+#ifdef SNORT_RELOAD
+#include "appdata_adjuster.h"
+#include "dce2_session.h"
+#ifdef REG_TEST
+#include "reg_test.h"
+#endif
+#endif
+
 #ifdef DCE2_LOG_EXTRA_DATA
 #include "Unified2_common.h"
 #endif
@@ -142,12 +151,22 @@ static int DCE2_LogSmbFileName(void *, uint8_t **, uint32_t *, uint32_t *);
 static void DCE2_ReloadGlobal(struct _SnortConfig *, char *, void **);
 static void DCE2_ReloadServer(struct _SnortConfig *, char *, void **);
 static int DCE2_ReloadVerify(struct _SnortConfig *, void *);
+static bool DCE2_ReloadAdjust(bool, tSfPolicyId, void *);
 static void * DCE2_ReloadSwap(struct _SnortConfig *, void *);
 static void DCE2_ReloadSwapFree(void *);
 #endif
 
 static void DCE2_AddPortsToPaf(struct _SnortConfig *, DCE2_Config *, tSfPolicyId);
 static void DCE2_ScAddPortsToPaf(struct _SnortConfig *, void *);
+static uint32_t max(uint32_t a, uint32_t b);
+static uint32_t DCE2_GetReloadSafeMemcap();
+
+static bool dce2_file_cache_is_enabled = false;
+static bool dce2_file_cache_was_enabled = false;
+#ifdef SNORT_RELOAD
+static bool dce2_ada_was_enabled = false;
+static bool dce2_ada_is_enabled = false;
+#endif
 
 /********************************************************************
  * Function: DCE2_RegisterPreprocessor()
@@ -203,6 +222,12 @@ static void DCE2_InitGlobal(struct _SnortConfig *sc, char *args)
     if (dce2_config == NULL)
     {
         dce2_config = sfPolicyConfigCreate();
+        dce2_file_cache_is_enabled = false;
+        dce2_file_cache_was_enabled = false;
+#ifdef SNORT_RELOAD
+        dce2_ada_was_enabled = false;
+        dce2_ada_is_enabled = false;
+#endif
         if (dce2_config == NULL)
         {
             DCE2_Die("%s(%d) \"%s\" configuration: Could not allocate memory "
@@ -314,6 +339,17 @@ static void DCE2_InitGlobal(struct _SnortConfig *sc, char *args)
     _dpd.streamAPI->set_service_filter_status
         (sc, dce2_proto_ids.nbss, PORT_MONITOR_SESSION, policy_id, 1);
 #endif
+
+#ifdef SNORT_RELOAD
+    if (!ada)
+    {
+        size_t memcap = DCE2_GetReloadSafeMemcap(dce2_config);
+        ada = ada_init(DCE2_MemInUse, PP_DCE2, memcap);
+        if (!ada)
+            _dpd.fatalMsg("Failed to initialize DCE ADA session cache.\n");
+    }
+    dce2_ada_is_enabled = true;
+#endif
 }
 
 /*********************************************************************
@@ -407,7 +443,10 @@ static int DCE2_CheckConfigPolicy(
         DCE2_RegMem(sfrt_usage(pPolicyConfig->sconfigs), DCE2_MEM_TYPE__RT);
 
     if (!pPolicyConfig->gconfig->legacy_mode)
+    {
         DCE2_Smb2Init(pPolicyConfig->gconfig->memcap);
+        dce2_file_cache_is_enabled = true;
+    }
 
     return 0;
 }
@@ -527,11 +566,11 @@ static void DCE2_Main(void *pkt, void *context)
 static int DCE2_LogSmbFileName(void *ssn_ptr, uint8_t **buf, uint32_t *len, uint32_t *type)
 {
     if ((_dpd.streamAPI->get_application_data(ssn_ptr, PP_DCE2) == NULL)
-            || (strlen(smb_file_name) == 0))
+            || (smb_file_name_len == 0))
         return 0;
 
-    *buf = (uint8_t *)smb_file_name; 
-    *len = strlen(smb_file_name);
+    *buf = (uint8_t *)smb_file_name;
+    *len = smb_file_name_len;
     *type = EVENT_INFO_SMB_FILENAME;
 
     return 1;
@@ -993,6 +1032,10 @@ static void DCE2_CleanExit(int signal, void *data)
 {
     DCE2_FreeConfigs(dce2_config);
     dce2_config = NULL;
+#ifdef SNORT_RELOAD
+    ada_delete( ada );
+    ada = NULL;
+#endif
 
     DCE2_FreeGlobals();
     DCE2_Smb2Close();
@@ -1027,7 +1070,10 @@ static void DCE2_ReloadGlobal(struct _SnortConfig *sc, char *args, void **new_co
     {
         //create a context
         dce2_swap_config = sfPolicyConfigCreate();
-
+        dce2_file_cache_was_enabled = !DCE2_IsFileCache(NULL);
+        dce2_file_cache_is_enabled = false;
+        dce2_ada_is_enabled = false;
+        dce2_ada_was_enabled = ada != NULL;
         if (dce2_swap_config == NULL)
         {
             DCE2_Die("%s(%d) \"%s\" configuration: Could not allocate memory "
@@ -1081,6 +1127,15 @@ static void DCE2_ReloadGlobal(struct _SnortConfig *sc, char *args, void **new_co
 
     if (policy_id != 0)
         pCurrentPolicyConfig->gconfig->memcap = pDefaultPolicyConfig->gconfig->memcap;
+
+    if (!ada)
+    {
+        size_t memcap = DCE2_GetReloadSafeMemcap(dce2_swap_config);
+        ada = ada_init(DCE2_MemInUse, PP_DCE2, memcap);
+        if (!ada)
+            _dpd.fatalMsg("Failed to initialize DCE ADA session cache.\n");
+    }
+    dce2_ada_is_enabled = true;
 }
 
 /*********************************************************************
@@ -1175,19 +1230,22 @@ static int DCE2_ReloadVerifyPolicy(
     if (swap_config->sconfigs != NULL)
         DCE2_RegMem(sfrt_usage(swap_config->sconfigs), DCE2_MEM_TYPE__RT);
 
+    if (!swap_config->gconfig->legacy_mode)
+    {
+        DCE2_Smb2Init(swap_config->gconfig->memcap);
+        dce2_file_cache_is_enabled = true;
+    }
+
     if (current_config == NULL)
         return 0;
 
-    if (swap_config->gconfig->memcap != current_config->gconfig->memcap)
-    {
-        _dpd.errMsg("dcerpc2 reload:  Changing the memcap requires a restart.\n");
-        return -1;
-    }
-
     return 0;
 }
+
 /*********************************************************************
  * Function: DCE2_ReloadVerify()
+ * WARNING This runs in a thread that is Asynchronous with the
+ * packet loop thread
  *
  * Purpose: Verifies a reloaded DCE/RPC preprocessor configuration
  *
@@ -1211,9 +1269,70 @@ static int DCE2_ReloadVerify(struct _SnortConfig *sc, void *swap_config)
         return -1;
     }
 
+    tSfPolicyId policy_id = _dpd.getParserPolicy(sc);
+    /* Look in DCE2_InitGlobal and DCE2_ReloadGlobal to find that DefaultPolicy can't be NULL
+    *  You'll also find that all memcaps are synched to DefaultPolicy as the memcap is global
+    *  to all policies
+    *  Here we are setting up the work that needs to be done to adjust to the new configuration
+    */
+    uint32_t current_memcap     = DCE2_GetReloadSafeMemcap(dce2_config);
+    uint32_t new_memcap         = DCE2_GetReloadSafeMemcap(dce2_swap_config);
+
+    if (dce2_ada_was_enabled && !dce2_ada_is_enabled)
+    {
+        ada_set_new_cap(ada, 0);
+        _dpd.reloadAdjustRegister(sc, "dce2-mem-reloader", policy_id, DCE2_ReloadAdjust, NULL, NULL);
+    }
+    else if (new_memcap != current_memcap ||
+        (dce2_file_cache_was_enabled && !dce2_file_cache_is_enabled))
+    {
+        ada_set_new_cap(ada, (size_t)(new_memcap));
+        _dpd.reloadAdjustRegister(sc, "dce2-mem-reloader", policy_id, DCE2_ReloadAdjust, NULL, NULL);
+    }
+
     return 0;
 }
 
+/*********************************************************************
+ * Function: DCE2_ReloadAdjust
+ *
+ * Purpose: After reloading with a different configuration that
+ * that requres work to adapt, this function will perform the
+ * work in small bursts.
+ *
+ * Arguments:   idle - if snort is idling (low packets) do more work
+ *              raPolicyId - identifies which policy the config is for
+ *              userData - data needed by this function
+ *
+ * Returns:
+ *  bool
+ *      return false if it needs to work more
+ *      return true if there is no work left
+ *
+ *********************************************************************/
+static bool DCE2_ReloadAdjust(bool idle, tSfPolicyId raPolicyId, void *userData)
+{
+    /* delete files until the file cache memcaps are met
+     * delete file cache if not needed
+     * delete dce session data from sessions until dce global memcap is met
+     * delete ada cache if not needed
+     * return true when completed successfully, else false
+    */
+    uint8_t maxWork = idle ? 512 : 32;
+    bool memcaps_satisfied =    DCE2_Smb2AdjustFileCache(maxWork, dce2_file_cache_is_enabled) &&
+                                ada_reload_adjust_func(idle, raPolicyId, (void *) ada);
+    if (memcaps_satisfied && dce2_ada_was_enabled && !dce2_ada_is_enabled)
+    {
+        ada_delete(ada);
+        ada = NULL;
+    }
+#ifdef REG_TEST
+    if (memcaps_satisfied && REG_TEST_FLAG_RELOAD & getRegTestFlags())
+        printf("dcerpc2-reload reload-adjust %d %d \n", ada != NULL, !DCE2_IsFileCache(NULL));
+#endif
+    return memcaps_satisfied;
+}
+
 static int DCE2_ReloadSwapPolicy(
         tSfPolicyUserContextId config,
         tSfPolicyId policyId,
@@ -1243,16 +1362,34 @@ static int DCE2_ReloadSwapPolicy(
  *********************************************************************/
 static void * DCE2_ReloadSwap(struct _SnortConfig *sc, void *swap_config)
 {
-    tSfPolicyUserContextId dce2_swap_config = (tSfPolicyUserContextId)swap_config;
-    tSfPolicyUserContextId old_config = dce2_config;
+    tSfPolicyUserContextId dce2_swap_config = (tSfPolicyUserContextId)  swap_config;
+    tSfPolicyUserContextId old_config       =                           dce2_config;
 
     if (dce2_swap_config == NULL)
         return NULL;
 
+    /* Set file cache's new memcaps so it doesn't misbehave
+    *  Ensure that the correct amount of memory is registered
+    *  See DCE2_Smb2Init for inspiration
+    */
+    uint32_t current_memcap = 0;
+    if (dce2_file_cache_was_enabled)
+        current_memcap = DCE2_GetReloadSafeMemcap(dce2_config);
+    uint32_t swap_memcap = 0;
+    if (dce2_file_cache_is_enabled)
+        swap_memcap    = DCE2_GetReloadSafeMemcap(dce2_swap_config);
+
+    DCE2_SetSmbMemcap((uint64_t) swap_memcap >> 1);
+
+    if (dce2_file_cache_was_enabled)
+    {
+        DCE2_UnRegMem(current_memcap >> 1, DCE2_MEM_TYPE__SMB_SSN);
+        if (dce2_file_cache_is_enabled)
+            DCE2_RegMem(swap_memcap >> 1, DCE2_MEM_TYPE__SMB_SSN);
+    }
+
     dce2_config = dce2_swap_config;
-
     sfPolicyUserDataFreeIterate (old_config, DCE2_ReloadSwapPolicy);
-
     if (sfPolicyUserPolicyGetActive(old_config) == 0)
         return (void *)old_config;
 
@@ -1343,3 +1480,46 @@ static void DCE2_ScAddPortsToPaf(struct _SnortConfig *snortConf, void *data)
     }
 }
 
+static uint32_t max(uint32_t a, uint32_t b)
+{
+    if (a >= b)
+        return a;
+    return b;
+}
+
+/*********************************************************************
+ * Function: DCE2_GetReloadSafeMemcap
+ *
+ * Purpose: Provide a safe memcap that can be used durring
+ * packet processing.
+ * This is based on how the memcaps are set in InitGlobal and
+ * ReloadGlobal
+ *
+ * The memcap in the config for the policyId currently running is
+ * what is used as a memcap, according to DCE2_Process calling 
+ * DCE2_GcMemcap.
+ *
+ * Based on InitGlobal and ReloadGlobal all memcaps are equal to
+ * the memcap in the default policy except for policy_id 0
+ *
+ * So to be safe we'll just return the max of the two.
+ *
+ * This function also helps to prevent segmentation faults caused
+ * by accessing null pointers.
+ *
+ * Arguments: pConfig - mapping between policyIDs and configs
+ *
+ * Returns: A safe memcap
+ *
+ *********************************************************************/
+static uint32_t DCE2_GetReloadSafeMemcap(tSfPolicyUserContextId pConfig)
+{
+  DCE2_Config *pDefaultPolicyConfig = sfPolicyUserDataGetDefault(pConfig);
+  DCE2_Config *pPolicyConfig        = sfPolicyUserDataGet(pConfig, 0);
+  uint32_t defaultMem = pDefaultPolicyConfig == NULL ?
+                        0 : pDefaultPolicyConfig->gconfig->memcap;
+  uint32_t policyMem  = pPolicyConfig == NULL ?
+                        0 : pPolicyConfig->gconfig->memcap;
+
+  return max(defaultMem,policyMem);
+}
