@@ -1,5 +1,5 @@
 /*
- ** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ ** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  ** Copyright (C) 2013-2013 Sourcefire, Inc.
  **
  ** This program is free software; you can redistribute it and/or modify
@@ -239,7 +239,7 @@ static void* FileCaptureThread(void *arg)
     return NULL;
 }
 
-void file_agent_init(void *config)
+void file_agent_init(struct _SnortConfig *sc, void *config)
 {
     FileInspectConf* conf = (FileInspectConf *)config;
 
@@ -247,22 +247,22 @@ void file_agent_init(void *config)
 
     if (conf->file_type_enabled)
     {
-        _dpd.fileAPI->enable_file_type(file_agent_type_callback);
+        _dpd.fileAPI->enable_file_type(sc, file_agent_type_callback);
         file_type_enabled = true;
     }
     if (conf->file_signature_enabled)
     {
-        _dpd.fileAPI->enable_file_signature(file_agent_signature_callback);
+        _dpd.fileAPI->enable_file_signature(sc, file_agent_signature_callback);
         file_signature_enabled = true;
     }
 
     if (conf->file_capture_enabled)
     {
-        _dpd.fileAPI->enable_file_capture(file_agent_signature_callback);
+        _dpd.fileAPI->enable_file_capture(sc, file_agent_signature_callback);
         file_capture_enabled = true;
     }
 
-    if (conf->hostname)
+    if (!sockfd && conf->hostname)
     {
         file_agent_init_socket(conf->hostname, conf->portno);
     }
