@@ -1,7 +1,7 @@
 /*
  * ftpp_si.c
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2004-2013 Sourcefire, Inc.
  * Steven A. Sturges <ssturges@sourcefire.com>
  * Daniel J. Roelker <droelker@sourcefire.com>
@@ -61,6 +61,7 @@
 #include "snort_ftptelnet.h"
 #include "sfPolicyUserData.h"
 #include "ssl_include.h"
+#include "sfmemcap.h"
 
 #ifndef WIN32
 # include <ctype.h>
@@ -668,6 +669,13 @@ static void FTPFreeSession(void *preproc_session)
 
     if (pPolicyConfig != NULL)
     {
+        if (ssn->path)
+        {
+            sfmemcap_free(&pPolicyConfig->mc,ssn->path);
+        }
+        if (ssn->cwd_path)
+            sfmemcap_free(&pPolicyConfig->mc,ssn->cwd_path);
+
         pPolicyConfig->ref_count--;
         if ((pPolicyConfig->ref_count == 0) &&
             (ssn->global_conf != ftp_telnet_config))
@@ -688,6 +696,10 @@ static void FTPFreeSession(void *preproc_session)
 
     if ( ssl_cb )
         ssl_cb->session_free(ssn->flow_id);
+
+    FTP_DATA_SESSION *datassn = ssn->datassn;
+    if(datassn && (ssn == datassn->ftpssn))
+        datassn->ftpssn = NULL;
     free(ssn);
 }
 
@@ -731,6 +743,10 @@ void FTPDataSessionFree(void *p_ssn)
     if (!ssn)
         return;
 
+    FTP_SESSION * ftpssn = ssn->ftpssn;
+    if(ftpssn && (ssn == ftpssn->datassn))
+        ftpssn->datassn = NULL;
+
     /* ftp-data key shouldn't exist without this but */
     if (ssn->ftp_key)
     {
@@ -807,6 +823,11 @@ static inline int FTPResetSession(FTP_SESSION *FtpSession)
     FtpSession->data_xfer_index = 0;
     FtpSession->ftp_cmd_pipe_index = 1;
 
+    FtpSession->path_hash = 0;
+    FtpSession->path = NULL;
+    FtpSession->cwd_path = NULL;
+    FtpSession->path_memory_alloc = 0;
+    FtpSession->cwd_memory_alloc = 0;
     FtpSession->event_list.stack_count = 0;
 
     return FTPP_SUCCESS;
