@@ -1,7 +1,7 @@
 /*
  * ftpp_si.h
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2004-2013 Sourcefire, Inc.
  * Steven A. Sturges <ssturges@sourcefire.com>
  * Daniel J. Roelker <droelker@sourcefire.com>
@@ -168,22 +168,35 @@ typedef struct s_FTP_SESSION
 
     /* A file is being transfered on ftp-data channel */
     char *filename;
+    char *path;
     int file_xfer_info; /* -1: ignore, 0: unknown, >0: filename length */
-    unsigned char flags;
     bool data_xfer_dir;
 
     /* Command/data channel encryption */
     bool encr_state_chello;
     int encr_state;
     uint32_t flow_id;
+    uint32_t path_hash;
 
     /* Alertable event list */
     FTP_EVENTS event_list;
     void *datassn;
+    char *cwd_path;
+    sfaddr_t      control_clientIP;
+    sfaddr_t      control_serverIP;
+    uint16_t control_clientPort;
+    uint16_t control_serverPort;
+    uint16_t path_len;
+    uint16_t path_memory_alloc;
+    uint16_t cwd_memory_alloc;
+    unsigned char flags;
 
 } FTP_SESSION;
 
 #define FTP_FLG_MALWARE  (1<<0)
+#define FTP_FLG_MALWARE_ENABLED (1<<1)
+#define FTP_FLG_MALWARE_CHECK   (1<<2)
+
 
 #ifdef TARGET_BASED
 
@@ -197,6 +210,7 @@ typedef struct s_FTP_DATA_SESSION
 {
     FTP_TELNET_SESSION ft_ssn;
     StreamSessionKey * ftp_key;
+    void* ftpssn;
     char *filename;
     int data_chan;
     int file_xfer_info;
@@ -205,12 +219,14 @@ typedef struct s_FTP_DATA_SESSION
     unsigned char mode;
     unsigned char flags;
     uint32_t flow_id;
+    uint32_t path_hash;
 } FTP_DATA_SESSION;
 
 #define FTPDATA_FLG_REASSEMBLY_SET  (1<<0)
 #define FTPDATA_FLG_FILENAME_SET    (1<<1)
 #define FTPDATA_FLG_STOP            (1<<2)
 #define FTPDATA_FLG_REST            (1<<3)
+#define FTPDATA_FLG_FLUSH           (1<<4)
 
 #endif
 
