@@ -1,6 +1,6 @@
 /* $Id$ */
 /*
- ** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ ** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  ** Copyright (C) 2004-2013 Sourcefire, Inc.
  **
  ** This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,8 @@
 
 /* your preprocessor header file goes here */
 
+#define _GNU_SOURCE
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -68,6 +70,7 @@
 #include "snort_debug.h"
 #include "stream_api.h"
 //#include "plugbase.h"
+#include "sfmemcap.h"
 
 #ifndef MAXHOSTNAMELEN /* Why doesn't Windows define this? */
 #define MAXHOSTNAMELEN 256
@@ -79,87 +82,12 @@
 extern int16_t ftp_data_app_id;
 extern unsigned s_ftpdata_eof_cb_id;
 #endif
-
+#define DEFAULT_FTP_MEMCAP (PATH_MAX*1024)
 #ifdef DUMP_BUFFER
 #include "ftptelnet_buffer_dump.h"
 #endif
 
-#if 0
-/*
- * Function: getIP(char **ip_start,
- *                 char *last_char,
- *                 char term_char,
- *                 uint32_t *ipRet,
- *                 uint16_t *portRet)
- *
- * Purpose: Returns a 32bit IP address and port from an FTP-style
- *          string -- ie, a,b,c,d,p1,p2.  Stops checking when term_char
- *          is seen.  Used to get address and port information from FTP
- *          PORT command and server response to PASV command.
- *
- * Arguments ip_start        => Pointer to pointer to the start of string.
- *                              Updated to end of IP address if successful.
- *           last_char       => End of string
- *           term_char       => Character delimiting the end of the address.
- *           ipRet           => Return pointer to 32bit address on success
- *           portRet         => Return pointer to 16bit port on success
- *
- * Returns: int => return code indicating error or success
- *
- */
-int getIP(const int type, const char **ip_start, const char *last_char, char *term_char,
-          sfaddr_t *ipRet, uint16_t *portRet)
-{
-    uint32_t ip=0;
-    uint16_t port=0;
-    int octet=0;
-    const char *this_param = *ip_start;
-
-    do
-    {
-        int value = 0;
-        do
-        {
-            if (!isdigit((int)(*this_param)))
-            {
-                return FTPP_NON_DIGIT;
-            }
-            value = value * 10 + (*this_param - '0');
-            this_param++;
-        } while ((this_param < last_char) &&
-                 (*this_param != ',') &&
-                 (*this_param != term_char));
-        if (value > 0xFF)
-        {
-            return FTPP_INVALID_ARG;
-        }
-        if (octet  < 4)
-        {
-            ip = (ip << 8) + value;
-        }
-        else
-        {
-            port = (port << 8) + value;
-        }
-
-        if (*this_param != term_char)
-            this_param++;
-        octet++;
-    } while ((this_param < last_char) && (*this_param != term_char) );
-
-    if (octet != 6)
-    {
-        return FTPP_MALFORMED_IP_PORT;
-    }
-
-// XXX-IPv6 NOT YET IMPLEMENTED - IPv4 only at the moment
-    sfip_set_raw(ipRet, &ip, AF_INET);
-    *portRet = port;
-    *ip_start = this_param;
-
-    return FTPP_SUCCESS;
-}
-#endif
+#define DEFAULT_MEM_ALLOC 512
 
 /*
  * Function: getIP959(char **ip_start,
@@ -1015,6 +943,8 @@ int initialize_ftp(FTP_SESSION *Session, SFSnortPacket *p, int iMode)
     char ignoreTelnetErase = FTPP_APPLY_TNC_ERASE_CMDS;
     FTPTELNET_GLOBAL_CONF *global_conf = (FTPTELNET_GLOBAL_CONF *)sfPolicyUserDataGet(Session->global_conf, Session->policy_id);
 
+    if (!global_conf->mc.memcap)
+         global_conf->mc.memcap  = DEFAULT_FTP_MEMCAP;
     /* Normalize this packet ala telnet */
     if (((iMode == FTPP_SI_CLIENT_MODE) &&
          (Session->client_conf->ignore_telnet_erase_cmds.on == 1)) ||
@@ -1186,7 +1116,7 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                             IP_COPY_VALUE(Session->clientIP, GET_DST_IP(p));
                             Session->clientPort = 0;
 #ifdef TARGET_BASED
-                            if ((_dpd.fileAPI->get_max_file_depth() > 0) || !(Session->server_conf->data_chan))
+                            if ((_dpd.fileAPI->get_max_file_depth(_dpd.getCurrentSnortConfig(), false) > 0) || !(Session->server_conf->data_chan))
                             {
                                 FTP_DATA_SESSION *ftpdata = FTPDataSessionNew(p);
 
@@ -1195,10 +1125,17 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                                     int result;
                                     /* This is a passive data transfer */
                                     ftpdata->mode = FTPP_XFER_PASSIVE;
-
                                     ftpdata->data_chan = Session->server_conf->data_chan;
-                                    if (Session->flags & FTP_FLG_MALWARE)
-                                        Session->datassn = ftpdata;
+                                    /* Store the FTP_SESSION in FTP_DATA_SESSION, needed when FTP_DATA_SESSION is freed.
+                                     * This is needed to handle pruning of FTP_DATA_SESSION */
+                                    ftpdata->ftpssn = Session;
+                                    /*If a FTP_DATA_SESSION already exists, the FTP_SESSION reference in that should be removed*/
+                                    if(Session->datassn)
+                                    {
+                                        FTP_DATA_SESSION * ssn = Session->datassn;
+                                        ssn->ftpssn = NULL;
+                                    }
+                                    Session->datassn = ftpdata;
 
                                     /* Call into Streams to mark data channel as ftp-data */
                                     result = _dpd.streamAPI->set_application_protocol_id_expected_preassign_callback(
@@ -1265,7 +1202,7 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                         Session->serverPort = ntohs(p->tcph->th_sport) -1;
                         */
 #ifdef TARGET_BASED
-                        if ((_dpd.fileAPI->get_max_file_depth() > 0) || !(Session->server_conf->data_chan))
+                        if ((_dpd.fileAPI->get_max_file_depth(_dpd.getCurrentSnortConfig(), false) > 0) || !(Session->server_conf->data_chan))
                         {
                             FTP_DATA_SESSION *ftpdata = FTPDataSessionNew(p);
 
@@ -1275,8 +1212,16 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                                 /* This is a active data transfer */
                                 ftpdata->mode = FTPP_XFER_ACTIVE;
                                 ftpdata->data_chan = Session->server_conf->data_chan;
-                                if (Session->flags & FTP_FLG_MALWARE)
-                                    Session->datassn = ftpdata;
+                                /* Store the FTP_SESSION in FTP_DATA_SESSION, needed when FTP_DATA_SESSION is freed.
+                                 * This is needed to handle pruning of FTP_DATA_SESSION */
+                                ftpdata->ftpssn = Session;
+                                /*If a FTP_DATA_SESSION already exists, the FTP_SESSION reference in that should be removed*/
+                                if(Session->datassn)
+                                {
+                                    FTP_DATA_SESSION * ssn = Session->datassn;
+                                    ssn->ftpssn = NULL;
+                                }
+                                Session->datassn = ftpdata;
 
                                 /* Call into Streams to mark data channel as ftp-data */
                                 result = _dpd.streamAPI->set_application_protocol_id_expected_preassign_callback(
@@ -1319,11 +1264,8 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                 if (rsp_code == 350)
                 {
                     FTP_DATA_SESSION *ftpdata = Session->datassn;
-                    if ((Session->flags & FTP_FLG_MALWARE) && ftpdata)
-                    {
+                    if(ftpdata)
                         ftpdata->flags |= FTPDATA_FLG_REST;
-                        Session->datassn = NULL;
-                    }
                 }
                 Session->data_chan_index = 0;
                 Session->data_chan_state &= ~DATA_CHAN_REST_CMD_ISSUED;
@@ -1338,7 +1280,6 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                 Session->data_xfer_index = 0;
                 if ((rsp_code == 150) || (rsp_code == 125))
                 {
-                    Session->data_chan_state &= ~DATA_CHAN_XFER_CMD_ISSUED;
                     Session->data_chan_state = DATA_CHAN_XFER_STARTED;
                 }
                 /* Clear the session info for next transfer -->
@@ -1346,7 +1287,6 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
                 IP_CLEAR(Session->serverIP);
                 IP_CLEAR(Session->clientIP);
                 Session->serverPort = Session->clientPort = 0;
-                Session->datassn = NULL;
                 Session->data_chan_state = NO_STATE;
             }
         }
@@ -1383,7 +1323,111 @@ static int do_stateful_checks(FTP_SESSION *Session, SFSnortPacket *p,
 
     return iRet;
 }
+#ifdef TARGET_BASED
+/*
+ * Function: ftp_path_update(FTP_SESSION *ftpssn, FTP_CLIENT_REQ *req, int rsp_code,unsigned long memcap)
+ *
+ * Purpose: Update the Present working directory path in FTPSSN form change working directory command path
+ *
+ * Arguments: Session        => Pointer to session info
+ *            req            => Pointer to the request.
+ *            rsp_code       => Response code
+ *            memcap         => MEMCAP value for FTP
+ *
+ * Note: Windows FTP client doesn't support REST hence not handling “\” in the path.
+ *
+ * Returns: int => return code indicating error or success
+ *
+ * */
+static int ftp_path_update(FTP_SESSION *ftpssn, FTP_CLIENT_REQ *req, int rsp_code, FTPTELNET_GLOBAL_CONF *global_conf)
+{
+    if( ftpssn->cwd_path && rsp_code == 250 )
+    {
+        char       *path = NULL;
+        uint32_t   pwdlen = 0,token_len = 0;
+        char       *token;
+        pwdlen       = ftpssn->path_len;
+        if (!ftpssn->path)
+        {
+            ftpssn->path = (char *)sfmemcap_alloc(&global_conf->mc,DEFAULT_MEM_ALLOC);
 
+            if (ftpssn->path)
+                ftpssn->path_memory_alloc = DEFAULT_MEM_ALLOC;
+            else
+            {
+                ftpssn->path_memory_alloc = 0;
+                return 0;
+            }
+        }
+        if (ftpssn->cwd_path[0] == '/')
+        {
+            ftpssn->path[0] = '/';
+            ftpssn->path[1] = '\0';
+            pwdlen = 1;
+        }
+        token = strtok(ftpssn->cwd_path, "/");
+        while (token != NULL)
+        {
+            if (!strcmp(token, ".."))
+            {
+                int j = pwdlen - 2;
+                while (j > 0 && ftpssn->path[j] != '/')
+                    j--;
+                if (j != 0) {
+                    ftpssn->path[j+1] = '\0';
+                    pwdlen = j+1;
+                }
+                else
+                {   ftpssn->path[0] = '\0';
+                    pwdlen = 0;
+                }
+            }
+            else if (!strcmp(token, "."))
+            {
+                /* go to the next /
+                 * do nothing */
+            }
+            else
+            {
+                token_len = strlen(token);
+                if (pwdlen + token_len + 2 > PATH_MAX)
+                    break;
+                if (pwdlen + token_len + 2 > ftpssn->path_memory_alloc)
+                {
+                    unsigned long neededMemory = ((ftpssn->path_memory_alloc + token_len + 2 + (DEFAULT_MEM_ALLOC - 1)) / DEFAULT_MEM_ALLOC)*DEFAULT_MEM_ALLOC;
+                    path = (char *)sfmemcap_alloc(&global_conf->mc, neededMemory);
+                    if (path)
+                    {
+                        memcpy(path,ftpssn->path,pwdlen+1);
+                        sfmemcap_free(&global_conf->mc,ftpssn->path);
+                        ftpssn->path_memory_alloc = neededMemory;
+                        ftpssn->path = path;
+                    }
+                    else
+                    {
+                        sfmemcap_free(&global_conf->mc,ftpssn->path);
+                        ftpssn->path_memory_alloc = 0;
+                        ftpssn->path = NULL;
+                        return 0;
+                    }
+                }
+                if (ftpssn->path)
+                {
+                    strcat(ftpssn->path, token);
+                    pwdlen += token_len ;
+                    ftpssn->path[pwdlen++] = '/';
+                    ftpssn->path[pwdlen] = '\0';
+                }
+            }
+            token = strtok(NULL, "/");
+        }
+        ftpssn->path_len = pwdlen;
+        ftpssn->path_hash = _dpd.fileAPI->str_to_hash((uint8_t *)ftpssn->path,pwdlen);
+    }
+	return 1;
+}
+
+#endif
 /*
  * Function: check_ftp(FTP_SESSION *Session, Packet *p, int iMode)
  *
@@ -1423,6 +1467,7 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
     FTPTELNET_GLOBAL_CONF *global_conf = (FTPTELNET_GLOBAL_CONF *)sfPolicyUserDataGet(ftpssn->global_conf, ftpssn->policy_id);
     FTP_CLIENT_REQ *req;
     FTP_CMD_CONF *CmdConf = NULL;
+    FTP_DATA_SESSION *datassn;
 
     const unsigned char *read_ptr;
     const unsigned char *end = p->payload + p->payload_size;
@@ -1809,7 +1854,69 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
                     NULL, NULL);
                 iRet = FTPP_ALERT;
             }
+#ifdef TARGET_BASED
+             if (!(ftpssn->flags & FTP_FLG_MALWARE_ENABLED) && !(ftpssn->flags & FTP_FLG_MALWARE_CHECK) &&
+                     _dpd.fileAPI->file_config_malware_check(p->stream_session, ftp_data_app_id))
+             {
+                 ftpssn->flags |= FTP_FLG_MALWARE_ENABLED;
+                 ftpssn->flags |= FTP_FLG_MALWARE_CHECK;
+             }
+             if (ftpssn->flags & FTP_FLG_MALWARE_ENABLED)
+             {
+                 if (rsp_code == 257 && req->param_begin != NULL && req->param_size > 0 && req->param_size < PATH_MAX
+                         &&  !strncmp("PWD",ftpssn->client.request.cmd_begin,ftpssn->client.request.cmd_size))
+                 {
+                     int len = 0;
+#ifdef HAVE_MEMRCHR
+                     char *param_end = (char*)memrchr(req->param_begin,'"',req->param_size);
+#else
+                     char *param_end = NULL;
+                     char *tmp = req->param_begin;
+                     unsigned n = req->param_size;
 
+                     while ( (tmp = (char*)memchr((char*)tmp, '"', n)) )
+                     {
+                         param_end = tmp++;
+                         n = req->param_size - (tmp - req->param_begin);
+                     }
+#endif
+                     if (param_end)
+                         len = param_end - req->param_begin - 1;
+
+                     if (len > 0)
+                     {
+                         if (ftpssn->path_memory_alloc < len + 2)
+                         {
+                             unsigned long neededMemory = ((len + 2 + (DEFAULT_MEM_ALLOC - 1)) / DEFAULT_MEM_ALLOC ) * DEFAULT_MEM_ALLOC;
+
+                             if (ftpssn->path)
+                                 sfmemcap_free(&global_conf->mc,ftpssn->path);
+
+                             ftpssn->path = (char *)sfmemcap_alloc(&global_conf->mc, neededMemory);
+
+                             if (ftpssn->path)
+                                 ftpssn->path_memory_alloc = neededMemory;
+                             else
+                                 ftpssn->path_memory_alloc = 0;
+                         }
+
+                         if (ftpssn->path)
+                         {
+                             memcpy(ftpssn->path, req->param_begin + 1, len);
+                             ftpssn->path[len] = '\0';
+                             if (ftpssn->path[len-1] != '/')
+                             {
+                                 ftpssn->path[len++] = '/';
+                                 ftpssn->path[len] = '\0';
+                             }
+                             ftpssn->path_len = len;
+                             ftpssn->path_hash = _dpd.fileAPI->str_to_hash((uint8_t *)ftpssn->path,strlen(ftpssn->path));
+                         }
+                     }
+                 }
+                 ftp_path_update(ftpssn,  req, rsp_code, global_conf);
+             }
+#endif
             if (global_conf->inspection_type ==
                 FTPP_UI_CONFIG_STATEFUL)
             {
@@ -1850,6 +1957,34 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
             DEBUG_WRAP(DebugMessage(DEBUG_FTPTELNET, "FTP command: CMD: %.*s : "
                 "P len %d : P %.*s\n", req->cmd_size, req->cmd_begin,
                 req->param_size, req->param_size, req->param_begin));
+#ifdef TARGET_BASED
+            if ( (ftpssn->flags & FTP_FLG_MALWARE_ENABLED) && !strncmp("CWD",req->cmd_begin, req->cmd_size))
+            {
+                  if ((req->param_begin != NULL) && (req->param_size > 0))
+                  {
+                       if (req->param_size + 1 > ftpssn->cwd_memory_alloc)
+                       {
+                           unsigned long neededMemory = ((req->param_size + 1 + (DEFAULT_MEM_ALLOC - 1)) / DEFAULT_MEM_ALLOC) * DEFAULT_MEM_ALLOC;
+
+                           if (ftpssn->cwd_path)
+                               sfmemcap_free(&global_conf->mc,ftpssn->cwd_path);
+
+                           ftpssn->cwd_path  = (char *)sfmemcap_alloc(&global_conf->mc, neededMemory);
+
+                           if (ftpssn->cwd_path)
+                               ftpssn->cwd_memory_alloc = neededMemory;
+                           else
+                               ftpssn->cwd_memory_alloc = 0;
+                       }
+
+                       if (ftpssn->cwd_path)
+                       {
+                           memcpy(ftpssn->cwd_path, req->param_begin, req->param_size);
+                           ftpssn->cwd_path[req->param_size] = '\0';
+                       }
+                  }
+            }
+#endif
             if (CmdConf)
             {
                 if ((req->param_size > CmdConf->max_param_len))
@@ -1878,7 +2013,7 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
                         ftpssn->data_chan_state &= ~DATA_CHAN_PORT_CMD_ISSUED;
                     }
                 }
-                else if ((ftpssn->flags & FTP_FLG_MALWARE) && CmdConf->data_rest_cmd)
+                else if (CmdConf->data_rest_cmd)
                 {
                     if ((req->param_begin != NULL) && (req->param_size > 0))
                     {
@@ -1899,7 +2034,7 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
                 {
 #ifdef TARGET_BASED
                     /* If we are not ignoring the data channel OR file processing is enabled */
-                    if (!ftpssn->server_conf->data_chan || (_dpd.fileAPI->get_max_file_depth() > -1))
+                    if (!ftpssn->server_conf->data_chan || (_dpd.fileAPI->get_max_file_depth(_dpd.getCurrentSnortConfig(), false) > -1))
                     {
                         /* The following  check cleans up filename  for failed data
                          * transfers.  If  the  transfer had  been  successful  the
@@ -1924,10 +2059,25 @@ int check_ftp(FTP_SESSION  *ftpssn, SFSnortPacket *p, int iMode)
                                 memcpy(ftpssn->filename, req->param_begin, req->param_size);
                                 ftpssn->filename[req->param_size] = '\0';
                                 ftpssn->file_xfer_info = req->param_size;
-                            }
-
+                                 if (ftpssn->flags & FTP_FLG_MALWARE_ENABLED)
+                                 {
+                                     IP_COPY_VALUE(ftpssn->control_clientIP, GET_SRC_IP(p));
+                                     IP_COPY_VALUE(ftpssn->control_serverIP, GET_DST_IP(p));
+                                     ftpssn->control_serverPort = ntohs(p->tcp_header->destination_port);
+                                     ftpssn->control_clientPort = ntohs(p->tcp_header->source_port);
+                                     datassn = (FTP_DATA_SESSION *)ftpssn->datassn;
+                                     if(datassn)
+                                     {
+                                         datassn->path_hash = (_dpd.fileAPI->str_to_hash((uint8_t *)ftpssn->filename, strlen(ftpssn->filename))
+                                                                 + ftpssn->path_hash);
+                                     }
+                                  }
+                             }
                             // 0 for Download, 1 for Upload
                             ftpssn->data_xfer_dir = CmdConf->file_get_cmd ? false : true;
+                            FTP_DATA_SESSION *ftpdata = ftpssn->datassn;
+                            if(ftpdata)
+                                ftpdata->direction = ftpssn->data_xfer_dir;
                         }
                         else
                         {
