@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2011-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -278,6 +278,7 @@ static POP * POP_GetNewSession(SFSnortPacket *p, tSfPolicyId policy_id)
 {
     POP *ssn;
     POPConfig *pPolicyConfig = NULL;
+    int ret = 0;
 
     pPolicyConfig = (POPConfig *)sfPolicyUserDataGetCurrent(pop_config);
 
@@ -297,11 +298,19 @@ static POP * POP_GetNewSession(SFSnortPacket *p, tSfPolicyId policy_id)
     pop_ssn->mime_ssn.decode_conf = &(pop_eval_config->decode_conf);
     pop_ssn->mime_ssn.mime_mempool = pop_mime_mempool;
     pop_ssn->mime_ssn.log_mempool = pop_mempool;
-    //smtp_ssn->mime_ssn.mime_stats = &(smtp_stats.mime_stats);
+    pop_ssn->mime_ssn.mime_stats = &(pop_stats.mime_stats);
     pop_ssn->mime_ssn.methods = &(mime_methods);
 
-    if (_dpd.fileAPI->set_log_buffers(&(pop_ssn->mime_ssn.log_state), &(pPolicyConfig->log_config), pop_mempool) < 0)
+    if (( ret = _dpd.fileAPI->set_log_buffers(&(pop_ssn->mime_ssn.log_state), &(pPolicyConfig->log_config),pop_mempool, p->stream_session)) < 0)
     {
+        if( ret == -1 )
+        {
+            if(pop_stats.log_memcap_exceeded % 10000 == 0)
+            {
+                _dpd.logMsg("WARNING: POP  memcap exceeded.\n");
+            }
+            pop_stats.log_memcap_exceeded++;
+        }
         free(ssn);
         return NULL;
     }
@@ -339,6 +348,10 @@ static POP * POP_GetNewSession(SFSnortPacket *p, tSfPolicyId policy_id)
     ssn->config = pop_config;
     ssn->flow_id = 0;
     pPolicyConfig->ref_count++;
+    pop_stats.sessions++;
+    pop_stats.conc_sessions++;
+    if(pop_stats.max_conc_sessions < pop_stats.conc_sessions)
+       pop_stats.max_conc_sessions = pop_stats.conc_sessions;
 
     return ssn;
 }
@@ -505,6 +518,8 @@ static void POP_SessionFree(void *session_data)
         ssl_cb->session_free(pop->flow_id);
 
     free(pop);
+    if(pop_stats.conc_sessions)
+       pop_stats.conc_sessions--;
 }
 
 static int POP_FreeConfigsPolicy(
@@ -783,7 +798,7 @@ static void POP_ProcessServerPacket(SFSnortPacket *p)
         {
             DEBUG_WRAP(DebugMessage(DEBUG_POP, "DATA STATE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"););
             //ptr = POP_HandleData(p, ptr, end);
-            ptr = _dpd.fileAPI->process_mime_data(p, ptr, end, &(pop_ssn->mime_ssn), 0, true);
+            ptr = _dpd.fileAPI->process_mime_data(p, ptr, end, &(pop_ssn->mime_ssn), 0, true, "POP");
             continue;
         }
         POP_GetEOL(ptr, end, &eol, &eolm);
