@@ -1,7 +1,7 @@
 /* $Id */
 
 /*
- ** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ ** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  ** Copyright (C) 2011-2013 Sourcefire, Inc.
  **
  **
@@ -371,6 +371,7 @@ static void ReputationMaintenanceCheck(int signal, void *data)
 {
     DEBUG_WRAP(DebugMessage(DEBUG_REPUTATION, "Reputation Preprocessor Maintenance!\n"););
     PrintShmemMgmtInfo();
+    reputation_eval_config = sfPolicyUserDataGetDefault(reputation_config);
     if (SHMEM_SERVER_ID == _dpd.getSnortInstance())
     {
         ManageUnusedSegments();
@@ -765,6 +766,9 @@ static inline void ReputationProcess(SFSnortPacket *p)
 #ifdef POLICY_BY_ID_ONLY
         _dpd.inlineForceDropSession(p);
         flags |= SSNFLAG_FORCE_BLOCK;
+        if (*_dpd.pkt_tracer_enabled)
+            _dpd.addPktTrace(VERDICT_REASON_REPUTATION, snprintf(_dpd.trace, _dpd.traceMax, "Reputation: packet blacklisted, drop\n"));
+        else _dpd.addPktTrace(VERDICT_REASON_REPUTATION, 0);
 #endif
         // disable all preproc analysis and detection for this packet
         _dpd.disablePacketAnalysis(p);
