@@ -1,7 +1,7 @@
 /* $Id */
 
 /*
- ** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ ** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  ** Copyright (C) 2011-2013 Sourcefire, Inc.
  **
  **
@@ -60,6 +60,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+#ifdef REG_TEST
+#include "reg_test.h"
+#endif
+
 #include "profiler.h"
 #ifdef PERF_PROFILING
 PreprocStats sipPerfStats;
@@ -71,6 +75,10 @@ PreprocStats sipPerfStats;
 #include "sip_buffer_dump.h"
 #endif
 
+#ifdef SNORT_RELOAD
+#include "appdata_adjuster.h"
+#endif
+
 const int MAJOR_VERSION = 1;
 const int MINOR_VERSION = 1;
 const int BUILD_VERSION = 1;
@@ -94,6 +102,8 @@ int16_t sip_app_id = SFTARGET_UNKNOWN_PROTOCOL;
  */
 SIPData * SIPGetNewSession(SFSnortPacket *, tSfPolicyId);
 static void SIPInit( struct _SnortConfig *, char* );
+static bool SIPGlobalIsEnabled(struct _SnortConfig *sc, tSfPolicyUserContextId sip_config);
+static int SIPPolicyIsEnabled(struct _SnortConfig *sc, tSfPolicyUserContextId pContext, tSfPolicyId policyId, void* config);
 static int SIPCheckConfig(struct _SnortConfig *);
 static void FreeSIPData( void* );
 static inline int SIP_Process(SFSnortPacket *, SIPData*);
@@ -118,6 +128,15 @@ SIP_Stats sip_stats;
 SIPConfig *sip_eval_config;
 tSfPolicyUserContextId sip_config;
 
+#ifdef SNORT_RELOAD
+static APPDATA_ADJUSTER *ada;
+#endif
+
+static size_t SIP_NumSessions()
+{
+    return (size_t) numSessions;
+}
+
 #ifdef SNORT_RELOAD
 static void SIPReload(struct _SnortConfig *, char *, void **);
 static int SIPReloadVerify(struct _SnortConfig *, void *);
@@ -226,6 +245,33 @@ static void SIPInit(struct _SnortConfig *sc, char *argp)
     sfPolicyUserDataSetCurrent(sip_config, pPolicyConfig);
     SIP_RegRuleOptions(sc);
     ParseSIPArgs(pPolicyConfig, (u_char *)argp);
+#ifdef SNORT_RELOAD
+    pDefaultPolicyConfig = (SIPConfig *)sfPolicyUserDataGetDefault(sip_config);
+    //we don't know the order in which policies are init
+    //maybe default (policy 0) isn't init until after another policy is init
+    //however a default policy is guranteed
+    //avoid core
+    //Also, if SIP isn't enabled, then why waste memory?
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_RELOAD & getRegTestFlags())
+    {
+        printf("SIP-reload SIPInit-before : %p\n", ada);
+    }
+#endif
+    if (pDefaultPolicyConfig != NULL && ada == NULL && SIPGlobalIsEnabled(sc, sip_config))
+    {
+        ada = ada_init(SIP_NumSessions, PP_SIP, (size_t)pDefaultPolicyConfig->maxNumSessions);
+        if (ada == NULL)
+            DynamicPreprocessorFatalMessage("Could not allocate memory for SIP ada\n");
+    }
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_RELOAD & getRegTestFlags())
+    {
+        printf("SIP-reload SIPInit-after : %p\n", ada);
+    }
+#endif
+#endif
+
 }
 
 /*********************************************************************
@@ -507,6 +553,10 @@ SIPData * SIPGetNewSession(SFSnortPacket *packetp, tSfPolicyId policy_id)
                 STREAM_FLPOLICY_FOOTPRINT, SSN_DIR_BOTH, STREAM_FLPOLICY_SET_ABSOLUTE);
     }
 
+#ifdef SNORT_RELOAD
+    ada_add(ada, (void *)datap, packetp->stream_session);
+#endif
+
     datap->policy_id = policy_id;
     datap->config = sip_config;
     ((SIPConfig *)sfPolicyUserDataGetCurrent(sip_config))->ref_count++;
@@ -540,6 +590,10 @@ static void FreeSIPData( void* idatap )
     if (numSessions > 0)
         numSessions--;
 
+#ifdef SNORT_RELOAD
+    ada_appdata_freed( ada, idatap );
+#endif
+
     /*Free all the dialog data*/
     sip_freeDialogs(&ssn->dialogs);
 
@@ -692,6 +746,20 @@ static int SIPCheckPolicyConfig(struct _SnortConfig *sc, tSfPolicyUserContextId
     return 0;
 }
 
+static bool SIPGlobalIsEnabled(struct _SnortConfig *sc, tSfPolicyUserContextId config)
+{
+   return sfPolicyUserDataIterate(sc, config, SIPPolicyIsEnabled) != 0;
+}
+
+static int SIPPolicyIsEnabled(struct _SnortConfig *sc, tSfPolicyUserContextId pContext, tSfPolicyId policyId, void* config)
+{
+    SIPConfig *sip_policy_config = (SIPConfig *) config;
+    if (sip_policy_config == NULL || sip_policy_config->disabled)
+        return 0;
+
+    return 1;
+}
+
 int SIPCheckConfig(struct _SnortConfig *sc)
 {
     int rval;
@@ -708,6 +776,10 @@ static void SIPCleanExit(int signal, void *data)
     {
         SIPFreeConfig(sip_config);
         sip_config = NULL;
+#ifdef SNORT_RELOAD
+        ada_delete(ada);
+        ada = NULL;
+#endif
     }
 }
 static int SIPFreeConfigPolicy(
@@ -779,6 +851,7 @@ static void SIPReload(struct _SnortConfig *sc, char *args, void **new_config)
 {
     tSfPolicyUserContextId sip_swap_config = (tSfPolicyUserContextId)*new_config;
     tSfPolicyId policy_id = _dpd.getParserPolicy(sc);
+    SIPConfig *pDefaultPolicyConfig = NULL;
     SIPConfig * pPolicyConfig = NULL;
 
     if (sip_swap_config == NULL)
@@ -808,39 +881,67 @@ static void SIPReload(struct _SnortConfig *sc, char *args, void **new_config)
     sfPolicyUserDataSetCurrent(sip_swap_config, pPolicyConfig);
     SIP_RegRuleOptions(sc);
     ParseSIPArgs(pPolicyConfig, (u_char *)args);
+
+    pDefaultPolicyConfig = (SIPConfig *)sfPolicyUserDataGetDefault(sip_config);
+    //we don't know the order in which policies are init
+    //maybe default (policy 0) isn't init until after another policy is init
+    //however a default policy is guranteed
+    //avoid core
+    //Also, if SIP isn't enabled, then why waste memory?
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_RELOAD & getRegTestFlags())
+    {
+        printf("SIP-reload SIPReload-before : %p\n", ada);
+    }
+#endif
+    if (pDefaultPolicyConfig != NULL && SIPGlobalIsEnabled(sc, sip_swap_config) && ada == NULL)
+    {
+        ada = ada_init(SIP_NumSessions, PP_SIP, (size_t)pDefaultPolicyConfig->maxNumSessions);
+        if (ada == NULL)
+            DynamicPreprocessorFatalMessage("Could not allocate memory for SIP ada\n");
+    }
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_RELOAD & getRegTestFlags())
+    {
+        printf("SIP-reload SIPReload-after : %p\n", ada);
+    }
+#endif
 }
 
 static int SIPReloadVerify(struct _SnortConfig *sc, void *swap_config)
 {
     tSfPolicyUserContextId sip_swap_config = (tSfPolicyUserContextId)swap_config;
-    SIPConfig * pPolicyConfig = NULL;
-    SIPConfig * pCurrentConfig = NULL;
+    SIPConfig * default_swap_config = NULL;
+    SIPConfig * current_default_config = NULL;
     int rval;
 
     if (sip_swap_config == NULL)
         return 0;
 
-    pPolicyConfig = (SIPConfig *)sfPolicyUserDataGet(sip_swap_config, _dpd.getDefaultPolicy());
-    if ( pPolicyConfig == NULL )
-        return 0;
+    // validate each policy and do per policy initialization processing
+    if ((rval = sfPolicyUserDataIterate (sc, sip_swap_config, SIPCheckPolicyConfig)))
+        return rval;
+
+    default_swap_config = (SIPConfig *)sfPolicyUserDataGet(sip_swap_config, _dpd.getDefaultPolicy());
 
     if (sip_config != NULL)
     {
-        pCurrentConfig = (SIPConfig *)sfPolicyUserDataGet(sip_config, _dpd.getDefaultPolicy());
-
-        if (!pCurrentConfig) 
+        current_default_config = (SIPConfig *)sfPolicyUserDataGet(sip_config, _dpd.getDefaultPolicy());
+        //not possible
+        if (!current_default_config)
             return 0;
 
-        if (pPolicyConfig->maxNumSessions != pCurrentConfig->maxNumSessions)
+        tSfPolicyId policy_id = _dpd.getParserPolicy(sc);
+        if (!SIPGlobalIsEnabled(sc, sip_swap_config))
         {
-            _dpd.errMsg("SIP reload: Changing the max_sessions requires a restart.\n");
-            return -1;
+            ada_reload_disable(&ada, sc, "sip-disable-mem-dump", policy_id);
+        }
+        else if (SIPGlobalIsEnabled(sc, sip_config) && default_swap_config->maxNumSessions < current_default_config->maxNumSessions)
+        {
+            ada_reload_adjust_register(ada, policy_id, sc, "sip-mem-reloader",
+                    (size_t) default_swap_config->maxNumSessions);
         }
     }
-    
-    // validate each policy and do per policy initialization processing
-    if ((rval = sfPolicyUserDataIterate (sc, sip_config, SIPCheckPolicyConfig)))
-        return rval;
 
    return 0;
 }
