@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2005-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -77,6 +77,10 @@ int smtpDetectCalled = 0;
 
 #include "file_api.h"
 
+#ifdef REG_TEST
+#include "reg_test.h"
+#endif
+
 #ifdef DUMP_BUFFER
 #include "smtp_buffer_dump.h"
 #endif
@@ -112,6 +116,11 @@ static void _addServicesToStreamFilter(struct _SnortConfig *, tSfPolicyId);
 static int SMTPCheckConfig(struct _SnortConfig *);
 
 #ifdef SNORT_RELOAD
+static int SMTPMempoolFreeUsedBucket(MemPool *memory_pool);
+static unsigned SMTPReloadMimeMempoolAdjust(unsigned smtpMaxWork);
+static unsigned SMTPReloadLogMempoolAdjust(unsigned smtpMaxWork);
+static bool SMTPMimeReloadAdjust(bool idle, tSfPolicyId raPolicyId, void* userData);
+static bool SMTPLogReloadAdjust(bool idle, tSfPolicyId raPolicyId, void* userData);
 static void SMTPReload(struct _SnortConfig *, char *, void **);
 static int SMTPReloadVerify(struct _SnortConfig *, void *);
 static void * SMTPReloadSwap(struct _SnortConfig *, void *);
@@ -428,7 +437,8 @@ static int CheckFilePolicyConfig(
 {
     SMTPConfig *context = (SMTPConfig *)pData;
 
-    context->decode_conf.file_depth = _dpd.fileAPI->get_max_file_depth();
+     /* Use new Snort config to get the max file depth */
+     context->decode_conf.file_depth = _dpd.fileAPI->get_max_file_depth(sc, true);
     if (context->decode_conf.file_depth > -1)
         context->log_config.log_filename = 1;
     updateMaxDepth(context->decode_conf.file_depth, &context->decode_conf.max_depth);
@@ -457,7 +467,6 @@ static int SMTPCheckPolicyConfig(
         DynamicPreprocessorFatalMessage("Streaming & reassembly must be enabled "
                                         "for SMTP preprocessor\n");
     }
-
     return 0;
 }
 
@@ -520,6 +529,8 @@ static void SMTP_PrintStats(int exiting)
         _dpd.logMsg("  Total Non-Encoded MIME bytes extracted            : " STDu64 "\n", smtp_stats.mime_stats.decoded_bytes[DECODE_BITENC]);
         if ( smtp_stats.mime_stats.memcap_exceeded )
             _dpd.logMsg("  Sessions not decoded due to memory unavailability : " STDu64 "\n", smtp_stats.mime_stats.memcap_exceeded);
+        if ( smtp_stats.log_memcap_exceeded )
+            _dpd.logMsg("  SMTP Sessions fastpathed due to memcap exceeded: " STDu64 "\n", smtp_stats.log_memcap_exceeded);
     }
 
 }
@@ -603,11 +614,113 @@ static void SMTPReload(struct _SnortConfig *sc, char *args, void **new_config)
 #endif
 }
 
+static int SMTPMempoolFreeUsedBucket(MemPool *memory_pool)
+{
+    MemBucket *lru_bucket = NULL;
+
+    lru_bucket = mempool_get_lru_bucket(memory_pool);
+    if(lru_bucket)
+    {
+        /* Deleting least recently used SMTP session data here to adjust to new max_memory */
+        _dpd.sessionAPI->set_application_data(lru_bucket->scbPtr, PP_SMTP, NULL, NULL);
+        return 1;
+    }
+    return 0;
+}
+
+static unsigned SMTPReloadMimeMempoolAdjust(unsigned smtpMaxWork)
+{
+    int retVal;
+
+    /* deleting MemBucket from free list in SMTP Mime Mempool */
+    smtpMaxWork = mempool_prune_freelist(smtp_mime_mempool, smtp_mime_mempool->max_memory, smtpMaxWork);
+
+    if(!smtpMaxWork)
+       return 0;
+
+    for( ; smtpMaxWork && ((smtp_mime_mempool->used_memory + smtp_mime_mempool->free_memory) > smtp_mime_mempool->max_memory); smtpMaxWork--)
+    {
+        /* deleting least recently used MemBucket from Used list in SMTP Mime Mempool */
+        retVal = SMTPMempoolFreeUsedBucket(smtp_mime_mempool);
+        if(!retVal)
+           break;
+    }
+
+    return smtpMaxWork;
+}
+
+static unsigned SMTPReloadLogMempoolAdjust(unsigned smtpMaxWork)
+{
+    int retVal;
+
+    /* deleting MemBucket from free list in SMTP Log Mempool */
+    smtpMaxWork = mempool_prune_freelist(smtp_mempool, smtp_mempool->max_memory, smtpMaxWork);
+
+    if(!smtpMaxWork)
+       return 0;
+
+    for( ; smtpMaxWork && ((smtp_mempool->used_memory + smtp_mempool->free_memory) > smtp_mempool->max_memory); smtpMaxWork--)
+    {
+        /* deleting least recently used MemBucket from Used list in SMTP Log Mempool */
+        retVal = SMTPMempoolFreeUsedBucket(smtp_mempool);
+        if(!retVal)
+           break;
+    }
+
+    return smtpMaxWork;
+}
+
+static bool SMTPMimeReloadAdjust(bool idle, tSfPolicyId raPolicyId, void* userData)
+{
+    unsigned initialMaxWork = idle ? 512 : 5;
+    unsigned maxWork;
+
+    /* If new max_mime_mem is less than old configured max_mime_mem, need to adjust SMTP Mime Mempool.
+     * In order to adjust to new max_memory of mime mempool, delete buckets from free list.
+     * After deleting buckets from free list, still new max_memory is less than old value , delete buckets
+     * (least recently used i.e head node of used list )from used list till total memory reaches to new max_memory.
+     */
+    maxWork = SMTPReloadMimeMempoolAdjust(initialMaxWork);
+
+    if (maxWork == initialMaxWork)
+    {
+       smtp_stats.max_conc_sessions = smtp_stats.conc_sessions;
+       smtp_stats.mime_stats.memcap_exceeded = 0;
+       return true;
+    }
+    else
+       return false;
+}
+
+static bool SMTPLogReloadAdjust(bool idle, tSfPolicyId raPolicyId, void* userData)
+{
+    unsigned initialMaxWork = idle ? 512 : 5;
+    unsigned maxWork;
+
+    /* If new memcap is less than old configured memcap, need to adjust SMTP Log Mempool.
+     * In order to adjust to new max_memory of log mempool, delete buckets from free list.
+     * After deleting buckets from free list, still new max_memory is less than old value , delete buckets
+     * (least recently used i.e head node of used list )from used list till total memory reaches to new max_memory.
+     */
+    maxWork = SMTPReloadLogMempoolAdjust(initialMaxWork);
+
+    if (maxWork == initialMaxWork)
+    {
+        smtp_stats.max_conc_sessions = smtp_stats.conc_sessions;
+        smtp_stats.mime_stats.memcap_exceeded = 0;
+        return true;
+    }
+    else
+        return false;
+}
+
 static int SMTPReloadVerify(struct _SnortConfig *sc, void *swap_config)
 {
     tSfPolicyUserContextId smtp_swap_config = (tSfPolicyUserContextId)swap_config;
     SMTPConfig *config = NULL;
     SMTPConfig *configNext = NULL;
+    tSfPolicyId policy_id = 0;
+
 
     if (smtp_swap_config == NULL)
         return 0;
@@ -627,34 +740,27 @@ static int SMTPReloadVerify(struct _SnortConfig *sc, void *swap_config)
     sfPolicyUserDataIterate (sc, smtp_swap_config, SMTPCheckPolicyConfig);
     sfPolicyUserDataIterate (sc, smtp_swap_config, CheckFilePolicyConfig);
 
+    policy_id = _dpd.getParserPolicy(sc);
+
     if (smtp_mime_mempool != NULL)
     {
-        if(_dpd.fileAPI->is_decoding_conf_changed(&(configNext->decode_conf),
-                &(config->decode_conf), "SMTP"))
-        {
-            return -1;
-        }
+        /* If max_mime_mem changes, mime mempool need to be adjusted bcz mempool max_memory will be changed.
+         * Registering here to adjust Mime memory Pool when max_mime_mem changes.
+         */
+        if( configNext->decode_conf.max_mime_mem  < config->decode_conf.max_mime_mem )
+              _dpd.reloadAdjustRegister(sc, "SMTP-MIME-MEMPOOL", policy_id, &SMTPMimeReloadAdjust, NULL, NULL);
+
     }
 
     if (smtp_mempool != NULL)
     {
-        if (configNext == NULL)
+        if(configNext)
         {
-            _dpd.errMsg("SMTP reload: Changing the memcap or email_hdrs_log_depth requires a restart.\n");
-            return -1;
-        }
-        if (configNext->memcap != config->memcap)
-        {
-            _dpd.errMsg("SMTP reload: Changing the memcap requires a restart.\n");
-            return -1;
-        }
-        if (configNext->log_config.email_hdrs_log_depth & 7)
-            configNext->log_config.email_hdrs_log_depth += (8 - (configNext->log_config.email_hdrs_log_depth & 7));
-
-        if(configNext->log_config.email_hdrs_log_depth != config->log_config.email_hdrs_log_depth)
-        {
-            _dpd.errMsg("SMTP reload: Changing the email_hdrs_log_depth requires a restart.\n");
-            return -1;
+             /* If memcap cahnges, log mempool need to be adjusted bcz mempool max_mempory will be changed.
+              * Registering here to adjust Log memory Pool when memcap changes.
+              */
+             if (configNext->memcap < config->memcap)
+                   _dpd.reloadAdjustRegister(sc, "SMTP-MEMPOOL", policy_id, &SMTPLogReloadAdjust, NULL, NULL);
         }
     }
     else if(configNext != NULL)
@@ -693,6 +799,7 @@ static int SMTPReloadSwapPolicy(
 
 static void * SMTPReloadSwap(struct _SnortConfig *sc, void *swap_config)
 {
+    SMTPConfig *configNew = NULL, *configOld = NULL;
     tSfPolicyUserContextId smtp_swap_config = (tSfPolicyUserContextId)swap_config;
     tSfPolicyUserContextId old_config = smtp_config;
 
@@ -701,6 +808,48 @@ static void * SMTPReloadSwap(struct _SnortConfig *sc, void *swap_config)
 
     smtp_config = smtp_swap_config;
 
+    configOld = (SMTPConfig *)sfPolicyUserDataGet(old_config, _dpd.getDefaultPolicy());
+    configNew = (SMTPConfig *)sfPolicyUserDataGet(smtp_config, _dpd.getDefaultPolicy());
+
+    if(configNew)
+    {
+         if(smtp_mime_mempool)
+         {
+              if( (configOld->decode_conf.max_mime_mem != configNew->decode_conf.max_mime_mem) ||
+                  (configOld->decode_conf.max_depth != configNew->decode_conf.max_depth) )
+              {
+#ifdef REG_TEST
+                  _dpd.fileAPI->displayMimeMempool(smtp_mime_mempool,&(configOld->decode_conf), &(configNew->decode_conf));
+#endif
+                  /* Update the smtp_mime_mempool with new max_memmory and object size when max_mime_mem changes. */
+                  _dpd.fileAPI->update_mime_mempool(smtp_mime_mempool, configNew->decode_conf.max_mime_mem, configNew->decode_conf.max_depth);
+             }
+         }
+         if(smtp_mempool)
+         {
+             if( ( configOld->memcap != configNew->memcap) ||
+                 ( configOld->log_config.email_hdrs_log_depth != configNew->log_config.email_hdrs_log_depth ) )
+             {
+#ifdef REG_TEST
+                  if (REG_TEST_EMAIL_FLAG_LOG_MEMPOOL_ADJUST & getRegTestFlagsForEmail())
+                  {
+                        printf("[SMTP] Email Headers Log depth is: OLD VALUE # %u \n",configOld->log_config.email_hdrs_log_depth);
+                        printf("[SMTP] Setting Email Headers Log depth to # (NEW VALUE) %u \n",configNew->log_config.email_hdrs_log_depth);
+                        fflush(stdout);
+                  }
+                 _dpd.fileAPI->displayLogMempool(smtp_mempool, configOld->memcap, configNew->memcap);
+#endif
+                 /* Update the smtp_mempool with new max_memory and objest size when memcap changes. */
+                 _dpd.fileAPI->update_log_mempool(smtp_mempool, configNew->memcap, configNew->log_config.email_hdrs_log_depth);
+                 smtp_stats.log_memcap_exceeded = 0;
+            }
+         }
+
+#ifdef REG_TEST
+         _dpd.fileAPI->displayDecodeDepth(&(configOld->decode_conf), &(configNew->decode_conf));
+#endif
+    }
+
     sfPolicyUserDataFreeIterate (old_config, SMTPReloadSwapPolicy);
 
     if (sfPolicyUserPolicyGetActive(old_config) == 0)
