@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -46,7 +46,7 @@ PreprocStats sslpp_perf_stats;
 
 tSfPolicyUserContextId ssl_config = NULL;
 
-static void SSLFreeConfig(tSfPolicyUserContextId config);
+static void SSLFreeConfig(tSfPolicyUserContextId config, bool full_cleanup);
 static void SSLCleanExit(int, void *);
 static void SSLResetStats(int, void *);
 static int SSLPP_CheckConfig(struct _SnortConfig *);
@@ -756,7 +756,7 @@ static int SSLFreeConfigPolicy(
     return 0;
 }
 
-static void SSLFreeConfig(tSfPolicyUserContextId config)
+static void SSLFreeConfig(tSfPolicyUserContextId config, bool full_cleanup)
 {
     SSLPP_config_t *defaultConfig;
     ssl_callback_interface_t *ssl_cb = (ssl_callback_interface_t *)_dpd.getSSLCallback();
@@ -767,13 +767,11 @@ static void SSLFreeConfig(tSfPolicyUserContextId config)
 
     if(defaultConfig && ssl_cb)
     {
-        ssl_cb->policy_free(&(defaultConfig->current_handle), defaultConfig->reload_handle);
-        defaultConfig->reload_handle = NULL;
+        ssl_cb->policy_free(&(defaultConfig->current_handle), full_cleanup);
 #ifdef ENABLE_HA
         if(defaultConfig->ssl_ha_config)
         {
             SSLHAConfigFree(defaultConfig->ssl_ha_config);
-            defaultConfig->ssl_ha_config = NULL;
         }
 #endif
     }
@@ -789,7 +787,7 @@ static void SSLCleanExit(int signal, void *data)
 #ifdef ENABLE_HA
         SSLCleanHA();
 #endif
-        SSLFreeConfig(ssl_config);
+        SSLFreeConfig(ssl_config, true);
         ssl_config = NULL;
     }
 }
@@ -1047,6 +1045,6 @@ void SSLReloadSwapFree(void *data)
     if (data == NULL)
         return;
 
-    SSLFreeConfig((tSfPolicyUserContextId)data);
+    SSLFreeConfig((tSfPolicyUserContextId)data, false);
 }
 #endif
