@@ -1,5 +1,5 @@
 /*
- ** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ ** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  ** Copyright (C) 2012-2013 Sourcefire, Inc.
  **
  **
@@ -33,7 +33,9 @@
 void file_resume_block_init(void);
 void file_resume_block_cleanup(void);
 int file_resume_block_add_file(void *pkt, uint32_t file_sig, uint32_t timeout,
-        File_Verdict verdict, uint32_t file_type_id, uint8_t *signature);
+        File_Verdict verdict, uint32_t file_type_id, uint8_t *signature,
+        uint16_t cli_port, uint16_t srv_port, bool create_pinhole );
 File_Verdict file_resume_block_check(void *pkt, uint32_t file_sig);
+bool file_config_malware_check(void *ssnptr, uint16_t app_id);
 
 #endif 
