@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,13 @@
  ****************************************************************************/
 
 #include "file_segment_process.h"
-
 #include "parser.h"
 
+#ifdef REG_TEST
+#include "reg_test.h"
+#include <stdio.h>
+#endif
+
 extern FileSession* get_file_session(void *ssnptr);
 
 static inline void file_segment_free(FileCache *fileCache, FileSegment* file_segment)
@@ -401,7 +405,6 @@ FileCache *file_cache_create(uint64_t memcap, uint32_t cleanup_files)
 
         sfxhash_set_max_nodes( fileCache->hashTable, max_files );
         fileCache->file_segment_memcap = file_segment_memcap;
-        fileCache->max_file_depth = file_api->get_max_file_depth();
         fileCache->cleanup_files = cleanup_files;
     }
     else
@@ -488,11 +491,6 @@ int file_segment_process( FileCache *fileCache, void* p, uint64_t file_id,
     Packet *pkt = (Packet *)p;
     void *ssnptr = pkt->ssnptr;
 
-    if (!fileCache || (offset > fileCache->max_file_depth))
-    {
-        return 0;
-    }
-
     fileEntry = file_cache_get(fileCache, p, file_id, true);
 
     if (fileEntry == NULL)
@@ -520,6 +518,7 @@ int file_segment_process( FileCache *fileCache, void* p, uint64_t file_id,
                 || !fileEntry->context->file_signature_enabled )
         {
             /* Just check file type and signature */
+            update_file_session(ssnptr, fileCache, file_id, fileEntry->context);
             ret = _process_one_file_segment(p, fileEntry, file_data, data_size, file_size);
             return ret;
         }
@@ -566,4 +565,64 @@ FileCacheStatus *file_cache_status(FileCache *fileCache)
     return (&(fileCache->status));
 }
 
+static bool file_cache_prune_files(FileCache *fileCache, uint8_t *pWork)
+{
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+        printf("file-cache prunefiles-before %u \n", sfxhash_count(fileCache->hashTable));
+#endif
+    for (; *pWork > 0 && sfxhash_count(fileCache->hashTable) > fileCache->hashTable->max_nodes; (*pWork)--)
+        pruneFileCache(fileCache,NULL);
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+        printf("file-cache prunefiles-after %u \n", sfxhash_count(fileCache->hashTable));
+#endif
+    return sfxhash_count(fileCache->hashTable) <= fileCache->hashTable->max_nodes;
+}
+
+static bool file_cache_prune_segment(FileCache *fileCache, uint8_t *pWork)
+{
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+        printf("file-cache prunesegment-before %"PRIu64" \n",
+            fileCache->status.segment_mem_in_use);
+#endif
+    for (; *pWork > 0 && fileCache->status.segment_mem_in_use > fileCache->file_segment_memcap; (*pWork)--)
+        pruneFileCache(fileCache,NULL);
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+        printf("file-cache prunesegment-after %"PRIu64" \n",
+            fileCache->status.segment_mem_in_use);
+#endif
+    return fileCache->status.segment_mem_in_use <= fileCache->file_segment_memcap;
+}
+
+bool file_cache_shrink_to_memcap(FileCache *fileCache, uint8_t *pWork)
+{
+    if (fileCache == NULL)
+        return true;
+
+    bool cache_shrunk = file_cache_prune_files(fileCache, pWork) &&
+                        file_cache_prune_segment(fileCache, pWork);
+#ifdef REG_TEST
+    if (cache_shrunk && REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+       printf("file-cache done 1\n");
+#endif
+    return cache_shrunk;
+}
+
+void file_cache_set_memcap(FileCache *fileCache, uint64_t memcap)
+{
+    if (fileCache == NULL)
+        return;
+    sfxhash_set_max_nodes(fileCache->hashTable, get_max_files_from_memcap(memcap/2));
+    fileCache->file_segment_memcap = memcap/2;
+#ifdef REG_TEST
+    if (REG_TEST_FLAG_FILE_CACHE & getRegTestFlags())
+    {
+        printf("file-cache mem-files %"PRIu32" \n", get_max_files_from_memcap(memcap/2));
+        printf("file-cache mem-segment %"PRIu64" \n", memcap/2);
+    }
+#endif
+}
 
