@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ * Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  * Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,7 +44,6 @@ typedef struct _FileCache
     uint32_t cleanup_files;
     FileCacheStatus status;
     uint64_t file_segment_memcap;
-    uint64_t max_file_depth;
 } FileCache;
 
 typedef struct _FileSegment
@@ -89,6 +88,9 @@ void file_cache_free(FileCache *fileCache);
 /* Get the status of file cache*/
 FileCacheStatus *file_cache_status(FileCache *fileCache);
 
+bool file_cache_shrink_to_memcap(FileCache *fileCache, uint8_t *pWork);
+void file_cache_set_memcap(FileCache *fileCache, uint64_t memcap);
+
 /* Add/update a file entry in the file cache*/
 void *file_cache_update_entry (FileCache *fileCache, void* p, uint64_t file_id,
         uint8_t *file_name, uint32_t file_name_size, uint64_t file_size);
