@@ -1,7 +1,7 @@
 /*
  **
  **
- **  Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ **  Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  **  Copyright (C) 2012-2013 Sourcefire, Inc.
  **
  **  This program is free software; you can redistribute it and/or modify
@@ -50,21 +50,17 @@
 #include "file_service.h"
 #include "file_segment_process.h"
 
-static bool file_type_id_enabled = false;
-static bool file_signature_enabled = false;
-static bool file_capture_enabled = false;
-static bool file_processing_initiated = false;
 static bool file_type_force = false;
 
 static uint32_t file_config_version = 0;
-static File_policy_callback_func file_policy_cb = NULL;
-File_type_callback_func  file_type_cb = NULL;
-File_signature_callback_func file_signature_cb = NULL;
-Log_file_action_func log_file_action = NULL;
+
+FileServiceConfig cur_config;
+static FileServiceConfig new_config;
+static FileServiceConfig init_config;
 
 /*Main File Processing functions */
 static int file_process(void* ssnptr, uint8_t* file_data, int data_size,
-        FilePosition position, bool upload, bool suspend_block_verdict);
+        FilePosition position, bool upload, bool suspend_block_verdict, bool do_flush);
 
 /*File properties*/
 static int get_file_name(void* ssnptr, uint8_t **fname, uint32_t *name_size);
@@ -78,12 +74,13 @@ static void set_file_name(void* ssnptr, uint8_t * fname, uint32_t name_size,
 static void set_file_direction(void* ssnptr, bool upload);
 
 static void set_file_policy_callback(File_policy_callback_func);
-static void enable_file_type(File_type_callback_func );
-static void enable_file_signature (File_signature_callback_func);
-static void enable_file_capture(File_signature_callback_func );
+static void enable_file_type(struct _SnortConfig* sc, File_type_callback_func);
+static void enable_file_signature(struct _SnortConfig* sc, File_signature_callback_func);
+static void enable_file_capture(struct _SnortConfig* sc, File_signature_callback_func );
 static void set_file_action_log_callback(Log_file_action_func);
 
-static int64_t get_max_file_depth(void);
+static int64_t get_max_file_depth(struct _SnortConfig *snort_conf, bool next);
+static bool is_file_signature_enabled(void);
 
 static uint32_t str_to_hash(uint8_t *str, int length );
 
@@ -112,6 +109,7 @@ static int64_t get_max_file_capture_size(void *ssn);
 static void file_session_free(void *session_data);
 extern FileEntry *file_cache_get(FileCache *fileCache, void* p, uint64_t file_id,
     bool can_create);
+static FileCharEncoding get_character_encoding(uint8_t *buffer, uint32_t length);
 
 FileAPI fileAPI;
 FileAPI* file_api = NULL;
@@ -135,8 +133,19 @@ void init_fileAPI(void)
     fileAPI.enable_file_signature = &enable_file_signature;
     fileAPI.enable_file_capture = &enable_file_capture;
     fileAPI.set_file_action_log_callback = &set_file_action_log_callback;
+    fileAPI.install_file_service = &FileServiceInstall;
     fileAPI.get_max_file_depth = &get_max_file_depth;
+    fileAPI.is_file_signature_enabled = &is_file_signature_enabled;
     fileAPI.set_log_buffers = &set_log_buffers;
+#ifdef SNORT_RELOAD
+    fileAPI.update_mime_mempool = &update_mime_mempool;
+    fileAPI.update_log_mempool = &update_log_mempool;
+#ifdef REG_TEST
+    fileAPI.displayMimeMempool = &displayMimeMempool;
+    fileAPI.displayLogMempool = &displayLogMempool;
+    fileAPI.displayDecodeDepth = &displayDecodeDepth;
+#endif
+#endif
     fileAPI.init_mime_mempool = &init_mime_mempool;
     fileAPI.init_log_mempool=  &init_log_mempool;
     fileAPI.file_resume_block_add_file = &file_resume_block_add_file;
@@ -149,7 +158,6 @@ void init_fileAPI(void)
     fileAPI.process_mime_data = &process_mime_data;
     fileAPI.free_mime_session = &free_mime_session;
     fileAPI.is_decoding_enabled = &is_decoding_enabled;
-    fileAPI.is_decoding_conf_changed = &is_decoding_conf_changed;
     fileAPI.check_decoding_conf = &check_decode_config;
     fileAPI.is_mime_log_enabled = &is_mime_log_enabled;
     fileAPI.finalize_mime_position = &finalize_mime_position;
@@ -179,44 +187,75 @@ void init_fileAPI(void)
     fileAPI.file_cache_create = &file_cache_create;
     fileAPI.file_cache_free = &file_cache_free;
     fileAPI.file_cache_status = &file_cache_status;
+    fileAPI.file_config_malware_check = &file_config_malware_check;
+    fileAPI.get_character_encoding = &get_character_encoding;
+    fileAPI.file_cache_shrink_to_memcap = &file_cache_shrink_to_memcap;
+    fileAPI.file_cache_set_memcap = &file_cache_set_memcap;
     file_api = &fileAPI;
     init_mime();
 }
 
-void FileAPIPostInit (void)
+
+#if defined(DEBUG_MSGS) || defined (REG_TEST)
+static void printFileServiceChanges()
 {
-    FileConfig *file_config = (FileConfig *)(snort_conf->file_config);
+   FileConfig *file_config =  (FileConfig *)(snort_conf->file_config);
+   if(cur_config.file_signature_enabled != new_config.file_signature_enabled)
+       printf("File service Install: file_signature %s\n",new_config.file_signature_enabled ? "enabled":"disabled");
+   if(cur_config.file_type_id_enabled != new_config.file_type_id_enabled)
+       printf("File service Install: file_type %s\n",new_config.file_type_id_enabled ? "enabled":"disabled");
+   if(cur_config.file_capture_enabled != new_config.file_capture_enabled)
+       printf("File service Install: file_capture %s\n",new_config.file_capture_enabled ? "enabled":"disabled");
 
-    if (file_type_id_enabled || file_signature_enabled || file_capture_enabled)
-    {
-        if (!file_config)
-        {
-            file_config =  file_service_config_create();
-            snort_conf->file_config = file_config;
-        }
-    }
-
-    if ( file_capture_enabled)
-        file_capture_init_mempool(file_config->file_capture_memcap,
-                file_config->file_capture_block_size);
-
-    if ( stream_api && file_signature_enabled )
-        s_cb_id = stream_api->register_event_handler(file_signature_callback);
-
-#ifdef SNORT_RELOAD
-    file_sevice_reconfig_set(false);
+   if (!file_config)
+       return;
+   if(new_config.file_type_id_enabled)
+       printf("File service Install: file_type_depth:%u \n",(unsigned)file_config->file_type_depth);
+   if(new_config.file_signature_enabled)
+       printf("File service Install: file_signature_depth:%u \n",(unsigned)file_config->file_signature_depth);
+   fflush(stdout);
+}
 #endif
 
+void FileServiceInstall(void)
+{
+#if defined(DEBUG_MSGS) || defined (REG_TEST)
+    printFileServiceChanges();
+#endif
+    cur_config = new_config;
+    new_config = init_config;
+
+    if (stream_api)
+    {
+        if(cur_config.file_signature_enabled &&
+                !s_cb_id )
+            s_cb_id = stream_api->register_event_handler(file_signature_callback);
+    }
 }
 
-static void start_file_processing(void)
+static void start_file_processing(struct _SnortConfig* sc, bool capture)
 {
+    static bool file_processing_initiated = false;
+
     if (!file_processing_initiated)
     {
         file_resume_block_init();
         RegisterPreprocStats("file", print_file_stats);
         file_processing_initiated = true;
     }
+
+    if (!sc)
+        sc = snort_conf;
+
+    if (!sc->file_config)
+        sc->file_config = file_service_config_create();
+
+    if (capture)
+    {
+        FileConfig* file_config = sc->file_config;
+        file_capture_init_mempool(file_config->file_capture_memcap,
+                file_config->file_capture_block_size);
+    }
 }
 
 void free_file_config(void *conf)
@@ -332,21 +371,21 @@ static void file_session_free(void *session_data)
 
 static void init_file_context(void *ssnptr, bool upload, FileContext *context)
 {
-    context->file_type_enabled = file_type_id_enabled;
-    context->file_signature_enabled = file_signature_enabled;
-    context->file_capture_enabled = file_capture_enabled;
+    context->file_type_enabled = cur_config.file_type_id_enabled;
+    context->file_signature_enabled = cur_config.file_signature_enabled;
+    context->file_capture_enabled = cur_config.file_capture_enabled;
     file_direction_set(context,upload);
     file_stats.files_total++;
 #ifdef TARGET_BASED
     /* Check file policy to see whether we want to do either file type, file
      * signature,  or file capture
      * Note: this happen only on the start of session*/
-    if (file_policy_cb)
+    if (cur_config.file_policy_cb)
     {
         uint32_t policy_flags = 0;
         context->app_id = session_api->get_application_protocol_id(ssnptr);
 
-        policy_flags = file_policy_cb(ssnptr, context->app_id, upload);
+        policy_flags = cur_config.file_policy_cb(ssnptr, context->app_id, upload);
 
         if ( !file_type_force && !(policy_flags & ENABLE_FILE_TYPE_IDENTIFICATION) )
                 context->file_type_enabled = false;
@@ -419,7 +458,7 @@ static inline void updateFileSize(FileContext* context, int data_size,
     context->processed_bytes += data_size;
     if ((position == SNORT_FILE_END) || (position == SNORT_FILE_FULL))
     {
-        if (get_max_file_depth() == (int64_t)context->processed_bytes)
+        if (get_max_file_depth(snort_conf, false) == (int64_t)context->processed_bytes)
             context->file_size = 0;
         else
             context->file_size = context->processed_bytes;
@@ -467,8 +506,12 @@ static inline void add_file_to_block(Packet *p, File_Verdict verdict,
         file_sig = str_to_hash(buf, len);
         file_resume_block_add_file(p, file_sig,
                 (uint32_t)file_config->file_block_timeout,
-                verdict, file_type_id, signature);
+                verdict, file_type_id, signature, 0, 0, true);
     }
+    if (pkt_trace_enabled)
+        addPktTraceData(VERDICT_REASON_FILE, snprintf(trace_line, MAX_TRACE_LINE,
+            "File Process: %s %s\n", getPktTraceActMsg(), (buf && len)? (char *)buf : ""));
+    else addPktTraceData(VERDICT_REASON_FILE, 0);
 }
 /*
  * Check HTTP partial content header
@@ -513,12 +556,13 @@ static inline void _file_signature_lookup(FileContext* context,
     Packet *pkt = (Packet *)p;
     void *ssnptr = pkt->ssnptr;
 
-    if (file_signature_cb)
+    if (cur_config.file_signature_cb)
     {
-        verdict = file_signature_cb(p, ssnptr, context->sha256,
+        verdict = cur_config.file_signature_cb(p, ssnptr, context->sha256,
                 context->file_size, &(context->file_state), context->upload,
                 context->file_id);
-        file_stats.verdicts_signature[verdict]++;
+        if(context->file_state.sig_state != FILE_SIG_FLUSH)
+            file_stats.verdicts_signature[verdict]++;
     }
 
     if (suspend_block_verdict)
@@ -526,7 +570,7 @@ static inline void _file_signature_lookup(FileContext* context,
 
     context->verdict = verdict;
 
-    if (verdict == FILE_VERDICT_LOG )
+    if ((verdict == FILE_VERDICT_LOG ) && (context->file_state.sig_state != FILE_SIG_FLUSH))
     {
         file_eventq_add(GENERATOR_FILE_SIGNATURE, FILE_SIGNATURE_SHA256,
                 FILE_SIGNATURE_SHA256_STR, RULE_TYPE__ALERT);
@@ -544,11 +588,24 @@ static inline void _file_signature_lookup(FileContext* context,
             {
                 if( !Active_DAQRetryPacket(pkt) )
                     Active_ForceDropPacket();
-
+                if (pkt_trace_enabled)
+                {
+                    addPktTraceData(VERDICT_REASON_FILE, snprintf(trace_line, MAX_TRACE_LINE,
+                        "File Process: malware detected, gid %u, sid %u, %s\n",
+                        GENERATOR_FILE_SIGNATURE, FILE_SIGNATURE_SHA256, getPktTraceActMsg()));
+                }
+                else addPktTraceData(VERDICT_REASON_FILE, 0);
                 return;
             }
             /*Timeout, let packet go through OR block based on config*/
             context->file_signature_enabled = false;
+            if (pkt_trace_enabled)
+            {
+                addPktTraceData(VERDICT_REASON_FILE, snprintf(trace_line, MAX_TRACE_LINE,
+                            "File Process: file signature lookup verdict pending timeout, %s\n", getPktTraceActMsg()));
+            }
+            else addPktTraceData(VERDICT_REASON_FILE, 0);
+
             if (file_config && file_config->block_timeout_lookup)
                 file_eventq_add(GENERATOR_FILE_SIGNATURE, FILE_SIGNATURE_SHA256,
                         FILE_SIGNATURE_SHA256_STR, RULE_TYPE__REJECT);
@@ -565,7 +622,12 @@ static inline void _file_signature_lookup(FileContext* context,
 
             if( !Active_DAQRetryPacket(pkt) )
                 Active_ForceDropPacket();
-
+            if (pkt_trace_enabled)
+            {
+                addPktTraceData(VERDICT_REASON_FILE, snprintf(trace_line, MAX_TRACE_LINE,
+                    "File Process: can't decide verdict and waiting, %s\n", getPktTraceActMsg()));
+            }
+            else addPktTraceData(VERDICT_REASON_FILE, 0);
             stream_api->set_event_handler(ssnptr, s_cb_id, SE_REXMIT);
             save_to_pending_context(ssnptr);
             return;
@@ -579,7 +641,8 @@ static inline void _file_signature_lookup(FileContext* context,
         return;
     }
 
-    finish_signature_lookup(context);
+    if(context->file_state.sig_state != FILE_SIG_FLUSH)
+        finish_signature_lookup(context);
 }
 
 static inline void finish_signature_lookup(FileContext *context)
@@ -711,7 +774,8 @@ static void file_signature_callback(Packet* p)
 
 static bool is_file_service_enabled()
 {
-    return (file_type_id_enabled || file_signature_enabled);
+    return (cur_config.file_type_id_enabled ||
+            cur_config.file_signature_enabled);
 }
 
 /*
@@ -724,10 +788,12 @@ static int process_file_context(FileContext *context, void *p, uint8_t *file_dat
 {
     Packet *pkt = (Packet *)p;
     void *ssnptr = pkt->ssnptr;
+    bool file_capture_enabled = false;
 
     if (!context)
         return 0;
 
+    file_capture_enabled = context->file_capture_enabled;
     set_current_file_context(ssnptr, context);
     file_stats.file_data_total += data_size;
 
@@ -775,9 +841,9 @@ static int process_file_context(FileContext *context, void *p, uint8_t *file_dat
 
         if (context->file_type_id != SNORT_FILE_TYPE_CONTINUE)
         {
-            if (file_type_cb)
+            if (cur_config.file_type_cb)
             {
-                verdict = file_type_cb(p, ssnptr, context->file_type_id,
+                verdict = cur_config.file_type_cb(p, ssnptr, context->file_type_id,
                         context->upload, context->file_id);
                 file_stats.verdicts_type[verdict]++;
                 context->verdict = verdict;
@@ -858,7 +924,16 @@ static int process_file_context(FileContext *context, void *p, uint8_t *file_dat
             FILE_REG_DEBUG_WRAP(if (context->sha256) file_sha256_print(context->sha256);)
             _file_signature_lookup(context, p, false, suspend_block_verdict);
         }
-
+        else if(context->file_state.sig_state == FILE_SIG_FLUSH)
+        {
+            _file_signature_lookup(context, p, false, suspend_block_verdict);
+            context->file_signature_enabled = true;
+            context->file_capture_enabled = file_capture_enabled;
+            if((context->verdict == FILE_VERDICT_BLOCK) || (context->verdict == FILE_VERDICT_REJECT))
+            {
+                FILE_REG_DEBUG_WRAP(if (context->sha256) file_sha256_print(context->sha256);)
+            }
+        }
     }
     else
     {
@@ -873,7 +948,7 @@ static int process_file_context(FileContext *context, void *p, uint8_t *file_dat
  *    0: ignore this file
  */
 static int file_process( void* p, uint8_t* file_data, int data_size,
-        FilePosition position, bool upload, bool suspend_block_verdict)
+        FilePosition position, bool upload, bool suspend_block_verdict, bool do_flush)
 {
     FileContext* context;
 
@@ -887,6 +962,17 @@ static int file_process( void* p, uint8_t* file_data, int data_size,
     FILE_REG_DEBUG_WRAP(DumpHexFile(stdout, file_data, data_size);)
 
     context = find_main_file_context(p, position, upload);
+    if((context->file_state.sig_state == FILE_SIG_FLUSH) && context->sha256)
+    {
+        free(context->sha256);
+        context->sha256 = NULL;
+    }
+    if(do_flush)
+        context->file_state.sig_state = FILE_SIG_FLUSH;
+    else
+    {
+        context->file_state.sig_state = FILE_SIG_PROCESSING;
+    }
     return process_file_context(context, p, file_data, data_size, position,
             suspend_block_verdict);
 }
@@ -938,7 +1024,7 @@ static uint8_t *get_file_sig_sha256(void* ssnptr)
 
 static void set_file_policy_callback(File_policy_callback_func policy_func_cb)
 {
-    file_policy_cb = policy_func_cb;
+    new_config.file_policy_cb = policy_func_cb;
 }
 
 /*
@@ -949,101 +1035,88 @@ static void set_file_policy_callback(File_policy_callback_func policy_func_cb)
  * TBD: Remove per-context "file_type_enabled" checking to simplify implementation.
  *
  */
-static void enable_file_type(File_type_callback_func callback)
+static void enable_file_type(struct _SnortConfig* sc, File_type_callback_func callback)
 {
-    if (!file_type_id_enabled)
-    {
-        file_type_id_enabled = true;
-#ifdef SNORT_RELOAD
-        file_sevice_reconfig_set(true);
-#endif
-        start_file_processing();
-        LogMessage("File service: file type enabled.\n");
-    }
+    new_config.file_type_id_enabled = true;
+    start_file_processing(sc, false);
+    LogMessage("File service: file type enabled.\n");
 
-    if ( callback == NULL )
+    if (!callback)
     {
         file_type_force = true;
     }
-    else if ( file_type_cb == NULL )
+    else if(!new_config.file_type_cb)
     {
-        file_type_cb = callback;
+        new_config.file_type_cb = callback;
     }
-    else if ( file_type_cb != callback )
+    else if(new_config.file_type_cb != callback)
     {
         FatalError("Attempt to register multiple file_type callbacks.");
     }
 }
 
 /* set file signature callback function*/
-static inline void _update_file_sig_callback(File_signature_callback_func cb)
+static void enable_file_signature(struct _SnortConfig* sc, File_signature_callback_func callback)
 {
-    if(!file_signature_cb)
+
+    new_config.file_signature_enabled = true;
+    start_file_processing(sc, false);
+
+    if(!new_config.file_signature_cb)
     {
-        file_signature_cb = cb;
+        new_config.file_signature_cb = callback;
+        LogMessage("File service: file signature enabled.\n");
     }
-    else if (file_signature_cb != cb)
+    else if(new_config.file_signature_cb != callback)
     {
         WarningMessage("File service: signature callback redefined.\n");
     }
 }
 
-static void enable_file_signature(File_signature_callback_func callback)
-{
-    _update_file_sig_callback(callback);
-
-    if (!file_signature_enabled)
-    {
-        file_signature_enabled = true;
-#ifdef SNORT_RELOAD
-        file_sevice_reconfig_set(true);
-#endif
-        start_file_processing();
-        LogMessage("File service: file signature enabled.\n");
-    }
-}
-
 /* Enable file capture, also enable file signature */
-static void enable_file_capture(File_signature_callback_func callback)
+static void enable_file_capture(struct _SnortConfig* sc, File_signature_callback_func callback)
 {
-    if (!file_capture_enabled)
-    {
-        file_capture_enabled = true;
-#ifdef SNORT_RELOAD
-        file_sevice_reconfig_set(true);
-#endif
-        LogMessage("File service: file capture enabled.\n");
-        /* Enable file signature*/
-        enable_file_signature(callback);
-    }
+    new_config.file_capture_enabled = true;
+    LogMessage("File service: file capture enabled.\n");
+    start_file_processing(sc, true);
+    /* Enable file signature*/
+    enable_file_signature(sc, callback);
 }
 
 static void set_file_action_log_callback(Log_file_action_func log_func)
 {
-    log_file_action = log_func;
+    new_config.log_file_action = log_func;
 }
 
 /* Get maximal file depth based on configuration
  * This function must be called after all file services are configured/enabled.
  */
-static int64_t get_max_file_depth(void)
+static int64_t get_max_file_depth(struct _SnortConfig *sc, bool next)
 {
-    FileConfig *file_config =  (FileConfig *)(snort_conf->file_config);
+    FileConfig *file_config;
+    FileServiceConfig *fs_config;
+
+    if (!sc)
+        sc = snort_conf;
+    file_config =  (FileConfig *)(sc->file_config);
+    fs_config = next ? &new_config:&cur_config;
 
     if (!file_config)
         return -1;
 
-    if (file_config->file_depth)
+     /* If next is set, proceed further to check the depth */
+    if (!next && file_config->file_depth)
         return file_config->file_depth;
 
     file_config->file_depth = -1;
 
-    if (file_type_id_enabled)
+
+    if (fs_config->file_type_id_enabled)
     {
         file_config->file_depth = file_config->file_type_depth;
     }
 
-    if (file_signature_enabled)
+    if (fs_config->file_signature_enabled)
     {
         if (file_config->file_signature_depth > file_config->file_depth)
             file_config->file_depth = file_config->file_signature_depth;
@@ -1061,6 +1134,11 @@ static int64_t get_max_file_depth(void)
     }
 }
 
+static bool is_file_signature_enabled()
+{
+    return cur_config.file_signature_enabled;
+}
+
 static FilePosition get_file_position(void *pkt)
 {
     FilePosition position = SNORT_FILE_POSITION_UNKNOWN;
@@ -1162,3 +1240,36 @@ static uint32_t str_to_hash(uint8_t *str, int length )
     final(a,b,c);
     return c;
 }
+bool file_config_malware_check(void *ssnptr, uint16_t app_id)
+{
+	uint32_t policy_flags = 0;
+	if (cur_config.file_policy_cb)
+	{
+        //Upload
+	    policy_flags = cur_config.file_policy_cb(ssnptr, app_id, true);
+        if ( (policy_flags & ENABLE_FILE_SIGNATURE_SHA256) || (policy_flags & ENABLE_FILE_TYPE_IDENTIFICATION) )
+            return true;
+
+        //Download
+	        policy_flags = cur_config.file_policy_cb(ssnptr, app_id, false);
+        if ( (policy_flags & ENABLE_FILE_SIGNATURE_SHA256) || (policy_flags & ENABLE_FILE_TYPE_IDENTIFICATION) )
+            return true;
+   }
+   return false;
+}
+
+static FileCharEncoding get_character_encoding(uint8_t *buffer, uint32_t length)
+{
+    FileCharEncoding encoding = SNORT_CHAR_ENCODING_ASCII;
+
+    if(length > UTF_16_LE_BOM_LEN)
+    {
+        if(memcmp(buffer, UTF_16_LE_BOM, UTF_16_LE_BOM_LEN) == 0)
+            encoding = SNORT_CHAR_ENCODING_UTF_16LE;
+        else if(memcmp(buffer, UTF_16_BE_BOM, UTF_16_BE_BOM_LEN) == 0)
+            encoding = SNORT_CHAR_ENCODING_UTF_16BE;
+    }
+
+    return encoding;
+}
+
