@@ -1,7 +1,7 @@
 /*
  **
  **
- **  Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+ **  Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
  **  Copyright (C) 2013-2013 Sourcefire, Inc.
  **
  **  This program is free software; you can redistribute it and/or modify
@@ -41,8 +41,9 @@
 FileStats file_stats;
 
 #if defined(DEBUG_MSGS) || defined (REG_TEST)
+#include <locale.h>
 #define MAX_CONTEXT_INFO_LEN 1024
-void printFileContext (FileContext* context)
+void printFileContext(FileContext* context)
 {
     char buf[MAX_CONTEXT_INFO_LEN + 1];
     int unused;
@@ -55,33 +56,53 @@ void printFileContext (FileContext* context)
         return;
     }
     unused = sizeof(buf) - 1;
-    used = snprintf(cur, unused, "File name: ");
 
-    if (used < 0)
+    printf("File name: ");
+    if(context->file_name && (context->file_name_size > 0))
     {
-        printf("Fail to output file context\n");
-        return;
-    }
-    unused -= used;
-    cur += used;
-
-    if ((context->file_name_size > 0) && (unused > (int) context->file_name_size))
-    {
-        uint32_t i;
-        uint32_t size = context->file_name_size;
-        for (i = 0; i < size; i++)
+        FileCharEncoding encoding = file_api->get_character_encoding(context->file_name, context->file_name_size);
+        if((encoding == SNORT_CHAR_ENCODING_UTF_16LE) || (encoding == SNORT_CHAR_ENCODING_UTF_16BE))
         {
-            if (isprint((int)context->file_name[i]))
-                cur[i] = (char)context->file_name[i];
-            else
-                cur[i] = '.';
+            setlocale(LC_ALL, "");
+            int i;
+            for (i = 2; i < context->file_name_size; i+=2)
+            {
+                uint16_t value = 0;
+                if(encoding == SNORT_CHAR_ENCODING_UTF_16LE)
+                    value = context->file_name[i] + (context->file_name[i + 1] << 8);
+                else if(encoding == SNORT_CHAR_ENCODING_UTF_16BE)
+                    value = (context->file_name[i] << 8) + context->file_name[i + 1];
+
+                if(0 == value)
+                    break;
+                printf("%lc", value);
+            }
         }
+        else if(encoding == SNORT_CHAR_ENCODING_ASCII)
+        {
+            if (unused > (int) context->file_name_size)
+            {
+                uint32_t i;
+                uint32_t size = context->file_name_size;
+                for (i = 0; i < size; i++)
+                {
+                    if (isprint((int)context->file_name[i]))
+                        cur[i] = (char)context->file_name[i];
+                    else
+                        cur[i] = '.';
+                }
 
-        if (!context->file_name[size-1])
-            size--;
+                if (!context->file_name[size-1])
+                    size--;
 
-        unused -= size;
-        cur += size;
+                unused -= size;
+                cur += size;
+            }
+        }
+        else
+        {
+            printf("<<Filename Encoding not supported>>");
+        }
     }
 
     if (unused > 0)
