@@ -3,7 +3,7 @@
 **
 **  fpdetect.c
 **
-**  Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+**  Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 **  Copyright (C) 2002-2013 Sourcefire, Inc.
 **  Author(s):  Dan Roelker <droelker@sourcefire.com>
 **              Marc Norton <mnorton@sourcefire.com>
@@ -247,6 +247,12 @@ int fpLogEvent(RuleTreeNode *rtn, OptTreeNode *otn, Packet *p)
             (rtn->type == RULE_TYPE__REJECT))
         {
             Active_DropSession(p);
+            if (pkt_trace_enabled)
+            {
+                addPktTraceData(VERDICT_REASON_SNORT, snprintf(trace_line, MAX_TRACE_LINE,
+                    "Snort fpdetect: gid %u, sid %u, %s\n", otn->sigInfo.generator, otn->sigInfo.id, getPktTraceActMsg()));
+            }
+            else addPktTraceData(VERDICT_REASON_SNORT, 0);
         }
         fpLogOther(p, otn, rtn->type);
         return 1;
@@ -304,6 +310,12 @@ int fpLogEvent(RuleTreeNode *rtn, OptTreeNode *otn, Packet *p)
              (action == RULE_TYPE__REJECT) )
         {
             Active_DropSession(p);
+            if (pkt_trace_enabled)
+            {
+                addPktTraceData(VERDICT_REASON_SNORT, snprintf(trace_line, MAX_TRACE_LINE,
+                    "Snort fpdetect_filter: gid %u, sid %u, %s\n", otn->sigInfo.generator, otn->sigInfo.id, getPktTraceActMsg()));
+            }
+            else addPktTraceData(VERDICT_REASON_SNORT, 0);
         }
         pc.event_limit++;
         fpLogOther(p, otn, action);
@@ -1107,7 +1119,7 @@ static inline int fpEvalHeaderSW(PORT_GROUP *port_group, Packet *p,
         tmp_ip4h = (void *)p->ip4h;
         tmp_payload = p->data;
         tmp_dsize = p->dsize;
-	 tmp_api = p->iph_api;
+        tmp_api = p->iph_api;
 
         /* Set the packet payload pointers to that of IP,
          ** since this is an IP rule. */
@@ -1119,7 +1131,7 @@ static inline int fpEvalHeaderSW(PORT_GROUP *port_group, Packet *p,
             p->ip4h = &p->outer_ip4h;
             p->data = p->outer_ip_data;
             p->dsize = p->outer_ip_dsize;
-	     p->iph_api = p->outer_iph_api;
+            p->iph_api = p->outer_iph_api;
             p->packet_flags |= PKT_IP_RULE;
             repeat = 2;
         }
@@ -1381,7 +1393,7 @@ static inline int fpEvalHeaderSW(PORT_GROUP *port_group, Packet *p,
             p->ip4h = &p->inner_ip4h;
             p->data = p->ip_data;
             p->dsize = p->ip_dsize;
-	     p->iph_api = tmp_api;
+            p->iph_api = tmp_api;
             p->packet_flags |= PKT_IP_RULE_2ND | PKT_IP_RULE;
             repeat--;
         }
@@ -1402,7 +1414,7 @@ fp_eval_header_sw_reset_ip:
         p->ip4h = tmp_ip4h;
         p->data = tmp_payload;
         p->dsize = tmp_dsize;
-	 p->iph_api = tmp_api;
+        p->iph_api = tmp_api;
         p->packet_flags &= ~(PKT_IP_RULE| PKT_IP_RULE_2ND);
     }
 
