@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2002-2013 Sourcefire, Inc.
 ** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
 ** Copyright (C) 2000-2001 Andrew R. Baker <andrewb@uab.edu>
@@ -100,7 +100,6 @@
 #define CONFIG_OPT__DETECTION                       "detection"
 #define CONFIG_OPT__DETECTION_FILTER                "detection_filter"
 #define CONFIG_OPT__PROTECTED_CONTENT               "protected_content"
-#define CONFIG_OPT__FTPDATA_TRIM_THRESHOLD          "ftp_data_trim_threshold"
 #ifdef INLINE_FAILOPEN
 # define CONFIG_OPT__DISABLE_INLINE_FAILOPEN         "disable_inline_init_failopen"
 #endif
@@ -368,7 +367,6 @@ void ConfigIpsPolicyMode(SnortConfig *, char *);
 void ConfigNapPolicyMode(SnortConfig *, char *);
 void ConfigPolicyVersion(SnortConfig *, char *);
 void ConfigProtectedContent(SnortConfig *, char *);
-void ConfigFTPDataTrimThreshold(SnortConfig *, char *);
 #ifdef PPM_MGR
 void ConfigPPM(SnortConfig *, char *);
 #endif
