@@ -2,7 +2,7 @@
 /*
 ** file_decomp.c
 **
-** Copyright (C) 2014-2016 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2017 Cisco and/or its affiliates. All rights reserved.
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -281,7 +281,7 @@ fd_status_t File_Decomp_CleanExit()
 /* The caller provides the memory size in the config struct. */
 fd_status_t File_Decomp_Config( fd_config_p_t ConfigPtr )
 {
-    PoolCount Max_Sessions;
+    unsigned Max_Sessions;
 
     if( ConfigPtr == NULL )
         return( File_Decomp_Error );
@@ -387,7 +387,7 @@ fd_status_t File_Decomp_SetBuf( fd_session_p_t SessionPtr )
 }
 
 /* Returns a new session object from the MemPool */
-fd_session_p_t File_Decomp_New()
+fd_session_p_t File_Decomp_New(void* scbPtr)
 {
     fd_session_p_t New_Session;
     MemBucket *bkt;
@@ -405,6 +405,7 @@ fd_session_p_t File_Decomp_New()
     }
     else
     {
+        bkt->scbPtr = scbPtr;
         New_Session = bkt->data;
         New_Session->bkt = bkt;
         New_Session->State = STATE_NEW;
